! ===========================================================================================================
! extract_traj.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20181109 JN     Eliminate extraneous unfiltered debug statements (AAAA, DDDD, etc)
! 20180920 CA     i1sp,i2sp,i3sp reset to zero between parsing of drive parameter lines
! 20180620 CA     i1sp,i2sp,i3sp are now initialized to '0' in the beginning of the
!                 loop. Previously the indices are reset to 0 at the end of the
!                 loop
! 20180417 VH     General WARNING revisions.
! 20171023 JN     See README.20171023_since_20171002 for traj changes
! 20171023 JN     Resett measurement_param before strarting new cycle
! 20170928 JN     Many changes to accomodate requests from Megan in July 2017
! 20160603 CS&JN  Modified "extract_traj.f90" to include a seperate subroutine "match_header_table"
! 20170228 JN     Fix to synchronize values between N_CYCLE arrays and N_MEASUREMENTS arrays.
!                 Improvements/Fixes with reference to the issues reported by FileChecker at GDAC.
! 20160511 all    Initial release of qcplus v1.1.0
!
!==================================================================================================
! Notes
!
! Function to extract traj parameters for netcdf files from qc file header
!
!==================================================================================================
!
! 1 = JULD
! 2 = LAT
! 3 = LON
! 4 = PRES
! 5 = TEMP
! 6 = PSAL
! 7 = SAT_NAME
! 8 = POS_ACY
! 9 = MCCODE
! 10= TIME DESCENT START STATUS
! 11= TIME DESCENT END STATUS
! 12= TIME DRIFT START STATUS
! 13= TIME DRIFT END STATUS
! 14= TIME ASCENT START STATUS
! 15= TIME ASCENT END STATUS
integer function extract_traj(jcyc)

use communication                       ! module
use data_arrays                         ! module
use nctraj_arrays                       ! module

implicit none

!--------------------------------- Variable definitions -----------------------------------------
character(len=14), parameter :: FUNCT_NAME = 'extract_traj';
integer :: iT, jcyc;
integer :: kyr,kmo,kdy,khh,kmn,ksc 
integer :: idate(6)
equivalence (idate(1),kyr), (idate(2),kmo),(idate(3),kdy),(idate(4),khh),(idate(5),kmn),(idate(6),ksc)
character(len=12), parameter :: sub = 'EXTRACT_TRAJ'
logical :: new_pqc			        ! To reset n_measurement value , to identify processing of a new qc file,gooddate
logical :: index_made_zero                      ! To reset n_measurement value , to identify processing of a new qc file,gooddate
character(len=1) :: result_juld_status          ! for the lines in qc files with up to 3 values
character(len=30) :: interim_str
integer :: mc_code,mc_code_prev 
extract_traj=SUCCESS  ! Assuming all traj variables extracted

! count_enter_traj = 3rd array dimension
! When count_enter_traj = 1 , it can read the content of the meta file and 
! profile "zero"

if(DEBUG)then;write (msg, *) " Extracting traj in cycle -", jcyc;  call wrtdbg(sub,msg);endif

if(count_enter_traj .eq. 1)then
   if(DEBUG)then;write (msg, *) "DBG:>>> Count enter traj is 1 , jcyc = ", jcyc;  call wrtdbg(sub,msg);endif
!Get launch time and position  from meta file
! 1 = JULD
! 2 = LAT
! 3 = LON
   !--------------------------  Filling traj array with launch values from meta file --------------
   kyr = launch_year
   kmo = launch_mon
   kdy = launch_day
   khh = launch_hh
   kmn = launch_mm
   ksc = launch_sc
   if (kyr .ne. 9999) double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=juldays1950(idate);
   call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement) !use the JULD_QC from read_pqc
   if (double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) .lt. 0)then
       double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement)=FILLDBLE; JULD_QC_TRAJ(n_measurement)='9';endif
   int_trajectory_parameter_measurement(MCCODE,n_measurement)=000;
! To prevent getting double values of very long positions after decimal
   write(interim_str,*)launch_lat
   read(interim_str,*)double_trajectory_parameter_measurement(LAT_TRAJ,n_measurement)
   write(interim_str,*)launch_lon
   read(interim_str,*)double_trajectory_parameter_measurement(LONG_TRAJ,n_measurement)
   CYCLE_NUMBER_TRAJ(n_measurement)=-1; ! values from meta file 
   call fill_juld_status('MET',result_juld_status);
   JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
   call fill_posn_qc(sub,n_measurement,0)
   !-----------------------------------------------------------------------------------------------
   if(ppos > 0 )then
      call match_header_table(jcyc)
! insert position block
      !if(trans_type == 'IRIDIUM')then
      !    call fill_from_lat_lon_line_irid(n_measurement,jcyc);  ! For cycle 0 or the very first cycle
      !else
      !    call fill_from_lat_lon_line_argos(n_measurement,jcyc);  
      !endif
! insert MC 800 with everything fill values
      if(int_trajectory_parameter_measurement(MCCODE,n_measurement) /= 800)then	
          n_measurement=n_measurement +1
          int_trajectory_parameter_measurement(MCCODE,n_measurement)=800;
          CYCLE_NUMBER_INDEX(cyc_count)=jcyc;
          CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
      endif    
   else
!  If Cycle zero is missing , Fill Values need to be printed for the cyc 
      double_trajectory_parameter_measurement(LAT_TRAJ,n_measurement) = FILLPOSN
      double_trajectory_parameter_measurement(LONG_TRAJ,n_measurement) = FILLPOSN; 
   endif
   if(jcyc .eq. 0)then
         ! Fill Start of Transmission if it exists
         if(PREV_TRANS_START_FOUND .and. START_OF_TRANS(cyc_count) == FILLDBLE )then
             START_OF_TRANS(cyc_count)= prev_start_of_trans_juld
             call fill_juld_status('SOT',result_juld_status);
             JULD_TRANSMISSION_START_STATUS(cyc_count)=result_juld_status
         endif
         CYCLE_NUMBER_INDEX(cyc_count)=jcyc;
   endif
! In case of missing profile zero
   first_profile_no = jcyc
   return;
endif

if(jcyc .gt. 0)then
   new_pqc = .true.
   !Filling QC file-independent N_CYCLE variables here.
   CYCLE_NUMBER_INDEX(cyc_count)=jcyc;
   call match_header_table(jcyc);
   if(START_OF_TRANS(cyc_count)/=FILLDBLE .and. &
   JULD_TRANSMISSION_START_STATUS(cyc_count)==FILLCHAR) then !in case there is no SOT STATUS line in the qc file.
       call fill_juld_status('SOT',result_juld_status);
       JULD_TRANSMISSION_START_STATUS(cyc_count)=result_juld_status;
   endif
endif  ! jcyc .gt.  0 

go to 999; ! Skip to THE_END
960 continue
    extract_traj=FAILURE
    write(errmsg(1),'("*** NON-FATAL ERROR 291 internal read *******")');
    write(errmsg(2),'("*** NON-FATAL ERROR 291 found premature end")')
    call error_report(sub,291,2,errmsg); ! extract_traj found premature end
    go to 999; ! Skip to THE_END
961 continue
    extract_traj=FAILURE
    write(errmsg(1),'("*** NON-FATAL ERROR 292 internal read *******")');
    write(errmsg(2),'("*** NON-FATAL ERROR 292 error occurred (e.g. format inconsistent)")')
    call error_report(sub,292,2,errmsg); ! extract_traj inconsistent format

999 continue ! This is THE_END

return;

end

!===========================================================================================================

subroutine match_header_table(jcyc)

use communication                       ! module
use data_arrays                         ! module
use nctraj_arrays			! module

implicit none

integer :: iH, iT, iTS, i_look, i_found, jcyc,count_posn_blk = 0;
character(len=SZC) :: str_pqc_tmp1;
character(len=40) :: str_pqc_tmp2,str_pqc_tmp2_prev;
character(len=128) :: str_pqc_tmp3;
character(len=5) :: str_pqc_tmp4;
character(len=128) :: str_nc_tmp;
integer :: i1o, i1c, i2o, i2c, i3o, i3c         ! for TECH_TABLE lines with "P,T,S" (-> 3 names & 3 values)
integer :: i1s, i2s, i1p, i2p, i3p              ! for the lines in qc files with up to 3 values
integer :: i1sp, i2sp, i3sp ,i4sp  ! for the lines in qc files with up to 3 values
integer :: measurement_param                    ! variable for p,t,s measurements
integer :: param_type,param_type_prev              ! Pressure = 4 , Temperature = 5 , PSaL = 6
integer :: mc_code,mc_code_prev
character(len=11) :: variable_type,dim_type
character(len=10) :: val_tmp1,val_tmp2,val_tmp3 ! for the lines in qc files with up to 3 values
integer :: kyr,kmo,kdy,khh,kmn,ksc 
integer :: idate(6)
integer :: count_trajectory_params,j
equivalence (idate(1),kyr), (idate(2),kmo),(idate(3),kdy),(idate(4),khh),(idate(5),kmn),(idate(6),ksc)
character(len=12), parameter :: sub = 'EXTRACT_TRAJ'
logical :: new_pqc, isnumber			        ! To reset n_measurement value , to identify processing of a new qc file,gooddate
logical :: index_made_zero                      ! To reset n_measurement value , to identify processing of a new qc file,gooddate
character(len=1) :: result_juld_status          ! for the lines in qc files with up to 3 values
character(len=30) :: interim_str
integer :: pt_tmp
first_msg_juld=FILLDBLE;  last_msg_juld=FILLDBLE; ! Resetting before strarting new cycle
measurement_param = 0 ; ! Resetting before strarting new cycle

  do iT=1,cnt_traj_table_param     
      str_nc_tmp=nc_traj_params_lst(iT);
      !Get the Param Type from the table
      read(str_nc_tmp(1:4),*)param_type;
      read(str_nc_tmp(5:8),*)mc_code;
      read(str_nc_tmp(21:35),*)variable_type;
      read(str_nc_tmp(36:),*)dim_type;
      !if (mc_code .eq. 800 .and. count_enter_traj .eq. 1) CYCLE; ! for first run MC 800 is added later.
      
      if(trans_type == 'IRIDIUM' )then
          if((mc_code .eq. 700 .or. mc_code .eq. 800) .and. param_type <= 1)then 
             !calling fill_from_lat_lon_line to add the first position  for iridium.
              call fill_from_lat_lon_line_irid(n_measurement,jcyc);
          endif
      else
      	  !calling fill_from_lat_lon_line to add position data in the default order - 700,702,703,704,800.... for ARGOS type.
          if(mc_code .eq. 800 .and. int_trajectory_parameter_measurement(MCCODE,n_measurement) .eq. 700 )then  
             !if(count_enter_traj /= 1)then
                 call fill_from_lat_lon_line_argos(n_measurement,jcyc);
             !endif
          endif     
      endif
       
      if (jcyc .eq. 0) then  ! No need of these MCs in for these types in cycle 0
         if((inst_type(1:5) .eq. 'APEX_' .or. inst_type(1:7) .eq. 'APEXIR_') .and. mc_code .eq. 290) CYCLE;
         if((inst_type(1:7) .eq. 'APEXIR_' .or. inst_type(1:8) .eq. 'NAVISIR_') .and. mc_code .eq. 299) CYCLE;
         if((inst_type(1:5) .eq. 'APEX_' .or. inst_type(1:5) .eq. 'SOLO_') .and. mc_code .eq. 250) CYCLE;
         if((inst_type(1:5) .eq. 'APEX_' .or. inst_type(1:5) .eq. 'SOLO_') .and. mc_code .eq. 296) CYCLE;
         if((inst_type(1:5) .eq. 'APEX_' .or. inst_type(1:5) .eq. 'SOLO_') .and. mc_code .eq. 300) CYCLE;
      endif
      if(trim(qc_traj_params_lst(iT)) .eq. "NOT AVAILABLE")then ! No need of these MCs in for these types in cycle 0 if they are "NOT AVAILABLE"
            if ((mc_code .eq. 100 .or. mc_code .eq. 200 .or. mc_code .eq. 250 .or. mc_code .eq. 300 .or. &
            mc_code .eq. 400 .or. mc_code .eq. 500 .or. mc_code .eq. 600 .or. &
            mc_code .eq. 800) .and. jcyc .eq. 0) CYCLE;
            n_measurement = n_measurement + 1
            CYCLE_NUMBER_TRAJ(n_measurement)=jcyc; 
            int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
            mc_code_prev = mc_code
            CYCLE;
      endif

! XXXXX introduce a variable that indicates if a line was found in the header that matches the line from the traj table
! set ti to .false. here and make it .true. in iH loop if match found (but, we may not need this at all because of
! the logic applied after the end of the iH loop XXXXX
      do iH=1,npqc
         hasTIME_reg=.false. ;hasSTATUS= .false.; index_made_zero = .false. ; isnumber =.false.; ! Resetting logical for every qc file line
         str_pqc_tmp1=pqcline(iH); str_pqc_tmp2=str_pqc_tmp1(1:40);
         str_pqc_tmp3=trim(str_pqc_tmp1(41:)); str_pqc_tmp4=str_pqc_tmp1(1:5);
         call to_upper(str_pqc_tmp2); ! In case the qc file has the header line(1:40) in lower-case.
         if (trim(str_pqc_tmp2).eq.'CONFIG_MISSION_NUMBER') then
            if(trim(str_pqc_tmp1(41:)) /= '-999' .and. CONFIG_MISSION_NUMBER_TRAJ(cyc_count) == FILLINT)then
              read(str_pqc_tmp1(41:),*)CONFIG_MISSION_NUMBER_TRAJ(cyc_count);
              if (CONFIG_MISSION_NUMBER_TRAJ(cyc_count) .eq. 0) CONFIG_MISSION_NUMBER_TRAJ(cyc_count)=FILLINT;
            endif
         elseif (trim(str_pqc_tmp2).eq.trim(qc_traj_params_lst(iT))) then;
            if (variable_type == "PRES" .and. .not. hasPRES_traj)then
              nparam_traj=nparam_traj+1
              hasPRES_traj=.true. ; hasNCPAR_traj(1) = .true.; hasQCPAR_traj(1) = .true.; 
            elseif (variable_type == "TEMP".and. .not. hasTEMP_traj)then
              nparam_traj=nparam_traj+1
              hasTEMP_traj=.true. ; hasNCPAR_traj(2) = .true.; hasQCPAR_traj(2) = .true.;
            elseif (variable_type == "PSAL".and. .not. hasPSAL_traj)then
              nparam_traj=nparam_traj+1
              hasPSAL_traj=.true. ; hasNCPAR_traj(4) = .true.; hasQCPAR_traj(4) = .true.;
            ! Notes from Claudia - "Traj" files don't have "CNDC" now, but it can have in future it has to be before "PSAL" so index 3.
            ! In case we need to add "Conductivity" , add hasCNDC_traj=.true. ; hasNCPAR_traj(3) = .true.; hasQCPAR_traj(3) = .true.; 
            ! elseif (variable_type == "CNDC".and. .not. hasCNDC_traj)then ! Add this later when "CNDC" needed in "traj" files.
            elseif(variable_type == "JULD")then
               hasTIME_reg=.true.
            elseif(variable_type == "JULD_STATUS")then
               hasSTATUS = .true.
	    endif
	    
	    if(hasTIME_reg)then ! check if the date and time has a proper numeric format.
	       isnumber = is_all_numeric(str_pqc_tmp3);
	       if(.not. isnumber)then
		    write(wrnmsg,'("WARNING 164 Date/time NOT parsed from ",a," for cyc ",i4)')trim(str_pqc_tmp3),jcyc; ! WRN
		    call warning(sub,164,wrnmsg); ! WARNING 164 in extract_traj ! WRN
                   n_measurement = n_measurement + 1 
                   CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
                   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
                   double_trajectory_parameter_measurement(param_type,n_measurement)=FILLDBLE;
                   kyr=9999;
                   call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement);
                   JULD_STATUS_TRAJ(n_measurement)='9';
                   go to 100;! 100 marks the end of the "do iH=1,npqc" loop
	       endif
	    endif !end of "if(hasTIME_reg)"
	    
            i1sp=0;                        !Needs to be reset to encounter the next DRIFT PRESSURE
            i2sp=0;                        !Needs to be reset to encounter the next DRIFT PRESSURE
            i3sp=0;                        !Needs to be reset to encounter the next DRIFT PRESSURE
   	    if(index(ADJUSTL(trim(str_pqc_tmp3)),' ')>0)then  ! calulating how many data values in the line based on space counts.
               call compact(str_pqc_tmp3);
    	       i1sp=index(ADJUSTL(trim(str_pqc_tmp3)),' ');
	       if(i1sp>0) i2sp=index(ADJUSTL(trim(str_pqc_tmp3(i1sp+1:))),' ');
	       if(i2sp>0) i3sp=index(ADJUSTL(trim(str_pqc_tmp3(i1sp+i2sp+1:))),' ');
    	    endif
 
            if(trim(str_pqc_tmp2_prev) .ne. trim(str_pqc_tmp2) )then  ! if similar lines appear in the file with more measurements
	        ! If its not a new file and the previous parameter no equal to the current incoming
	        ! reset the n_measurement to previous value
                if( .not. new_pqc )then
	           if(mc_code_prev == mc_code)then
		       n_measurement = n_measurement - measurement_param;
                   endif
                endif
                new_pqc = .false.
		measurement_param = 0; !reset measurement_param
            elseif(str_pqc_tmp2_prev .eq. str_pqc_tmp2 .and. mc_code_prev .ne. mc_code)then
		measurement_param = 0; !reset measurement_param
	    endif
		  
	    ! Note: To handle the following kinda lines.
	    ! >0   >>>> DRIFT PRESSURE (DBAR)                   1102.07 1106.87 1103.58 1099.01
	    ! = 0  >>>> DRIFT PRESSURE (DBAR)                   1102.07
            if(i1sp >= 0)then
    	       measurement_param = measurement_param + 1;
	       n_measurement = n_measurement + 1 
   	       CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
   	       int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
               if(hasTIME_reg)then ! To prosess all the qc line which have a "time" value.
	          if(i1sp /= 0)then
		      if(trim(str_pqc_tmp2) .eq. "START OF TRANSMISSION (YYYYMMDDHHMMSS)") then 
			     read(str_pqc_tmp3,'(i4,i2,i2,i2,i2,i2)')kyr,kmo,kdy,khh,kmn,ksc
			     i2sp = 0;i3sp = 0 
			     index_made_zero = .true.
		      elseif (trim(str_pqc_tmp2) .eq. "START OF TRANSMISSION" )then
			     read(str_pqc_tmp3,'(i4,x,i2,x,i2,x,i2,x,i2,x,i2)')kyr,kmo,kdy,khh,kmn,ksc ! NOT sure about adding the ",x,"
			  ! To handle line "START OF TRANSMISSION                   2015 04 19 09 57 28"
			     i2sp = 0;i3sp = 0 
			     index_made_zero = .true.
		      elseif(trim(str_pqc_tmp2) .eq. "DOWN TIME EXPIRED EPOCH (GMT)")then
			  read(str_pqc_tmp3,*)kyr,kmo,kdy,khh,kmn,ksc 
			  i2sp = 0;i3sp = 0 
			  index_made_zero = .true.	  
		      else
			 read(str_pqc_tmp3(1:i1sp),'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc
		      endif
	          else
	              read(str_pqc_tmp3,'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc
	          endif ! end of "if(i1sp /= 0)then"
	         
	          if(kyr .ne. 9999)then;
	             double_trajectory_parameter_measurement(param_type,n_measurement)=juldays1950(idate);
              else;
                 double_trajectory_parameter_measurement(param_type,n_measurement)=FILLDBLE;
              endif;  
	          if(mc_code == 700)then;call fill_juld_status('SOT',result_juld_status); 
	          else;call fill_juld_status('OTH',result_juld_status);endif
                  JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
	          call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement)
	         
	          if(trim(str_pqc_tmp2) .eq. "TIME DESCENT START (YYYYMMDDHHMMSS)")then
	             JULD_DESCENT_START(cyc_count) = double_trajectory_parameter_measurement(param_type,n_measurement)
	          elseif(trim(str_pqc_tmp2) .eq. "TIME DESCENT END (YYYYMMDDHHMMSS)")then
	             JULD_DESCENT_END(cyc_count) = double_trajectory_parameter_measurement(param_type,n_measurement);
	          elseif(trim(str_pqc_tmp2) .eq. "TIME ASCENT START (YYYYMMDDHHMMSS)")then
	             JULD_ASCENT_START(cyc_count)=double_trajectory_parameter_measurement(param_type,n_measurement);
	          elseif(trim(str_pqc_tmp2) .eq. "TIME ASCENT END (YYYYMMDDHHMMSS)")then
	             JULD_ASCENT_END(cyc_count)=double_trajectory_parameter_measurement(param_type,n_measurement)
	          elseif(trim(str_pqc_tmp2) .eq. "TIME DRIFT START (YYYYMMDDHHMMSS)")then
	             JULD_PARK_START(cyc_count)= double_trajectory_parameter_measurement(param_type,n_measurement)
	          elseif(trim(str_pqc_tmp2) .eq. "TIME DRIFT END (YYYYMMDDHHMMSS)")then
	             JULD_PARK_END(cyc_count)=double_trajectory_parameter_measurement(param_type,n_measurement)
	          elseif(trim(str_pqc_tmp2) .eq. "START OF TRANSMISSION" .or. &
	          trim(str_pqc_tmp2) .eq. "START OF TRANSMISSION (YYYYMMDDHHMMSS)")then
                     START_OF_TRANS(cyc_count)=double_trajectory_parameter_measurement(param_type,n_measurement)
                     JULD_TRANSMISSION_START_STATUS(cyc_count)=result_juld_status; ! in case "START OF TRANSMISSION STATUS" is not specifed. eg: 6483_150642_002.qc
	          endif
               else ! hasTIME_reg is .false. , i.e. it is not TIME
		  if(i1sp /= 0)then
                       if (trim(str_pqc_tmp2(1:5)) .eq. "P,T,S")then  ! "P,T,S BEFORE ASCENT"
                          read(str_pqc_tmp3(1:i1sp),*)double_trajectory_parameter_measurement(param_type,n_measurement);
                          int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
                          call fill_param_qc(param_type,n_measurement);
                          read(str_pqc_tmp3(i1sp+1:i1sp+i2sp),*)double_trajectory_parameter_measurement(param_type+1,n_measurement);
                          pt_tmp=param_type+1; call fill_param_qc(pt_tmp,n_measurement);
                          read(str_pqc_tmp3(i1sp+i2sp+1:),*) & 
		          double_trajectory_parameter_measurement(param_type+2,n_measurement);
           		  pt_tmp=param_type+2; call fill_param_qc(pt_tmp,n_measurement);
		          i2sp=0;
                          i3sp=0;
                          index_made_zero = .true.
		       else
                          ! Check for the string , if it contains STATUS.. then save it in a char array
                          if(hasSTATUS)then
		             JULD_STATUS_TRAJ(n_measurement)=str_pqc_tmp3(1:i1sp);
                          else 
                             read(str_pqc_tmp3(1:i1sp),*)double_trajectory_parameter_measurement(param_type,n_measurement)
                             call fill_param_qc(param_type,n_measurement);
                          endif
		       endif
		  else ! of  "if(i1sp /= 0)"
                          ! Check for the string , if it contains STATUS.. then save it in a char array
                          if(hasSTATUS)then
		             if(double_trajectory_parameter_measurement(JULD_TRAJ,n_measurement) /= FILLDBLE)then
		             	 JULD_STATUS_TRAJ(n_measurement)=trim(str_pqc_tmp3);
		             endif
		             if(trim(str_pqc_tmp2) .eq. "TIME DESCENT START STATUS")then
		                 JULD_DESCENT_START_STATUS(cyc_count) = JULD_STATUS_TRAJ(n_measurement)
		             elseif(trim(str_pqc_tmp2) .eq. "TIME DESCENT END STATUS")then
		                 JULD_DESCENT_END_STATUS(cyc_count) = JULD_STATUS_TRAJ(n_measurement)
	      	             elseif(trim(str_pqc_tmp2) .eq. "TIME ASCENT START STATUS")then
		                 JULD_ASCENT_START_STATUS(cyc_count) = JULD_STATUS_TRAJ(n_measurement)
		             elseif(trim(str_pqc_tmp2) .eq. "TIME ASCENT END STATUS")then
		                 JULD_ASCENT_END_STATUS(cyc_count)=JULD_STATUS_TRAJ(n_measurement)
		             elseif(trim(str_pqc_tmp2) .eq. "TIME DRIFT START STATUS")then
		                 JULD_PARK_START_STATUS(cyc_count)=JULD_STATUS_TRAJ(n_measurement)
		             elseif(trim(str_pqc_tmp2) .eq. "TIME DRIFT END STATUS")then
		                 JULD_PARK_END_STATUS(cyc_count)=JULD_STATUS_TRAJ(n_measurement)
		             elseif(trim(str_pqc_tmp2) .eq. "START OF TRANSMISSION STATUS")then  
		                 JULD_TRANSMISSION_START_STATUS(cyc_count)=JULD_STATUS_TRAJ(n_measurement)
		             endif
                          else
			     read(str_pqc_tmp3,*)double_trajectory_parameter_measurement(param_type,n_measurement)
			     call fill_param_qc(param_type,n_measurement);
                          endif
           		  int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
		   endif ! end of "if(i1sp /= 0)"
                endif ! end of "if(i1sp >= 0)"
                
                if((i2sp > 0 .or. (i2sp .eq. 0 .and. i1sp > 0)) .and. .not. index_made_zero)then
                   measurement_param = measurement_param + 1;
                   n_measurement = n_measurement + 1
   		   CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
   		   int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
                   if(hasTIME_reg)then
		       if(i2sp > 0)then
		           read(str_pqc_tmp3(i1sp+1:i1sp+i2sp),'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc
		       else 	
		           read(str_pqc_tmp3(i1sp+1:),'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc
		       endif
			if (kyr .ne. 9999) then
     			double_trajectory_parameter_measurement(param_type,n_measurement)=juldays1950(idate);
                        endif

                        if (double_trajectory_parameter_measurement(param_type,n_measurement) .lt. 0) &
                            double_trajectory_parameter_measurement(param_type,n_measurement)=FILLDBLE;
           	       int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
                       call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement) 
		       if (kyr .ne. 9999)then
		           call fill_juld_status('OTH',result_juld_status)
   		           JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
   		       endif    
		   else
		      if(i2sp > 0)then
                          if(hasStatus)then
			       JULD_STATUS_TRAJ(n_measurement)=trim(str_pqc_tmp3(i1sp:i1sp+i2sp));
                          else
			       read(str_pqc_tmp3(i1sp:i1sp+i2sp),*)double_trajectory_parameter_measurement(param_type,n_measurement)
			       call fill_param_qc(param_type,n_measurement);
                          endif
		      else 	
                          if(hasStatus)then
			       JULD_STATUS_TRAJ(n_measurement)=trim(str_pqc_tmp3(i1sp:));
                           else
				read(str_pqc_tmp3(i1sp:),*)double_trajectory_parameter_measurement(param_type,n_measurement);
				call fill_param_qc(param_type,n_measurement);
                           endif
		      endif
           	       int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
		   endif
                   if(i3sp > 0 .or. (i3sp .eq. 0 .and. i2sp > 0) .and. (.not. index_made_zero))then
    	              measurement_param = measurement_param + 1
		      n_measurement = n_measurement + 1 
                      int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
   		      CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
                      if(hasTIME_reg)then
		         if(i3sp > 0)then
		            read(str_pqc_tmp3(i1sp+i2sp+1:i1sp+i2sp+i3sp),'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc
			  else
		            read(str_pqc_tmp3(i1sp+i2sp+1:),'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc
		         endif
			 if (kyr .ne. 9999) then
			 	 double_trajectory_parameter_measurement(param_type,n_measurement)=juldays1950(idate);
			 endif 	 
                         if (double_trajectory_parameter_measurement(param_type,n_measurement) .lt. 0) &
                             double_trajectory_parameter_measurement(param_type,n_measurement)=FILLDBLE;
                         call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement) 
		         if (kyr .ne. 9999)then
                             call fill_juld_status('OTH',result_juld_status)
   			     JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
   			 endif    
           		 int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
		      else
		         if(i3sp > 0)then
                             if(hasStatus)then
			         JULD_STATUS_TRAJ(n_measurement)=trim(str_pqc_tmp3(i1sp+i2sp:i1sp+i2sp+i3sp));
                              else
		                read(str_pqc_tmp3(i1sp+i2sp:i1sp+i2sp+i3sp),*)&
				    double_trajectory_parameter_measurement(param_type,n_measurement)
				    call fill_param_qc(param_type,n_measurement);
			      endif
                         else
                             if(hasStatus)then
		                JULD_STATUS_TRAJ(n_measurement)=trim(str_pqc_tmp3(i1sp+i2sp:));
			     else
		                read(str_pqc_tmp3(i1sp+i2sp:),*)double_trajectory_parameter_measurement(param_type,n_measurement)
		                call fill_param_qc(param_type,n_measurement);
			     endif
			 endif
           		 int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
		      endif
    	              
		      measurement_param = measurement_param + 1
		      n_measurement = n_measurement + 1 
                      int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
   		      CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
		      if(hasTIME_reg)then
		         read(str_pqc_tmp3(i1sp+i2sp+i3sp+1:),'(i4,5i2)')kyr,kmo,kdy,khh,kmn,ksc
		         if (kyr .ne. 9999)then
		         	 double_trajectory_parameter_measurement(param_type,n_measurement)=juldays1950(idate);
                         endif
                         if (double_trajectory_parameter_measurement(param_type,n_measurement) .lt. 0) &
                             double_trajectory_parameter_measurement(param_type,n_measurement)=FILLDBLE;
                         call fill_juld_qc(kyr,kmo,kdy,khh,kmn,ksc,n_measurement) 
		         if (kyr .ne. 9999)then
                             call fill_juld_status('OTH',result_juld_status)
   			     JULD_STATUS_TRAJ(n_measurement)=result_juld_status;
           		 endif
   			 int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
		      else
                          if(hasStatus)then
		             JULD_STATUS_TRAJ(n_measurement)=trim(str_pqc_tmp3(i1sp+i2sp+i3sp+1:));
			  else
		             read(str_pqc_tmp3(i1sp+i2sp+i3sp+1:),*)double_trajectory_parameter_measurement(param_type,n_measurement);
		             call fill_param_qc(param_type,n_measurement);
			  endif
           		 int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code;
		      endif
    	           endif
    	        endif
    	    endif
    	    100 continue
		  !param_type_prev = param_type
                  str_pqc_tmp2_prev = str_pqc_tmp2
                  mc_code_prev = mc_code
    	 endif
      enddo  ! do iH=1,npqc
! here we need to check if int_trajectory_parameter_measurement(MCCODE,n_measurement) has the same valuen as mc_code
! but only if mc_code is either 100, 200, 300, 400, 500, 600, 700, 800
! if the mc_code is nt found at n_measurement index, then it is added (increment n_measurement by adding 1)
!
! cycle 0 no profile no drift phase    (APEX, APEXIR) - launch, 700, ...
! cycle 0 with profile no drift phase  (SOLO_TS12); other SOLO_ have no cycle 0 (I think) - launch, descent, ascent, 700
! cycle 0 with profile and drift phase (SOLO_TS16) - launch, descent, drift, ascent, 700 (no need to enforce adding of any MC codes)
! The following is good for cycles >0:
	   
	   if(jcyc > 0)then
	      if (int_trajectory_parameter_measurement(MCCODE,n_measurement) .ne. mc_code .and. (mc_code .eq. 100 &
	         .or. mc_code .eq. 200 .or. mc_code .eq. 300 .or. mc_code .eq. 400 .or. mc_code .eq. 500 .or. mc_code .eq. 600  &
 	         .or. mc_code .eq. 700 .or. mc_code .eq. 800 .or. mc_code .eq. 250) )then
	          n_measurement = n_measurement + 1
	          int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
	          CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
	      endif
           else
	      if(int_trajectory_parameter_measurement(MCCODE,n_measurement) .ne. mc_code .and. (mc_code .eq. 700 &
	         .or. mc_code .eq. 800) .and. trim(qc_traj_params_lst(iT)) .ne. "NOT AVAILABLE" )then
	          n_measurement = n_measurement + 1
	          int_trajectory_parameter_measurement(MCCODE,n_measurement)=mc_code
	          CYCLE_NUMBER_TRAJ(n_measurement)=jcyc;
	      endif
	   endif
   enddo ! do iT=1,cnt_traj_table_param
end subroutine match_header_table
! ===========================================================================================================
