! ===========================================================================================================
! module ncmeta_arrays.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20190708 JN     Read the correct line from ascii meta file for sensor type - MCOMS_FLBB2 
! 20190117 JN     Modify duration_to_convert to 3 decimal places, help diffing files (installed 20190416)
! 20180920 JN&VH  Convert launch_lat,launch_lon from real to double, create read-launch... for calls to
!                 impossible_speed and land_or_water.  Initialize more variables (not all yet)
! 20180417 VH     General WARNING revisions.
! 20180307 JN     small change to increase size of sensor_param variable line from 120 to 160
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes:

! This module contains variable/array definitions used by write_ncmeta to create the nc metadata file.
! ===========================================================================================================

module ncmeta_arrays

use communication                                                ! module
use data_arrays                                                  ! module
use netcdf_arrays                                                ! module

implicit none

! ===========================================================================================================
! moedule varialbe/array declarations
! ===========================================================================================================

integer :: PTT_id,                        PTT_dims;                           character(len=256)  :: PTT;
integer :: TRANS_SYSTEM_id,               TRANS_SYSTEM_dims(ND2);             character(len=16)   :: TRANS_SYSTEM(ND1);
integer :: TRANS_SYSTEM_IDX_id,           TRANS_SYSTEM_IDX_dims(ND2);         character(len=32)   :: TRANS_SYSTEM_IDX(ND1);       ! Appended X with the name since "TRANS_SYSTEM_ID" is already existing
integer :: TRANS_FREQUENCY_id,            TRANS_FREQUENCY_dims(ND2);          character(len=16)   :: TRANS_FREQUENCY(ND1);
integer :: POSITIONING_SYSTEM_id,         POSITIONING_SYSTEM_dims(2);         character(len=8)    :: POSITIONING_SYSTEM(ND2);
integer, parameter :: MAX_NC_CONFIG_PARAMS_COUNT = 500! Max. # of params in table
character(len=128), dimension(MAX_NC_CONFIG_PARAMS_COUNT) :: launch_configuration_parameter_name
double precision, dimension(MAX_NC_CONFIG_PARAMS_COUNT) :: launch_configuration_parameter_value=FILLNC
character(len=128), dimension(MAX_NC_CONFIG_PARAMS_COUNT) :: configuration_parameter_name

character(len=128), dimension(MAX_NC_CONFIG_PARAMS_COUNT,2) :: combined_launch_config_param_name   ! Added by JN
double precision,   dimension(MAX_NC_CONFIG_PARAMS_COUNT)   :: combined_launch_config_param_value=FILLNC   ! Added by JN

double precision, dimension(MAXCYC,MAX_NC_CONFIG_PARAMS_COUNT) :: configuration_parameter_value=FILLNC
character(len=256),  dimension(MAXCYC) :: config_mission_comments(MAXCYC) = ""

integer :: PLATFORM_FAMILY_id,            PLATFORM_FAMILY_dims;               character(len=256)  :: PLATFORM_FAMILY;
integer :: PLATFORM_TYPE_id,              PLATFORM_TYPE_dims;                 character(len=32)   :: PLATFORM_TYPE;
integer :: PLATFORM_MAKER_id,             PLATFORM_MAKER_dims;                character(len=256)  :: PLATFORM_MAKER;
integer :: FIRMWARE_VERSION_id,           FIRMWARE_VERSION_dims;              character(len=32)   :: FIRMWARE_VERSION;
integer :: MANUAL_VERSION_id,             MANUAL_VERSION_dims;                character(len=16)   :: MANUAL_VERSION;
integer :: FLOAT_SERIAL_NO_id,            FLOAT_SERIAL_NO_dims;               character(len=32)   :: FLOAT_SERIAL_NO;
integer :: STANDARD_FORMAT_ID_id,         STANDARD_FORMAT_ID_dims;            character(len=16)   :: STANDARD_FORMAT_ID;
integer :: DAC_FORMAT_ID_id,              DAC_FORMAT_ID_dims;                 character(len=16)   :: DAC_FORMAT_ID;
integer :: WMO_INST_TYPE_id,              WMO_INST_TYPE_dims;                 character(len=4)    :: WMO_INST_TYPE;
integer :: PROJECT_NAME_id,               PROJECT_NAME_dims;                  character(len=64)   :: PROJECT_NAME;
integer :: PI_NAME_id,                    PI_NAME_dims;                       character(len=64)   :: PI_NAME;
integer :: ANOMALY_id,                    ANOMALY_dims;                       character(len=256)  :: ANOMALY;
integer :: BATTERY_TYPE_id,               BATTERY_TYPE_dims;                  character(len=64)   :: BATTERY_TYPE;
integer :: BATTERY_PACKS_id,              BATTERY_PACKS_dims;                 character(len=64)   :: BATTERY_PACKS;
integer :: CONTROLLER_BOARD_TYPE_PRIMARY_id,         CONTROLLER_BOARD_TYPE_PRIMARY_dims;         
character(len=32)       :: CONTROLLER_BOARD_TYPE_PRIMARY;
integer :: CONTROLLER_BOARD_TYPE_SECONDARY_id,       CONTROLLER_BOARD_TYPE_SECONDARY_dims;       
character(len=32)       :: CONTROLLER_BOARD_TYPE_SECONDARY;
integer :: CONTROLLER_BOARD_SERIAL_NO_PRIMARY_id,    CONTROLLER_BOARD_SERIAL_NO_PRIMARY_dims;    
character(len=32)       :: CONTROLLER_BOARD_SERIAL_NO_PRIMARY;
integer :: CONTROLLER_BOARD_SERIAL_NO_SECONDARY_id,  CONTROLLER_BOARD_SERIAL_NO_SECONDARY_dims;  
character(len=32)       :: CONTROLLER_BOARD_SERIAL_NO_SECONDARY;
integer :: SPECIAL_FEATURES_id,  SPECIAL_FEATURES_dims;                       character(len=1024) :: SPECIAL_FEATURES;
integer :: FLOAT_OWNER_id,  FLOAT_OWNER_dims;                                 character(len=64)   :: FLOAT_OWNER;
integer :: OPERATING_INSTITUTION_id,  OPERATING_INSTITUTION_dims;             character(len=64)   :: OPERATING_INSTITUTION;
integer :: CUSTOMISATION_id,  CUSTOMISATION_dims;                             character(len=1024) :: CUSTOMISATION;
integer :: LAUNCH_DATE_id,  LAUNCH_DATE_dims;                                 !character(len=14)   :: LAUNCH_DATE;
integer :: LAUNCH_LATITUDE_id , LAUNCH_LATITUDE_dims;                         double precision :: LAUNCH_LATITUDE;
integer :: LAUNCH_LONGITUDE_id, LAUNCH_LONGITUDE_dims;                        double precision :: LAUNCH_LONGITUDE;
integer :: LAUNCH_QC_id, LAUNCH_QC_dims;                                      character :: LAUNCH_QC
integer :: START_DATE_id,  START_DATE_dims;                                   !character(len=14)   :: START_DATE;
integer :: START_DATE_QC_id, START_DATE_QC_dims;                              character :: START_DATE_QC
integer :: STARTUP_DATE_id,  STARTUP_DATE_dims;                               !character(len=14)   :: STARTUP_DATE;
integer :: STARTUP_DATE_QC_id, STARTUP_DATE_QC_dims;                          character :: STARTUP_DATE_QC
integer :: DEPLOYMENT_PLATFORM_id,  DEPLOYMENT_PLATFORM_dims;                 character(len=32)   :: DEPLOYMENT_PLATFORM;
integer :: DEPLOYMENT_CRUISE_ID_id,  DEPLOYMENT_CRUISE_ID_dims;               character(len=32)   :: DEPLOYMENT_CRUISE_ID;
integer :: DEPLOYMENT_REFERENCE_STATION_ID_id,  DEPLOYMENT_REFERENCE_STATION_ID_dims;
character(len=256)   :: DEPLOYMENT_REFERENCE_STATION_ID;
integer :: END_MISSION_DATE_id,END_MISSION_DATE_dims;                         character(len=14)    :: END_MISSSION_DATE;
integer :: END_MISSION_STATUS_id,END_MISSION_STATUS_dims;                     character(len=1)     :: END_MISSSION_STATUS;
integer :: SENSOR_id,                     SENSOR_dims(ND2);                   !character(len=32)   :: SENSOR(N_SENSOR_len);
integer :: SENSOR_MAKER_id,               SENSOR_MAKER_dims(ND2);             !character(len=256)  :: SENSOR_MAKER(N_SENSOR_len);
integer :: SENSOR_MODEL_id,               SENSOR_MODEL_dims(ND2);             !character(len=256)  :: SENSOR_MODEL(N_SENSOR_len);
integer :: SENSOR_SERIAL_NO_id,           SENSOR_SERIAL_NO_dims(ND2);         !character(len=16)   :: SENSOR_SERIAL_NO(N_SENSOR_len);
integer :: PARAMETERX_id,                 PARAMETERX_dims(ND2);               !character(len=64)   :: PARAMETERX;         ! Appended X with the name since "PARAMETER" is a keyword
integer :: PARAMETER_UNITS_id,            PARAMETER_UNITS_dims(ND2);          !character(len=32)   :: PARAMETER_UNITS;
integer :: PARAMETER_SENSOR_id,           PARAMETER_SENSOR_dims(ND2);         !character(len=128)  :: PARAMETER_SENSOR;
integer :: PARAMETER_ACCURACY_id,         PARAMETER_ACCURACY_dims(ND2);
integer :: PARAMETER_RESOLUTION_id,       PARAMETER_RESOLUTION_dims(ND2);
integer :: PREDEPLOYMENT_CALIB_EQUATION_id,     PREDEPLOYMENT_CALIB_EQUATION_dims(ND2);
integer :: PREDEPLOYMENT_CALIB_COEFFICIENT_id,  PREDEPLOYMENT_CALIB_COEFFICIENT_dims(ND2);
integer :: PREDEPLOYMENT_CALIB_COMMENT_id,      PREDEPLOYMENT_CALIB_COMMENT_dims(ND2);
integer :: LAUNCH_CONFIG_PARAMETER_NAME_id, LAUNCH_CONFIG_PARAMETER_NAME_dims(ND2);
integer :: LAUNCH_CONFIG_PARAMETER_VALUE_id, LAUNCH_CONFIG_PARAMETER_VALUE_dims(ND1)
integer :: CONFIG_PARAMETER_NAME_id, CONFIG_PARAMETER_NAME_dims(ND2);
integer :: CONFIG_PARAMETER_VALUE_id, CONFIG_PARAMETER_VALUE_dims(ND2)
integer :: CONFIG_MISSION_NUMBER_id, CONFIG_MISSION_NUMBER_dims(ND1)
integer :: CONFIG_MISSION_COMMENT_id, CONFIG_MISSION_COMMENT_dims(ND2)

logical :: has_ido_doxy_sensor =.false.
integer :: idx_row, idx_column
integer :: iret;
integer :: ret = 0;
integer :: loc
double precision :: config_param_value_vector(MAX_NC_CONFIG_PARAMS_COUNT)
character(len=14) :: ascii_meta_launch_date
character (len=14) :: start_date_metanc, ascii_meta_start_date,startup_date_metanc

integer, parameter :: META_SENSOR_ARRAY_LEN = 500;
character(len=64), dimension(META_SENSOR_ARRAY_LEN,3) :: sensor_model_table
character(len=160), dimension(MAX_PARM_TABLE) :: sensor_params_table;
integer :: cnt_config_table_param,cnt_config_table_skip_param,n_sensor_params_table;
character(len=40), dimension(MAX_PARM_TABLE) ::  qc_config_params_lst  !Config parameters
character(len=40), dimension(MAX_PARM_TABLE) ::  qc_config_skip_lst    !non-Config parameters
character(len=128), dimension(MAX_PARM_TABLE) :: nc_config_params_lst  !new NC Config parameters
integer, parameter :: MAXSENSOR = 30
!character(len=32)  :: all_sensors_per_float(MAXSENSOR) = "";
character(len=32)  :: ascii_sensor_type(MAXSENSOR) = "";	
character(len=32)  :: sensor_name_array(MAXSENSOR) = "";
character(len=256) :: sensor_model_array(MAXSENSOR) = "";
character(len=256) :: sensor_maker_array(MAXSENSOR) = "Unknown";
character(len=16)  :: sensor_ser_no_array(MAXSENSOR) = "Unknown";
character(len=256)  :: param_info_metline(MAXSENSOR) = "";
integer :: sensor_type_count, sensor_manufa_count, sensor_serno_count
character(len=64)    :: parameter_name_array(MAXPAR_META) = 'UNKNOWN',param_name;
character(len=128)   :: parameter_sensor_array(MAXPAR_META) =  'n/a',param_sensor;
character(len=32)    :: parameter_unit_array(MAXPAR_META) =  'n/a',param_unit;
character(len=32)    :: parameter_accuracy_array(MAXPAR_META) =  'n/a';
character(len=32)    :: parameter_resolution_array(MAXPAR_META) =  'n/a';
character(len=1024)  :: predep_calib_equation(MAXPAR_META) =  'n/a';
character(len=1024)  :: predep_calib_coeff(MAXPAR_META) =  'n/a';
character(len=1024)  :: predep_calib_comment(MAXPAR_META) = 'n/a';
character(len=256),dimension(MAXPAR_META,2) :: info_multiple_param_sensor;
character(len=256) :: calib_date_oxy, calib_date_temp, calib_date_press, calib_date_conduct;
integer :: count_multiple_param_sensor;
! ===========================================================================================================

!==============================================================================================
! Begin Traj Variables
!==============================================================================================

character(len=1)  :: prof_JULD_STATUS ;
character(len=1)  :: prof_JULD_ADJUSTED;
character(len=1)  :: prof_JULD_ADJUSTED_STATUS;
character(len=1)  :: prof_JULD_ADJUSTED_QC;
integer           :: prof_MEASUREMENT_CODE;
integer, parameter :: MAX_ENTER_TRAJ = 300;
character(len=1)  :: prof_POSITION_ACCURACY = '0';
integer :: cnt_traj_table_param,cnt_traj_table_skip_param
character(len=40), dimension(MAX_PARM_TABLE) ::  qc_traj_skip_lst    !non-Traj parameters 
character(len=40), dimension(MAX_PARM_TABLE) ::  qc_traj_params_lst  !Traj parameters 
character(len=128), dimension(MAX_PARM_TABLE) :: nc_traj_params_lst  !new NC Traj parameters 
! character(len=128), dimension(MAX_PARAM_TYPE,MAX_MEASUREMENT) :: trajectory_parameter_measurement =FILLCHAR 
integer , dimension(MAX_PARAM_TYPE,MAX_MEASUREMENT) :: int_trajectory_parameter_measurement =FILLINT 
character(len=1), dimension(MAX_PARAM_TYPE,MAX_MEASUREMENT) :: status_trajectory_parameter_measurement =FILLCHAR
character(len=1), dimension(MAX_MEASUREMENT) :: SATELLITE_NAME =FILLCHAR
character(len=1), dimension(MAX_MEASUREMENT) :: POSITION_ACCURACY=FILLCHAR
character(len=128), dimension(MAXTRAJ) :: trajectory_parameter_name  
integer :: trajectory_parameter_cycles(MAXTRAJ),cyc_no(MAXTRAJ)                      
integer :: current_cyc,itp


! Old only used by traj logical :: pos_added_flag(MAXPOS)
character(len=10) :: dummy_date(MAXPOS)
character(len=8)  ::  dummy_time(MAXPOS)
logical  :: found_CLASS=.false. ! Class is not present, POSQC is present instead
integer                 ::  CONFIG_MISSION_NUMBER_TRAJ(MAXCYC)=FILLINT
! OD character(len=1)        ::  JULD_FIRST_STABILIZATION_STATUS(MAXCYC)=FILLCHAR;
character(len=1), parameter      ::  JULD_FLAG_FOR_UNKNOWN = '9'
double precision, parameter :: ONE_MIN_AS_DAY = 0.000694454  ! value of 1/1440 which is  1/(24X60)
character(len=16)       ::  var_MANUAL_VERSION ! manual/decoder version from Ascii meta file 
character(len=14)            ::  start_descent_date; ! Descent start time/date from phy/qc file.

! resolution and C/FORTRAN_format attributes



! <PARAM> id definitions
!  PRES     TEMP     CNDC     PSAL    DOXY



!================================================================================
!   ! SCIENTIFIC_CALIB declarations
!================================================================================


! <PARAM> id definitions

integer :: PROF_QC_ids(MAXPAR);               ! nc ids for Profile_<>)qc
!==============================================================================================
! End Traj Variables
!==============================================================================================

contains

! ===========================================================================================================
! module functions/subroutines
! ===========================================================================================================

  !----------------------------------------------------------------------------------------------
  !       Subroutine to define and prepare data which goes into the netcdf file.
  !----------------------------------------------------------------------------------------------
subroutine def_ncmeta_vars(caller,nctype,ncid)

    character(*), intent( in ) :: caller        ! who is calling me?
    character(*), intent( in ) :: nctype        ! prof/traj/meta
    integer,      intent( in ) :: ncid          ! file id
    integer :: read_configtable
    integer :: idx
    character(len=15), parameter :: sub = 'def_ncmeta_vars'
    character(len=100) :: scan_meta_result =""
    double precision   :: doublefill(1);

    !---------!  calling function which reads the cross-reference table with config params  --------
    iret = read_configtable("CONFIG_TABLE")
    if(iret == 0 )then;
           if(DEBUG)then; write(msg, '("NCMETA_ARRAYS: read config table successful!!")'); call wrtdbg(sub,msg); endif
    else
	 write(errmsg(1),'("NON-FATAL ERROR 257  in def_ncmeta_vars in ncmeta_arrays")')
         write(errmsg(2),'("NON-FATAL ERROR 257  Error occured while reading Config Table")');
         call error_report(sub,257,2,errmsg); ! def_ncmeta_vars in ncmeta_arrays error reading CONFIG_TABLE
    endif
    !-------------------------------------------------------------------------------------------------------------
    PTT_dims = STRING256_dimid;
     ret=nf_def_var(ncid, 'PTT', NF_CHAR,1, PTT_dims,PTT_id);
    loc=01; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PTT_id, 'long_name', 46, 'Transmission identifier (ARGOS, ORBCOMM, etc.)');
    loc=02; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PTT_id, '_FillValue', 1, ' ');
    loc=03; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    TRANS_SYSTEM_dims(1) = STRING16_dimid;
    TRANS_SYSTEM_dims(2) = N_TRANS_SYSTEM_dimid;
     ret=nf_def_var(ncid, 'TRANS_SYSTEM', NF_CHAR,2, TRANS_SYSTEM_dims,TRANS_SYSTEM_id);
    loc=04; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, TRANS_SYSTEM_id, 'long_name', 30, 'Telecommunications system used');
                                                                   ! NOTE :  "Telecommunication" as per manual
    loc=05; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, TRANS_SYSTEM_id, '_FillValue', 1, ' ');
    loc=06; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    TRANS_SYSTEM_IDX_dims (1) = STRING32_dimid;
    TRANS_SYSTEM_IDX_dims (2) = N_TRANS_SYSTEM_dimid;
     ret=nf_def_var(ncid, 'TRANS_SYSTEM_ID', NF_CHAR,2, TRANS_SYSTEM_IDX_dims,TRANS_SYSTEM_IDX_id);
    loc=07; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, TRANS_SYSTEM_IDX_id, 'long_name', 50, 'Program identifier used by the transmission system');
    loc=08; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, TRANS_SYSTEM_IDX_id, '_FillValue', 1, ' ');
    loc=09; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    TRANS_FREQUENCY_dims (1) = STRING16_dimid;
    TRANS_FREQUENCY_dims (2) = N_TRANS_SYSTEM_dimid;
     ret=nf_def_var(ncid, 'TRANS_FREQUENCY', NF_CHAR,2, TRANS_FREQUENCY_dims,TRANS_FREQUENCY_id);
    loc=10; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, TRANS_FREQUENCY_id, 'long_name', 40, 'Frequency of transmission from the float');
    loc=11; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, TRANS_FREQUENCY_id, 'units', 5, 'hertz');
    loc=12; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, TRANS_FREQUENCY_id, '_FillValue', 1, ' ');
    loc=13; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    POSITIONING_SYSTEM_dims(1) = STRING8_dimid;
    POSITIONING_SYSTEM_dims(2) = N_POSITIONING_SYSTEM_dimid;
     ret=nf_def_var(ncid, 'POSITIONING_SYSTEM', NF_CHAR,2, POSITIONING_SYSTEM_dims,POSITIONING_SYSTEM_id);
    loc=14; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITIONING_SYSTEM_id, 'long_name', 18, 'Positioning system');
    loc=15; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITIONING_SYSTEM_id, 'conventions', 23, 'Argo reference table 22');
    loc=16; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, POSITIONING_SYSTEM_id, '_FillValue', 1, ' ');
    loc=17; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PLATFORM_FAMILY_dims = STRING256_dimid;
     ret=nf_def_var(ncid, 'PLATFORM_FAMILY', NF_CHAR,1, PLATFORM_FAMILY_dims,PLATFORM_FAMILY_id);
    loc=18; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_FAMILY_id, 'long_name', 22, 'Category of instrument');
    loc=19; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_FAMILY_id, 'conventions', 23, 'Argo reference table 22');
    loc=20; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_FAMILY_id, '_FillValue', 1, ' ');
    loc=21; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PLATFORM_TYPE_dims = STRING32_dimid;
     ret=nf_def_var(ncid, 'PLATFORM_TYPE', NF_CHAR,1, PLATFORM_TYPE_dims,PLATFORM_TYPE_id);
    loc=22; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_TYPE_id, 'long_name', 13, 'Type of float');
    loc=23; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_TYPE_id, 'conventions', 23, 'Argo reference table 23');
    loc=24; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_TYPE_id, '_FillValue', 1, ' ');
    loc=25; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PLATFORM_MAKER_dims = STRING256_dimid;
     ret=nf_def_var(ncid, 'PLATFORM_MAKER', NF_CHAR,1, PLATFORM_MAKER_dims,PLATFORM_MAKER_id);
    loc=26; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_MAKER_id, 'long_name', 24, 'Name of the manufacturer');
    loc=27; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_MAKER_id, 'conventions', 23, 'Argo reference table 24');
    loc=28; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PLATFORM_MAKER_id, '_FillValue', 1, ' ');
    loc=29; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    FIRMWARE_VERSION_dims = STRING32_dimid;
     ret=nf_def_var(ncid, 'FIRMWARE_VERSION', NF_CHAR,1, FIRMWARE_VERSION_dims,FIRMWARE_VERSION_id);
    loc=30; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, FIRMWARE_VERSION_id, 'long_name', 30, 'Firmware version for the float');
    loc=31; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, FIRMWARE_VERSION_id, '_FillValue', 1, ' ');
    loc=32; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    MANUAL_VERSION_dims = STRING16_dimid;
     ret=nf_def_var(ncid, 'MANUAL_VERSION', NF_CHAR,1, MANUAL_VERSION_dims,MANUAL_VERSION_id);
    loc=33; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, MANUAL_VERSION_id, 'long_name', 28, 'Manual version for the float');
    loc=34; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, MANUAL_VERSION_id, '_FillValue', 1, ' ');
    loc=35; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    FLOAT_SERIAL_NO_dims = STRING32_dimid;
     ret=nf_def_var(ncid, 'FLOAT_SERIAL_NO', NF_CHAR,1, FLOAT_SERIAL_NO_dims,FLOAT_SERIAL_NO_id);
    loc=36; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,FLOAT_SERIAL_NO_id, 'long_name', 26, 'Serial number of the float');
    loc=37; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, FLOAT_SERIAL_NO_id, '_FillValue', 1, ' ');
    loc=38; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    STANDARD_FORMAT_ID_dims = STRING16_dimid;
     ret=nf_def_var(ncid, 'STANDARD_FORMAT_ID', NF_CHAR,1, STANDARD_FORMAT_ID_dims,STANDARD_FORMAT_ID_id);
    loc=39; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,STANDARD_FORMAT_ID_id, 'long_name', 70, 'Standard format &
                                                                &number to describe the data format type for each float');
    loc=40; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, STANDARD_FORMAT_ID_id, '_FillValue', 1, ' ');
    loc=41; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    DAC_FORMAT_ID_dims = STRING16_dimid;
     ret=nf_def_var(ncid, 'DAC_FORMAT_ID', NF_CHAR,1, DAC_FORMAT_ID_dims,DAC_FORMAT_ID_id);
    loc=42; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DAC_FORMAT_ID_id, 'long_name', 77, 'Format number used by the DAC to &
                                                                    &describe the data format type for each float');
    loc=43; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DAC_FORMAT_ID_id, '_FillValue', 1, ' ');
    loc=44; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    WMO_INST_TYPE_dims = STRING4_dimid;
     ret=nf_def_var(ncid, 'WMO_INST_TYPE', NF_CHAR,1, WMO_INST_TYPE_dims,WMO_INST_TYPE_id);
    loc=45; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, WMO_INST_TYPE_id, 'long_name', 21, 'Coded instrument type');
    loc=46; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, WMO_INST_TYPE_id, 'conventions', 22, 'Argo reference table 8');
    loc=47; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, WMO_INST_TYPE_id, '_FillValue', 1, ' ');
    loc=48; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PROJECT_NAME_dims = STRING64_dimid;
     ret=nf_def_var(ncid, 'PROJECT_NAME', NF_CHAR,1, PROJECT_NAME_dims,PROJECT_NAME_id);
    loc=49; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PROJECT_NAME_id, 'long_name',42, 'Program under which the float was deployed');
    loc=50; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PROJECT_NAME_id, '_FillValue', 1, ' ');
    loc=51; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(msg,*)'NCTRAJ def DATA_CENTRE for meta'; call wrtlog(sub,msg);
    DATA_CENTRE_dims = STRING2_dimid;
     ret=nf_def_var(ncid,'DATA_CENTRE', NF_CHAR,NDIMS1, DATA_CENTRE_dims,DATA_CENTRE_id);  call cke(sub,ret,31); ! For ncmeta
     ret=nf_put_att_text(ncid,DATA_CENTRE_id,'long_name', 51, DC_long_name_meta);          call cke(sub,ret,33);
     ret=nf_put_att_text(ncid,DATA_CENTRE_id,'conventions', 22, 'Argo reference table 4'); call cke(sub,ret,34);
     ret=nf_put_att_text(ncid,DATA_CENTRE_id,'_FillValue', 1, ' ');                        call cke(sub,ret,35);
    !--------------------------------------------------------------------------------------------------------------
    PI_NAME_dims = STRING64_dimid;
     ret=nf_def_var(ncid, 'PI_NAME', NF_CHAR,1, PI_NAME_dims,PI_NAME_id);
    loc=52; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PI_NAME_id, 'long_name',34, 'Name of the principal investigator');
    loc=53; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PI_NAME_id, '_FillValue', 1, ' ');
    loc=54; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    ANOMALY_dims = STRING256_dimid;
     ret=nf_def_var(ncid, 'ANOMALY', NF_CHAR,1, ANOMALY_dims,ANOMALY_id);
    loc=55; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, ANOMALY_id, 'long_name',57, 'Describe any anomalies or problems the float may have had');
    loc=56; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, ANOMALY_id, '_FillValue', 1, ' ');
    loc=57; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    BATTERY_TYPE_dims = STRING64_dimid;
     ret=nf_def_var(ncid, 'BATTERY_TYPE', NF_CHAR,1, BATTERY_TYPE_dims,BATTERY_TYPE_id);
    loc=58; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, BATTERY_TYPE_id, 'long_name', 34, 'Type of battery packs in the float');
    loc=59; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, BATTERY_TYPE_id, '_FillValue', 1, ' ');
    loc=60; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    BATTERY_PACKS_dims = STRING64_dimid;
     ret=nf_def_var(ncid, 'BATTERY_PACKS', NF_CHAR,1, BATTERY_PACKS_dims,BATTERY_PACKS_id);
    loc=61; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, BATTERY_PACKS_id, 'long_name', 43, 'Configuration of battery packs in the float');
    loc=62; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, BATTERY_PACKS_id, '_FillValue', 1, ' ');
    loc=63; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    CONTROLLER_BOARD_TYPE_PRIMARY_dims = STRING32_dimid;
     ret=nf_def_var(ncid, 'CONTROLLER_BOARD_TYPE_PRIMARY', NF_CHAR,1,&
                                                      CONTROLLER_BOARD_TYPE_PRIMARY_dims, CONTROLLER_BOARD_TYPE_PRIMARY_id);
    loc=64; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CONTROLLER_BOARD_TYPE_PRIMARY_id, 'long_name', 32, 'Type of primary controller board');
    loc=65; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CONTROLLER_BOARD_TYPE_PRIMARY_id, '_FillValue', 1, ' ');
    loc=66; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    CONTROLLER_BOARD_TYPE_SECONDARY_dims = STRING32_dimid;
     ret=nf_def_var(ncid, 'CONTROLLER_BOARD_TYPE_SECONDARY', NF_CHAR,1,&
                                                      CONTROLLER_BOARD_TYPE_SECONDARY_dims, CONTROLLER_BOARD_TYPE_SECONDARY_id);
    loc=67; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CONTROLLER_BOARD_TYPE_SECONDARY_id, &
                                                     'long_name', 34, 'Type of secondary controller board');
    loc=68; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CONTROLLER_BOARD_TYPE_SECONDARY_id, '_FillValue', 1, ' ');
    loc=69; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    CONTROLLER_BOARD_SERIAL_NO_PRIMARY_dims = STRING32_dimid;
     ret=nf_def_var(ncid, 'CONTROLLER_BOARD_SERIAL_NO_PRIMARY', &
                   NF_CHAR,1,CONTROLLER_BOARD_SERIAL_NO_PRIMARY_dims, CONTROLLER_BOARD_SERIAL_NO_PRIMARY_id);
    loc=70; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CONTROLLER_BOARD_SERIAL_NO_PRIMARY_id,&
                            'long_name', 45, 'Serial number of the primary controller board');
    loc=71; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CONTROLLER_BOARD_SERIAL_NO_PRIMARY_id, '_FillValue', 1, ' ');
    loc=72; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    CONTROLLER_BOARD_SERIAL_NO_SECONDARY_dims = STRING32_dimid;
     ret=nf_def_var(ncid, 'CONTROLLER_BOARD_SERIAL_NO_SECONDARY', NF_CHAR,1, &
                 CONTROLLER_BOARD_SERIAL_NO_SECONDARY_dims, CONTROLLER_BOARD_SERIAL_NO_SECONDARY_id);
    loc=73; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CONTROLLER_BOARD_SERIAL_NO_SECONDARY_id, &
                                             'long_name', 47, 'Serial number of the secondary controller board');
    loc=74; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CONTROLLER_BOARD_SERIAL_NO_SECONDARY_id, '_FillValue', 1, ' ');
    loc=75; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    SPECIAL_FEATURES_dims = STRING1024_dimid;
     ret=nf_def_var(ncid, 'SPECIAL_FEATURES', NF_CHAR,1, SPECIAL_FEATURES_dims,  SPECIAL_FEATURES_id);
    loc=76; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SPECIAL_FEATURES_id, 'long_name', &
                                                         57, 'Extra features of the float (algorithms, compressee etc.)');
    loc=77; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SPECIAL_FEATURES_id, '_FillValue', 1, ' ');
    loc=78; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    FLOAT_OWNER_dims = STRING64_dimid;
     ret=nf_def_var(ncid, 'FLOAT_OWNER', NF_CHAR,1, FLOAT_OWNER_dims, FLOAT_OWNER_id);
    loc=79; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, FLOAT_OWNER_id, 'long_name', 11, 'Float owner');
    loc=80; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, FLOAT_OWNER_id, '_FillValue', 1, ' ');
    loc=81; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    OPERATING_INSTITUTION_dims = STRING64_dimid;
     ret=nf_def_var(ncid, 'OPERATING_INSTITUTION', NF_CHAR,1,OPERATING_INSTITUTION_dims,  OPERATING_INSTITUTION_id);
    loc=82; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, OPERATING_INSTITUTION_id, 'long_name', 34, 'Operating institution of the float');
    loc=83; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, OPERATING_INSTITUTION_id, '_FillValue', 1, ' ');
    loc=84; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    CUSTOMISATION_dims = STRING1024_dimid;
     ret=nf_def_var(ncid, 'CUSTOMISATION', NF_CHAR,1, CUSTOMISATION_dims, CUSTOMISATION_id);
    loc=85; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CUSTOMISATION_id, 'long_name', 57, &
                             'Float customisation, i.e. (institution and modifications)');
    loc=86; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, CUSTOMISATION_id, '_FillValue', 1, ' ');
    loc=87; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    LAUNCH_DATE_dims = DATE_TIME_dimid;
     ret=nf_def_var(ncid, 'LAUNCH_DATE', NF_CHAR,1,LAUNCH_DATE_dims,LAUNCH_DATE_id);
    loc=88; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_DATE_id, 'long_name', 28,&
                                            'Date (UTC) of the deployment');
    loc=89; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_DATE_id, 'conventions',14,'YYYYMMDDHHMISS')
    loc=90; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_DATE_id, '_FillValue', 1, fillchar);
    loc=91; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    LAUNCH_LATITUDE_dims = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'LAUNCH_LATITUDE', NF_DOUBLE,0,LAUNCH_LATITUDE_dims,LAUNCH_LATITUDE_id);
    loc=92; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_LATITUDE_id, 'long_name', 35,'Latitude of the float when deployed')
    loc=93; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_LATITUDE_id, 'units', 12, 'degree_north');
    loc=94; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, LAUNCH_LATITUDE_id, '_FillValue', NF_DOUBLE,1,FILLPOSN);
    doublefill(1)=-90.0;  ret=nf_put_att_double(ncid, LAUNCH_LATITUDE_id, 'valid_min', NF_DOUBLE,1,doublefill(1))
    doublefill(1)=90.0;  ret=nf_put_att_double(ncid, LAUNCH_LATITUDE_id, 'valid_max', NF_DOUBLE,1,doublefill(1))
     ret=nf_put_att_text(ncid, LAUNCH_LATITUDE_id, 'axis', 1, 'Y')
    loc=95; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    LAUNCH_LONGITUDE_dims = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'LAUNCH_LONGITUDE', NF_DOUBLE,0, LAUNCH_LONGITUDE_dims,LAUNCH_LONGITUDE_id);
    loc=96; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_LONGITUDE_id, 'long_name', 36,'Longitude of the float when deployed')
    loc=98; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_LONGITUDE_id, 'units', 11, 'degree_east');
    loc=99; call cke(sub,ret,loc);
     ret=nf_put_att_double(ncid, LAUNCH_LONGITUDE_id, '_FillValue', NF_DOUBLE,1,FILLPOSN);
    loc=100; call cke(sub,ret,loc);
    doublefill(1)=-180;
     ret=nf_put_att_double(ncid, LAUNCH_LONGITUDE_id, 'valid_min', NF_DOUBLE, 1,doublefill(1))
    loc=101; call cke(sub,ret,loc);
    doublefill(1)=180;
     ret=nf_put_att_double(ncid, LAUNCH_LONGITUDE_id, 'valid_max', NF_DOUBLE, 1, doublefill(1))
    loc=102; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_LONGITUDE_id, 'axis', 1, 'X')
    loc=103; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    LAUNCH_QC_dims = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'LAUNCH_QC', NF_CHAR,0, LAUNCH_QC_dims,LAUNCH_QC_id);
    loc=104; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_QC_id, 'long_name', 41,'Quality on launch date, time and location')
    loc=105; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_QC_id, 'conventions', 22, 'Argo reference table 2');
    loc=106; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, LAUNCH_QC_id, '_FillValue', 1, ' ');
    loc=107; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    START_DATE_dims = DATE_TIME_dimid;
     ret=nf_def_var(ncid, 'START_DATE', NF_CHAR,1,START_DATE_dims, START_DATE_id);
    loc=108; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, START_DATE_id, 'long_name', 44, 'Date (UTC) of the first descent of the float');
    loc=109; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, START_DATE_id, 'conventions',14,'YYYYMMDDHHMISS')
    loc=110; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, START_DATE_id, '_FillValue', 1, fillchar);
    loc=111; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    START_DATE_QC_dims = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'START_DATE_QC',NF_CHAR,0,START_DATE_QC_dims,START_DATE_QC_id);
    loc=112; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, START_DATE_QC_id, 'long_name', 21, 'Quality on start date');
    loc=113; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, START_DATE_QC_id, 'conventions',22,'Argo reference table 2')
    loc=114; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, START_DATE_QC_id, '_FillValue', 1, ' ');
    loc=115; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    STARTUP_DATE_dims = DATE_TIME_dimid;
     ret=nf_def_var(ncid, 'STARTUP_DATE', NF_CHAR,1,STARTUP_DATE_dims, STARTUP_DATE_id);
    loc=116; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, STARTUP_DATE_id, 'long_name', 41, 'Date (UTC) of the activation of the float');
    loc=117; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, STARTUP_DATE_id, 'conventions',14,'YYYYMMDDHHMISS')
    loc=118; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, STARTUP_DATE_id, '_FillValue', 1, fillchar);
    loc=119; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    STARTUP_DATE_QC_dims = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'STARTUP_DATE_QC',NF_CHAR,0,STARTUP_DATE_QC_dims,STARTUP_DATE_QC_id);
    loc=120; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, STARTUP_DATE_QC_id, 'long_name', 23, 'Quality on startup date');
    loc=121; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, STARTUP_DATE_QC_id, 'conventions',22,'Argo reference table 2')
    loc=122; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, STARTUP_DATE_QC_id, '_FillValue', 1, ' ');
    !--------------------------------------------------------------------------------------------------------------
    DEPLOYMENT_PLATFORM_dims = STRING32_dimid;
     ret=nf_def_var(ncid, 'DEPLOYMENT_PLATFORM', NF_CHAR,1, DEPLOYMENT_PLATFORM_dims,DEPLOYMENT_PLATFORM_id);
    loc=123; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DEPLOYMENT_PLATFORM_id, 'long_name', 37, 'Identifier of the deployment platform');
    loc=124; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DEPLOYMENT_PLATFORM_id, '_FillValue', 1, ' ');
    loc=125; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    DEPLOYMENT_CRUISE_ID_dims = STRING32_dimid;
     ret=nf_def_var(ncid, 'DEPLOYMENT_CRUISE_ID', NF_CHAR,1, DEPLOYMENT_CRUISE_ID_dims,DEPLOYMENT_CRUISE_ID_id);
    loc=126; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DEPLOYMENT_CRUISE_ID_id, 'long_name', 80,&
                           'Identification number or reference number of the cruise used to deploy the float');
    loc=127; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DEPLOYMENT_CRUISE_ID_id, '_FillValue', 1, ' ');
    loc=128; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    DEPLOYMENT_REFERENCE_STATION_ID_dims = STRING256_dimid;
     ret=nf_def_var(ncid, 'DEPLOYMENT_REFERENCE_STATION_ID', &
                         NF_CHAR,1, DEPLOYMENT_REFERENCE_STATION_ID_dims,DEPLOYMENT_REFERENCE_STATION_ID_id);
    loc=129; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DEPLOYMENT_REFERENCE_STATION_ID_id,&
	'long_name', 86, 'Identifier or reference number of co-located stations used to verify the first profile');
    loc=130; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, DEPLOYMENT_REFERENCE_STATION_ID_id, '_FillValue', 1, ' ');
    loc=131; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    END_MISSION_DATE_dims = DATE_TIME_dimid;
     ret=nf_def_var(ncid, 'END_MISSION_DATE', &
                         NF_CHAR,1,END_MISSION_DATE_dims,END_MISSION_DATE_id);
    loc=132; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, END_MISSION_DATE_id,&
                          'long_name', 45, 'Date (UTC) of the end of mission of the float');
    loc=134; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, END_MISSION_DATE_id, 'conventions',14,'YYYYMMDDHHMISS')
    loc=135; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, END_MISSION_DATE_id, '_FillValue', 1, ' ');
    loc=136; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    END_MISSION_STATUS_dims = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'END_MISSION_STATUS', &
                         NF_CHAR,0, END_MISSION_STATUS_dims,END_MISSION_STATUS_id);
    loc=137; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, END_MISSION_STATUS_id,&
                          'long_name', 41, 'Status of the end of mission of the float');
    loc=138; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, END_MISSION_STATUS_id, 'conventions',44,'T:No more transmission received, R:Retrieved')
    loc=139; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, END_MISSION_STATUS_id, '_FillValue', 1, ' ');
    loc=140; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    LAUNCH_CONFIG_PARAMETER_NAME_dims(2)  = N_LAUNCH_CONFIG_PARAM_dimid
    LAUNCH_CONFIG_PARAMETER_NAME_dims(1) = STRING128_dimid
     ret=nf_def_var(ncid, 'LAUNCH_CONFIG_PARAMETER_NAME', NF_CHAR, ND2, LAUNCH_CONFIG_PARAMETER_NAME_dims, &
                  LAUNCH_CONFIG_PARAMETER_NAME_id)
    loc = 141; call cke(sub, iret, loc);
     ret=nf_put_att_text(ncid, LAUNCH_CONFIG_PARAMETER_NAME_id, 'long_name', 41,  'Name of configuration parameter at launch')
    loc = 142; call cke(sub, iret, loc);
     ret=nf_put_att_text(ncid, LAUNCH_CONFIG_PARAMETER_NAME_id, '_FillValue', 1,' ');
    loc = 143; call cke(sub, iret, loc);
    !--------------------------------------------------------------------------------------------------------------
    !write(msg,*) "DBG:>>> n_launch_config_param= ", n_launch_config_param; call wrtlog(sub,msg);
    LAUNCH_CONFIG_PARAMETER_VALUE_dims(1) = N_LAUNCH_CONFIG_PARAM_dimid
     ret=nf_def_var(ncid, 'LAUNCH_CONFIG_PARAMETER_VALUE', NF_DOUBLE, ND1,&
           LAUNCH_CONFIG_PARAMETER_VALUE_dims, LAUNCH_CONFIG_PARAMETER_VALUE_id)
    loc = 144; call cke(sub, iret, loc);
     ret=nf_put_att_text(ncid, LAUNCH_CONFIG_PARAMETER_VALUE_id, 'long_name', 42, 'Value of configuration parameter at launch')
    loc = 145; call cke(sub, iret, loc);
     ret=nf_put_att_double(ncid, LAUNCH_CONFIG_PARAMETER_VALUE_id,'_FillValue', NF_DOUBLE,1,FILLCNFPAR); ! DEF AS FLOAT IN 3.1?
    ! old doublefill(1));
    loc = 146; call cke(sub, iret, loc);
    !--------------------------------------------------------------------------------------------------------------
    !if(N_CONFIG_PARAM_len > 0 .AND. N_MISSIONS_len > 0)then
       CONFIG_PARAMETER_NAME_dims(2)  = N_CONFIG_PARAM_dimid
       CONFIG_PARAMETER_NAME_dims(1)  = STRING128_dimid
        ret=nf_def_var(ncid, 'CONFIG_PARAMETER_NAME', NF_CHAR, ND2, CONFIG_PARAMETER_NAME_dims, CONFIG_PARAMETER_NAME_id)
       loc = 147; call cke(sub, iret, loc);
        ret=nf_put_att_text(ncid, CONFIG_PARAMETER_NAME_id, 'long_name', 31, 'Name of configuration parameter')
       loc = 148; call cke(sub, iret, loc);
        ret=nf_put_att_text(ncid, CONFIG_PARAMETER_NAME_id, '_FillValue', 1,' ');
       loc = 149; call cke(sub, iret, loc);
     !endif
    !--------------------------------------------------------------------------------------------------------------
    !if(N_CONFIG_PARAM_len > 0 .AND. N_MISSIONS_len > 0)then
       !write (msg,*) " DBG:>>>>> N N_CONFIG_PARAM_len = " , N_CONFIG_PARAM_len ; call wrtlog(sub,msg);
       CONFIG_PARAMETER_VALUE_dims(2) = N_MISSIONS_dimid
       CONFIG_PARAMETER_VALUE_dims(1) = N_CONFIG_PARAM_dimid
        ret=nf_def_var(ncid, 'CONFIG_PARAMETER_VALUE', NF_DOUBLE, ND2, CONFIG_PARAMETER_VALUE_dims, CONFIG_PARAMETER_VALUE_id)
       loc = 150; call cke(sub, iret, loc);
        ret=nf_put_att_text(ncid, CONFIG_PARAMETER_VALUE_id, 'long_name', 32, 'Value of configuration parameter')
       loc = 151; call cke(sub, iret, loc);
        ret=nf_put_att_double(ncid, CONFIG_PARAMETER_VALUE_id,'_FillValue', NF_DOUBLE,1,FILLCNFPAR); ! DEF AS FLOAT IN 3.1?
       loc = 152; call cke(sub, iret, loc);
    !endif
    !--------------------------------------------------------------------------------------------------------------
    ! CONFIG_MISSION_NUMBER and CONFIG_MISSION_COMMENT may not be present always and
    ! They are non-mandatory variables as per the manual.
    if(cnt_missions > 0)then
        write(msg,*)'NC CONFIG def  CONFIG_MISSION_NUMBER for meta'; call wrtlog(sub,msg);
        CONFIG_MISSION_NUMBER_dims(1) = N_MISSIONS_dimid
         ret=nf_def_var(ncid, 'CONFIG_MISSION_NUMBER', NF_INT, ND1, CONFIG_MISSION_NUMBER_dims, CONFIG_MISSION_NUMBER_id)
        loc = 153; call cke(sub, iret, loc);
         ret=nf_put_att_text(ncid, CONFIG_MISSION_NUMBER_id, 'long_name', 58, &
                            'Unique number denoting the missions performed by the float')
        loc = 154; call cke(sub, iret, loc);
         ret=nf_put_att_text(ncid, CONFIG_MISSION_NUMBER_id, 'conventions',33,'1...N, 1 : first complete mission')
        loc = 155; call cke(sub, iret, loc);
         ret=nf_put_att_int(ncid,CONFIG_MISSION_NUMBER_id, '_FillValue', NF_INT, 1, FILLINT) ! FILLINT int fill value in netcdf_arrays
        loc = 156; call cke(sub, iret, loc);
    else
	write(wrnmsg,'("WARNING 165 N_MISSION  =  0 in META NC File! @1")'); ! WRN
        call warning(sub,165,wrnmsg); ! WARNING 165 in ncmeta_arrays  subroutine def_ncmeta_vars ! WRN
    endif
    !--------------------------------------------------------------------------------------------------------------
    if(cnt_missions > 0)then
        write(msg,*)'NC CONFIG def  CONFIG_MISSION_COMMENT for meta'; call wrtlog(sub,msg);
        CONFIG_MISSION_COMMENT_dims(2) = N_MISSIONS_dimid
        CONFIG_MISSION_COMMENT_dims(1) = STRING256_dimid
         ret=nf_def_var(ncid, 'CONFIG_MISSION_COMMENT', NF_CHAR, ND2, CONFIG_MISSION_COMMENT_dims, CONFIG_MISSION_COMMENT_id);
        loc = 157; call cke(sub, iret, loc);
         ret=nf_put_att_text(ncid, CONFIG_MISSION_COMMENT_id, 'long_name', 24,'Comment on configuration')
        loc = 158; call cke(sub, iret, loc);
         ret=nf_put_att_text(ncid, CONFIG_MISSION_COMMENT_id, '_FillValue', 1,' ');
        loc = 159; call cke(sub, iret, loc);
    else
	write(wrnmsg,'("WARNING 166 N_MISSION  =  0 in META NC File! @2")'); ! WRN
        call warning(sub,166,wrnmsg); ! WARNING 166 in ncmeta_arrays subroutine def_ncmeta_vars ! WRN
    endif
    !--------------------------------------------------------------------------------------------------------------
    SENSOR_dims(1) = STRING32_dimid;
    SENSOR_dims(2) = N_SENSOR_dimid;
     ret=nf_def_var(ncid, 'SENSOR', NF_CHAR,ND2, SENSOR_dims,SENSOR_id);
    loc=160; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,SENSOR_id, 'long_name', 39, 'Name of the sensor mounted on the float');
    loc=161; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SENSOR_id, 'conventions', 23, 'Argo reference table 25');
    loc=162; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SENSOR_id, '_FillValue', 1, ' ');
    loc=163; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    SENSOR_MAKER_dims(1) = STRING256_dimid;
    SENSOR_MAKER_dims(2) = N_SENSOR_dimid;
     ret=nf_def_var(ncid, 'SENSOR_MAKER', NF_CHAR,ND2, SENSOR_MAKER_dims,SENSOR_MAKER_id);
    loc=164; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,SENSOR_MAKER_id, 'long_name', 31, 'Name of the sensor manufacturer');
    loc=165; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SENSOR_MAKER_id, 'conventions', 23, 'Argo reference table 26');
    loc=166; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SENSOR_MAKER_id, '_FillValue', 1, ' ');
    loc=167; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    SENSOR_MODEL_dims(1) = STRING256_dimid;
    SENSOR_MODEL_dims(2) = N_SENSOR_dimid;
     ret=nf_def_var(ncid, 'SENSOR_MODEL', NF_CHAR,ND2, SENSOR_MODEL_dims,SENSOR_MODEL_id);
    loc=168; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,SENSOR_MODEL_id, 'long_name', 14, 'Type of sensor');
    loc=169; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SENSOR_MODEL_id, 'conventions', 23, 'Argo reference table 27');
    loc=170; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SENSOR_MODEL_id, '_FillValue', 1, ' ');
    loc=171; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    SENSOR_SERIAL_NO_dims(1) = STRING16_dimid;
    SENSOR_SERIAL_NO_dims(2) = N_SENSOR_dimid;
     ret=nf_def_var(ncid, 'SENSOR_SERIAL_NO', NF_CHAR,ND2, SENSOR_SERIAL_NO_dims,SENSOR_SERIAL_NO_id);
    loc=172; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,SENSOR_SERIAL_NO_id, 'long_name', 27, 'Serial number of the sensor');
    loc=173; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, SENSOR_SERIAL_NO_id, '_FillValue', 1, ' ');
    loc=174; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PARAMETERX_dims(1) = STRING64_dimid;
    PARAMETERX_dims(2) = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'PARAMETER', NF_CHAR,ND2, PARAMETERX_dims,PARAMETERX_id);
    loc=175; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,PARAMETERX_id, 'long_name', 50, 'Name of parameter computed from float measurements');
    loc=176; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PARAMETERX_id, 'conventions', 22, 'Argo reference table 3');
    loc=177; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PARAMETERX_id, '_FillValue', 1, ' ');
    loc=178; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PARAMETER_SENSOR_dims(1) = STRING128_dimid;
    PARAMETER_SENSOR_dims(2) = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'PARAMETER_SENSOR', NF_CHAR,ND2, PARAMETER_SENSOR_dims,PARAMETER_SENSOR_id);
    loc=179; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,PARAMETER_SENSOR_id, 'long_name', 47, 'Name of the sensor that measures this parameter');
    loc=180; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PARAMETER_SENSOR_id, 'conventions', 23, 'Argo reference table 25');
    loc=181; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PARAMETER_SENSOR_id, '_FillValue', 1, ' ');
    loc=182; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PARAMETER_UNITS_dims(1) = STRING32_dimid;
    PARAMETER_UNITS_dims(2) = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'PARAMETER_UNITS', NF_CHAR,ND2, PARAMETER_UNITS_dims,PARAMETER_UNITS_id);
    loc=183; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,PARAMETER_UNITS_id, 'long_name', 49, 'Units of accuracy and resolution of the parameter');
    loc=184; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PARAMETER_UNITS_id, '_FillValue', 1, ' ');
    loc=185; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PARAMETER_ACCURACY_dims(1) = STRING32_dimid;
    PARAMETER_ACCURACY_dims(2) = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'PARAMETER_ACCURACY', NF_CHAR,ND2, PARAMETER_ACCURACY_dims,PARAMETER_ACCURACY_id);
    loc=186; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,PARAMETER_ACCURACY_id, 'long_name', 25, 'Accuracy of the parameter');
    loc=187; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PARAMETER_ACCURACY_id, '_FillValue', 1, ' ');
    loc=189; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PARAMETER_RESOLUTION_dims(1) = STRING32_dimid;
    PARAMETER_RESOLUTION_dims(2) = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'PARAMETER_RESOLUTION', NF_CHAR,ND2, PARAMETER_RESOLUTION_dims,PARAMETER_RESOLUTION_id);
    loc=190; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,PARAMETER_RESOLUTION_id, 'long_name', 27, 'Resolution of the parameter');
    loc=191; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PARAMETER_RESOLUTION_id, '_FillValue', 1, ' ');
    loc=192; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PREDEPLOYMENT_CALIB_EQUATION_dims(1) = STRING1024_dimid;
    PREDEPLOYMENT_CALIB_EQUATION_dims(2) = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'PREDEPLOYMENT_CALIB_EQUATION', NF_CHAR,ND2, &
                              PREDEPLOYMENT_CALIB_EQUATION_dims,PREDEPLOYMENT_CALIB_EQUATION_id);
    loc=193; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,PREDEPLOYMENT_CALIB_EQUATION_id, 'long_name', &
                           39, 'Calibration equation for this parameter');
    loc=194; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PREDEPLOYMENT_CALIB_EQUATION_id, '_FillValue', 1, ' ');
    loc=195; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PREDEPLOYMENT_CALIB_COEFFICIENT_dims(1) = STRING1024_dimid;
    PREDEPLOYMENT_CALIB_COEFFICIENT_dims(2) = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'PREDEPLOYMENT_CALIB_COEFFICIENT', &
                         NF_CHAR,ND2, PREDEPLOYMENT_CALIB_COEFFICIENT_dims,PREDEPLOYMENT_CALIB_COEFFICIENT_id);
    loc=196; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,PREDEPLOYMENT_CALIB_COEFFICIENT_id, &
                        'long_name', 42, 'Calibration coefficients for this equation');
    loc=197; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PREDEPLOYMENT_CALIB_COEFFICIENT_id, '_FillValue', 1, ' ');
    loc=198; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    PREDEPLOYMENT_CALIB_COMMENT_dims(1) = STRING1024_dimid;
    PREDEPLOYMENT_CALIB_COMMENT_dims(2) = N_PARAM_dimid;
     ret=nf_def_var(ncid, 'PREDEPLOYMENT_CALIB_COMMENT', NF_CHAR,ND2, &
                      PREDEPLOYMENT_CALIB_COMMENT_dims,PREDEPLOYMENT_CALIB_COMMENT_id);
    loc=199; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid,PREDEPLOYMENT_CALIB_COMMENT_id, &
                      'long_name', 46, 'Comment applying to this parameter calibration');
    loc=200; call cke(sub,ret,loc);
     ret=nf_put_att_text(ncid, PREDEPLOYMENT_CALIB_COMMENT_id, '_FillValue', 1, ' ');
    loc=201; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
end subroutine def_ncmeta_vars

  !----------------------------------------------------------------------------------------------
  !       Subroutine to write data into the meta netcdf file.
  !----------------------------------------------------------------------------------------------
subroutine put_ncmeta_data(caller,nctype,ncid)


    character(*), intent( in ) :: caller        ! who is calling me?
    character(*), intent( in ) :: nctype        ! prof/traj/meta
    integer,      intent( in ) :: ncid          ! file id
    integer :: space_count_to_pad
    character(len=15), parameter :: sub = 'put_ncmeta_data'
    !Variables to format NC data.
    character(len=4)   :: c04
    character(len=5)   :: c05
    character(len=7)   :: c07
    character(len=8)   :: c08
    character(len=14)  :: c14
    character(len=16)  :: c16
    character(len=32)  :: c32
    character(len=64)  :: c64
    character(len=128) :: c128
    character(len=256) :: c256
    character(len=1024):: c1024

    !Intermediate variables used while preparing NC data.
    !Common
    character(len=256) :: scan_meta_result,tmp_str1, tmp_str2;
    integer :: i_counter, i_tmp
    !real :: tmp_real

    !For battery pack
    character(len=64) :: batt_pack
    character (len=8) :: batt_count_str
    integer :: batt_count

    ! For date conversion
    integer :: start_day,start_mon,start_year,start_hh,start_mm
    integer :: lnch_day,lnch_mon,lnch_year,lnch_hh,lnch_mm
    integer :: time_array(9) ! array holds UTC time



    !For launch Config arrays
    !integer :: launch_config_param_start(ND2), launch_config_param_count(ND2)
    !For Config arrays
    !integer :: config_name_start(ND2), config_name_count(ND2)
    !integer :: config_value_start(ND2), config_value_count(ND2)  ! >>>>>> TO change?
    !integer :: mission_start(ND1), mission_count(ND1)
    !integer :: put_ncmeta_start(ND2), ncmeta_put_count(ND2)

    !start and count arrays for Netcdf variables of type 1D and 2D.
    integer :: nc_array_start1(ND1), nc_array_count1(ND1)
    integer :: nc_array_start2(ND2), nc_array_count2(ND2)

    ! Used to call impossible_location using real, not double
    real :: real_launch_lat, real_launch_lon

    !for debugging purposes
    integer :: i_debug
    logical meta_check_cdate

    i_debug = 0
    ascii_meta_launch_date = ""
    batt_pack=""
    scan_meta_result =""; tmp_str1=""; tmp_str2="";
    
    if(DEBUG .AND. DBLEV==70)then; write(msg,'(" Entering put_ncmeta_data ")'); call wrtdbg(sub,msg); endif
    !--------------------------------------------------------------------------------------------------------------
    write(c256,'(a256)') var_TRANSMISSION_NO
    if(DEBUG .AND. DBLEV==71)then; write(msg, *) "PTT (var_TRANSMISSION_NO) ", c256 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, PTT_id,adjustl(c256));
    loc=202; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c16, '(a16)') trans_type
    TRANS_SYSTEM(1) = adjustl(c16)
    nc_array_start2(1) = 1
    nc_array_count2(1) = 16
    nc_array_start2(2) = 1
    nc_array_count2(2) = 1
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "TRANS_SYSTEM (trans_type) : ", c16 ; call wrtdbg(sub,msg); endif
     ret=nf_put_vara_text(ncid, TRANS_SYSTEM_id, nc_array_start2,nc_array_count2, TRANS_SYSTEM);
    loc=202; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    call scan_metline_for_literal("program number",scan_meta_result);
    if(len_trim(scan_meta_result)>0)then
            write(c32, '(a32)') scan_meta_result(41:);
    else
            write(c32, '(a32)') 'n/a';
    endif
    TRANS_SYSTEM_IDX (1) = adjustl(c32)
    nc_array_count2(1) = 32
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "TRANS_SYSTEM_IDX (trans_type) : ", TRANS_SYSTEM_IDX(1) ; call wrtdbg(sub,msg); endif
     ret=nf_put_vara_text(ncid, TRANS_SYSTEM_IDX_id,nc_array_start2,nc_array_count2,TRANS_SYSTEM_IDX);
    loc=203; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c16, '(a16)') 'n/a'
    TRANS_FREQUENCY (1) = adjustl(c16)
    nc_array_count2(1) = 16
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "TRANS_FREQUENCY (trans_type) : ", TRANS_FREQUENCY(1) ; call wrtdbg(sub,msg); endif
     ret=nf_put_vara_text(ncid, TRANS_FREQUENCY_id, nc_array_start2,nc_array_count2 ,TRANS_FREQUENCY);
    loc=204; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    if(trim(trans_type) == 'ARGOS')then;
        POSITIONING_SYSTEM (1) = 'ARGOS   '
    else
        POSITIONING_SYSTEM (1) = 'GPS     '
        if(has_irid_positioning) POSITIONING_SYSTEM (2) = 'IRIDIUM'
    endif
    nc_array_count2(1) = 8
    nc_array_count2(2) = N_POSITIONING_SYSTEM_len
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "POSITIONING_SYSTEM  : ", POSITIONING_SYSTEM(1) ; call wrtdbg(sub,msg); endif
     ret=nf_put_vara_text(ncid, POSITIONING_SYSTEM_id,nc_array_start2,nc_array_count2,POSITIONING_SYSTEM);
    loc=205; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c256,'(a256)')'FLOAT'
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " PLATFORM_FAMILY  : ", c256 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, PLATFORM_FAMILY_id, adjustl(c256))
    loc=206; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    call scan_metline_for_literal("float manufacturer",scan_meta_result );
    tmp_str1 = scan_meta_result(41:)
    call to_upper(tmp_str1)
    tmp_str1 = trim(tmp_str1)

    tmp_str2 = chr_PLATFORM_TYPE
    call to_upper(tmp_str2)
    tmp_str2 = trim(tmp_str2)

    call get_platform_type(tmp_str1,tmp_str2)
    write(c32, '(a32)') tmp_str2
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " PLATFORM_TYPE  : ", c32 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, PLATFORM_TYPE_id, adjustl(c32));
    loc=207; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    call get_platform_maker_reftable24(tmp_str1)
    write(c256, '(a256)') tmp_str1
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " PLATFORM_MAKER  : ", c256 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, PLATFORM_MAKER_id, adjustl(c256));
    loc=208; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c32, '(a32)') var_FIRMWARE_VERSION
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " FIRMWARE_VERSION  : ", c32 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, FIRMWARE_VERSION_id, adjustl(c32));
    loc=209; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c16, '(a16)') var_FIRMWARE_VERSION
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " MANUAL_VERSION  : ", c16 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, MANUAL_VERSION_id, adjustl(c16));
    loc=210; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c32, '(a32)')  var_FLOAT_SERIAL_NO
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " FLOAT_SERIAL_NO  : ", c32 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, FLOAT_SERIAL_NO_id, adjustl(c32));
    loc=211; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c16, '(a16)') 'n/a';
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " STANDARD_FORMAT_ID  : ", c16 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, STANDARD_FORMAT_ID_id, adjustl(c16));
    loc=212; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c16, '(a16)') inst_type;
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " DAC_FORMAT_ID  : ", c16 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, DAC_FORMAT_ID_id, adjustl(c16));
    loc=213; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " WMO_INST_TYPE  : ", var_WMO_INST_TYPE ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, WMO_INST_TYPE_id, var_WMO_INST_TYPE);
    loc=214; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    if(INDEX(chr_PROJECT_NAME, 'UW, Argo')> 0)then
         write(c64, '(a64)') 'US ARGO PROJECT'
    else
         write(c64, '(a64)') chr_PROJECT_NAME
    endif
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " PROJECT_NAME  : ", c64 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, PROJECT_NAME_id,adjustl(c64));
    loc=215; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c64,'(a64)')var_PI_NAME
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " PI_NAME  : ", c64 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, PI_NAME_id, adjustl(c64));
    loc=215; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    call scan_metline_for_literal("anomaly",scan_meta_result );
    if(len(trim(scan_meta_result)) == 0)then
	write(c256, '(a256)')'n/a';
    else
	write(c256,'(a256)') scan_meta_result(41:);
    endif
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "  ANOMALY  : ", c256 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, ANOMALY_id, adjustl(c256));
    loc=216; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    call scan_metline_for_literal("battery type",scan_meta_result );
    write(c64, '(a64)') scan_meta_result(41:);
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "  BATTERY_TYPE  : ", c64 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, BATTERY_TYPE_id, adjustl(c64));
    loc=217; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    !Note: Possible values for battery type field are:
    !------------------------------------------------
    !     n/a
    !     Lithium
    !     Alkaline
    !     Lithium/Alkaline
    !     Alkaline/Lithium
    !     Lithium and Alkaline
    !     Alkaline and lithium
    !------------------------------------------------

    !   For identifying and counting the  number of Board/CPU Batteries
    batt_pack = ""
    scan_meta_result = ""
 
    call scan_metline_for_literal("Board serial number",scan_meta_result);
    if(len_trim(scan_meta_result)>0)then
        batt_pack = "board - "
    else
    	call scan_metline_for_literal("board serial number",scan_meta_result);
    	if(len_trim(scan_meta_result)>0)then
           batt_pack = "board - "
        else
          call scan_metline_for_literal("board battery serial number",scan_meta_result );
          if(len_trim(scan_meta_result)>0)then
             batt_pack = "board - "
          endif
        endif  
    endif
    
    if(batt_pack /= "")then
          tmp_str1 = scan_meta_result(41:)
          call count_given_char_in_string (trim(tmp_str1), ' ', batt_count)
          batt_count = batt_count + 1; ! Number of batteries = number of space + 1
          write(batt_count_str,'(i3)')batt_count
          batt_pack = trim(batt_pack)//trim(batt_count_str)//" (s/n: "//trim(tmp_str1)//");"
    else
          batt_pack = trim(batt_pack)//"n/a"
    endif

    !  For identifying and counting the  number of Pump Batteries
    tmp_str1 = ""
    batt_count = 0;
    batt_count_str = ""
    scan_meta_result = ""
    call scan_metline_for_literal("pump battery serial number",scan_meta_result );
    if(len_trim(scan_meta_result)>0)then
          batt_pack =  batt_pack//" pump - "
          tmp_str1 = scan_meta_result(41:)
          !write(msg, *) " BATTERY_PACKS scan_meta_result :", scan_meta_result ;
          call wrtlog(sub,msg);
          tmp_str1 = trim(tmp_str1)
          call count_given_char_in_string (tmp_str1, ' ', batt_count)
          batt_count = batt_count + 1;
          write( batt_count_str, '(i2)' ) batt_count
          batt_pack = trim(batt_pack)//trim(batt_count_str)//" (s/n: "//trim(tmp_str1)//");"
    else
         batt_pack =  batt_pack//" pump - n/a;"
    endif

    if(batt_pack == "")then
          batt_pack = 'n/a'
    endif

    write(c64, '(a64)') batt_pack;
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "   BATTERY_PACKS  : ", c64 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, BATTERY_PACKS_id, adjustl(c64));
    loc=218; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    scan_meta_result =""
    call scan_metline_for_literal("Board version",scan_meta_result );
    if(len_trim(scan_meta_result) > 0)then
       write(c32, '(a32)') scan_meta_result(41:);
    else
        write(c32, '(a32)')'n/a';
    endif
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "   CONTROLLER_BOARD_TYPE_PRIMARY  : ", c32 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, CONTROLLER_BOARD_TYPE_PRIMARY_id, adjustl(c32));
    loc=219; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c32, '(a32)') 'n/a';
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "   CONTROLLER_BOARD_TYPE_SECONDARY  : ", c32 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, CONTROLLER_BOARD_TYPE_SECONDARY_id, adjustl(c32));
    loc=220; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    scan_meta_result = ""
    call scan_metline_for_literal("Board serial number",scan_meta_result );
    if(len_trim(scan_meta_result)> 0)then
       write(c32, '(a32)') scan_meta_result(41:);
    else
        write(c32, '(a32)') 'n/a';
    endif
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "   CONTROLLER_BOARD_SERIAL_NO_PRIMARY  : ", c32 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, CONTROLLER_BOARD_SERIAL_NO_PRIMARY_id, adjustl(c32));
    loc=221; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c32, '(a32)') 'n/a';
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "   CONTROLLER_BOARD_SERIAL_NO_SECONDARY : ", c32 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, CONTROLLER_BOARD_SERIAL_NO_SECONDARY_id, adjustl(c32));
    loc=221; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    call scan_metline_for_literal("special features",scan_meta_result );
    if(len(trim(scan_meta_result)) == 0)then
	write(c1024, '(a1024)') 'n/a';
    else
	write(c1024, '(a1024)') scan_meta_result(41:);
    endif
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "   SPECIAL_FEATURES : ", c1024 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, SPECIAL_FEATURES_id, adjustl(c1024));
    loc=222; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c64, '(a64)')var_PI_NAME
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "   FLOAT_OWNER : ", c64 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, FLOAT_OWNER_id, adjustl(c64));
    loc=223; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    scan_meta_result = ""
    call scan_metline_for_literal("principal investigator address",scan_meta_result);
    if(len_trim(scan_meta_result) > 0)then
       write(c64, '(a64)') scan_meta_result(41:);
    else
        write(c64, '(a64)') 'n/a';
    endif
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "   OPERATING_INSTITUTION : ", c64 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, OPERATING_INSTITUTION_id, adjustl(c64));
    loc=224; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    scan_meta_result = ""
    call scan_metline_for_literal("customisation",scan_meta_result);
    if(len_trim(scan_meta_result) > 0)then
       write(c1024, '(a1024)') scan_meta_result(41:);
    else
        write(c1024, '(a1024)') 'n/a';
    endif
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "  CUSTOMISATION : ", c1024 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, CUSTOMISATION_id, adjustl(c1024));
    loc=225; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    scan_meta_result = ""
    call scan_metline_for_literal("launch time [dd mm yyyy hh mm (Z)]",&
                                                    scan_meta_result );
    tmp_str1 = trim(scan_meta_result(41:));
    read(tmp_str1,'(i2,x,i2,x,i4,x,i2,x,i2)')lnch_day,lnch_mon,lnch_year,lnch_hh,lnch_mm
    write(tmp_str2, '(i4,i2.2,i2.2,i2.2,i2.2)')lnch_year,lnch_mon,lnch_day,lnch_hh,lnch_mm
    write(c14, '(a14)') trim(tmp_str2)//"00"        ! Since input does not contain value for seconds
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "  LAUNCH_DATE : ", c14 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, LAUNCH_DATE_id, adjustl(c14));
    loc=226; call cke(sub,ret,loc);
    ascii_meta_launch_date = trim(c14);
    write (msg, '( "return after LAUNCH_DATE:" i2)')iret ; call wrtdbg(sub,msg);
    !--------------------------------------------------------------------------------------------------------------
    LAUNCH_LATITUDE = DBLE(launch_lat);
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "  LAUNCH_LATITUDE : ", LAUNCH_LATITUDE ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_double(ncid, LAUNCH_LATITUDE_id, LAUNCH_LATITUDE );
    loc=227; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    LAUNCH_LONGITUDE =DBLE(launch_lon);
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "  LAUNCH_LONGITUDE : ", LAUNCH_LONGITUDE ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_double(ncid, LAUNCH_LONGITUDE_id, LAUNCH_LONGITUDE);
    loc=228; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    LAUNCH_QC = "0"
    real_launch_lat = REAL(launch_lat); real_launch_lon = REAL(launch_lon);
    if(.not. impossible_location(real_launch_lat,real_launch_lon))then
        !write (msg, *) "impossible_location FALSE " ; call wrtdbg(sub,msg);
        if(check_cdate(ascii_meta_launch_date))then
                write (msg, *) "check_cdate TRUE " ; call wrtdbg(sub,msg);
                real_launch_lat = REAL(launch_lat); real_launch_lon = REAL(launch_lon);
                if(land_or_water('ncmeta_arrays', real_launch_lat,real_launch_lon) == 0)then
                        !write (msg, *) "land_or_water - 0 " ; call wrtdbg(sub,msg);
                        LAUNCH_QC = "1"
                endif
        endif
    else
	write(wrnmsg,'("WARNING 167 LAUNCH_QC Failed in META NC File! ")'); ! WRN
        call warning(sub,167,wrnmsg); ! WARNING 167 in ncmeta_arrays subroutine def_ncmeta_vars ! WRN
    endif
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " LAUNCH_QC : ", LAUNCH_QC ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, LAUNCH_QC_id, LAUNCH_QC);
    loc=229; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    !  Calling start time calc logic
    call prepare_metanc_startdate(caller,nctype,ncid);
    !--------------------------------------------------------------------------------------------------------------
    tmp_str1="";tmp_str2="";
    !According to the requirement in the Email from Claudia subj:"meta.nc cruise info" on 09/18/2017.
    call scan_metline_for_literal("deployment type",scan_meta_result );
    if(len_trim(scan_meta_result(41:))>0 .and. index(scan_meta_result(41:),'n/a')<=0)tmp_str1 = scan_meta_result(41:);
    call scan_metline_for_literal("deployment platform",scan_meta_result );
    if(len_trim(scan_meta_result(41:))>0 .and. index(scan_meta_result(41:),'n/a')<=0)tmp_str2 = scan_meta_result(41:);
    write(c32,'(a32)')trim(tmp_str1)//" "//trim(tmp_str2);
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " DEPLOYMENT_PLATFORM: ", c32 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, DEPLOYMENT_PLATFORM_id, adjustl(c32));
    loc=234; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    call scan_metline_for_literal("deployment cruise id",scan_meta_result );
    write(c32, '(a32)') scan_meta_result(41:);
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " DEPLOYMENT_CRUISE: ", c32 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, DEPLOYMENT_CRUISE_ID_id, adjustl(c32));
    loc=235; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    call scan_metline_for_literal("profile at deployment",scan_meta_result );
    write(c256, '(a256)') scan_meta_result(41:);
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " DEPLOYMENT_REFERENCE_STATION_ID: ", c256 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, DEPLOYMENT_REFERENCE_STATION_ID_id, adjustl(c256));
    loc=236; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    write(c14, '(a14)') ' '
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " END_MISSION_DATE: ", c14 ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, END_MISSION_DATE_id, adjustl(c14));
    loc=237; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) "END_MISSION_STATUS : ", " " ; call wrtdbg(sub,msg); endif
     ret=nf_put_var_text(ncid, END_MISSION_STATUS_id, " ");
    loc=238; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    nc_array_start2(1) = 1
    nc_array_start2(2) = 1
    nc_array_count2(1) = STRING128_len
    nc_array_count2(2) = n_launch_config_param

!   -------- Code to Debug -------
!   do i_debug = 1, n_launch_config_param
!       write (msg,*) ">>> launch_config name  " // launch_configuration_parameter_name(i_debug)
!       call wrtlog(sub,msg);
!    enddo
!    -------------- End ----------
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " Count of LAUNCH_CONFIG_PARAMETER_NAME : ",n_launch_config_param ; call wrtdbg(sub,msg); endif
     ret=nf_put_vara_text(ncid, LAUNCH_CONFIG_PARAMETER_NAME_id, &
            nc_array_start2, nc_array_count2, launch_configuration_parameter_name)
    loc=239; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    nc_array_start1(1) = 1
    nc_array_count1(1) = n_launch_config_param
    !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " Count of launch_configuration_parameter_value : ",n_launch_config_param ; call wrtdbg(sub,msg); endif
     ret=nf_put_vara_double(ncid, LAUNCH_CONFIG_PARAMETER_VALUE_id, &
                nc_array_start1, nc_array_count1, launch_configuration_parameter_value)
    loc=240; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    !Just in case no Config info has been found ,adding config from ascii meta to config array.
    if(n_config_param == 0)then
       n_config_param = n_launch_config_param
       do i_counter = 1, n_config_param
         configuration_parameter_name(i_counter)= trim(launch_configuration_parameter_name(i_counter))
         configuration_parameter_value(1,i_counter) = launch_configuration_parameter_value(i_counter)
       enddo
       if(ncmeta_noprof)then
           write(msg,'("NOTE: No Config Params except Launch, ncmeta_noprof is T, no qc files were read.")'); call wrtlog(sub,msg);
       else
           write(msg,'("NOTE: No Config Params except Launch, ncmeta_noprof is F, issue a warning.")'); call wrtlog(sub,msg);
           write(wrnmsg,'("WARNING 168 No Config Params found apart from Launch Config! ")'); ! WRN
           call warning(sub,168,wrnmsg); ! WARNING 168 in ncmeta_arrayssubroutine put_ncmeta_data ! WRN
       endif
    endif

    if(N_CONFIG_PARAM_len > 0 .AND. cnt_missions > 0)then
       nc_array_start2(1) = 1
       nc_array_start2(2) = 1
       nc_array_count2(1) = STRING128_len
       nc_array_count2(2) = n_config_param
       !if(DEBUG .AND. DBLEV==71)then; write(msg, *) " Count of CONFIG_PARAMETER_NAME : ",n_config_param ; call wrtdbg(sub,msg); endif
        ret=nf_put_vara_text(ncid, CONFIG_PARAMETER_NAME_id, nc_array_start2, nc_array_count2, configuration_parameter_name)
       loc=241; call cke(sub,ret,loc);
    endif
    !--------------------------------------------------------------------------------------------------------------
    ! -- Convert the 2-D array with config_param_values to a single -D list, so that it can be written to .nc file.
    ! Reason:  .nc file does not like the data to come in as a matrix!
    if(N_CONFIG_PARAM_len > 0 .AND. cnt_missions > 0)then
       nc_array_start2(1) = 1
       nc_array_count2(1) = n_config_param     ! No. of columns

       nc_array_start2(2) = 1                  ! start(2) will set to be cnt_missions.
       nc_array_count2(2) = 1

       ! if(DEBUG)then; write(msg, *) " Count of CONFIG_PARAMETER_VALUE : ",n_config_param ; call wrtdbg(sub,msg); endif
       do idx_row = 1,cnt_missions
          do idx_column = 1,n_config_param
          !    write (msg, '(" in column = " , i2 ", row = ", i2)') idx_column,idx_row ; call wrtlog(sub,msg);
          !    write (msg, *) 'NCMETA >>>>>> configuration_parameter_value = ', &
          !                                configuration_parameter_value(idx_row,idx_column)
          !             call wrtlog("extract_config",msg);
          !     if(DEBUG) write(DBG,'("CMNCMNT ADD BEF idx_row: ",i4," N_MISSIONS_len: ",i4)')idx_row,N_MISSIONS_len
               config_param_value_vector(idx_column) = configuration_parameter_value(idx_row,idx_column)
          !     if(DEBUG) write(DBG,'("CMNCMNT ADD AFT idx_row: ",i4," N_MISSIONS_len: ",i4)')idx_row,N_MISSIONS_len
          !    write (msg, '(">>>>>> n_config_param  = " i2)') n_config_param; call wrtlog("extract_config",msg);
          enddo
          nc_array_start2(2) = idx_row
          ! NOTE NF_UNLIMITED This is the call that increments the UNLIMITED dimension for meta
	  if(DEBUG)write(DBG,'("NF_UNLIM ncmeta nc_array_start2: ",2i5," nc_array_count2: ",2i5)')nc_array_start2, nc_array_count2
           ret=nf_put_vara_double(ncid, CONFIG_PARAMETER_VALUE_id, &
                nc_array_start2, nc_array_count2, config_param_value_vector)
	  if(DBLEV==63 .and. DBLEVEXIT)then;
		write(DBG,'("NF_UNLIM ncmeta close nc file, retest")');        ! DBG LEV63
		call close_ncfile(sub,ncid);                                   ! DBG LEV63
		write(DBG,'("NF_UNLIM ncmeta CLS <>_len: ",i6)')N_MISSIONS_len ! DBG LEV63
	  endif
          loc=242; call cke(sub,ret,loc);
       enddo
    endif
    !--------------------------------------------------------------------------------------------------------------
    if(cnt_missions > 0)then
       nc_array_start1(1) = 1
       nc_array_count1(1) = cnt_missions
        ret=nf_put_vara_int(ncid, CONFIG_MISSION_NUMBER_id, nc_array_start1, nc_array_count1, configuration_mission_numbers)
       loc=243; call cke(sub,ret,loc);
    else
       write(wrnmsg,'("WARNING 169 CONFIG_MISSION_NUMBER is blank in META NC File! ")'); ! WRN
       call warning(sub,169,wrnmsg); ! WARNING 16 in ncmeta_arrayssubroutine def_ncmeta_vars ! WRN
    endif
    !--------------------------------------------------------------------------------------------------------------
    if(cnt_missions > 0)then
       if(DEBUG) write(DBG,'("CMCMNT try to write the config_mission_comments, cnt_missions: ",i3)')cnt_missions
       nc_array_start1(1) = 1
       nc_array_count1(1) = cnt_missions;
       nc_array_start2(1) = 1
       nc_array_count2(1) = 256
       nc_array_start2(2) = 1
       nc_array_count2(2) = cnt_missions
        ret=nf_put_vara_text(ncid, CONFIG_MISSION_COMMENT_id, nc_array_start2, nc_array_count2, config_mission_comments);
       loc=244; call cke(sub,ret,loc);
       if(DEBUG) write(DBG,'("CMCMNT after call to nf_put_vara_text, iret: ",i3)')iret
       if(iret == -57)then
           write(msg,'("NCMETA ERROR netcdf library returns -57, cnt_missions: ",i6)')cnt_missions;
           if(DEBUG)then; call wrtstd(sub,msg); else; call wrtstd(sub,msg); endif
       ! #define NC_EEDGE        (-57)   /* Edge+start exceeds dimension bound */
       endif
    endif
    !--------------------------------------------------------------------------------------------------------------
    ! This section is to write all the sensor variables.
     nc_array_start2(1) = 1; nc_array_start2(2) = 1;
     nc_array_count2(1) = 32; nc_array_count2(2) = N_SENSOR_len;
      ret=nf_put_vara_text(ncid, SENSOR_id, nc_array_start2,nc_array_count2,sensor_name_array);
     loc=245; call cke(sub,ret,loc);

     nc_array_count2(1) = 256;
     !count2(2) = sensor_manufa_count;
      ret=nf_put_vara_text(ncid, SENSOR_MAKER_id, nc_array_start2,nc_array_count2,sensor_maker_array);
     loc=246; call cke(sub,ret,loc);

     !count2(2) = sensor_manufa_count;
      ret=nf_put_vara_text(ncid, SENSOR_MODEL_id, nc_array_start2,nc_array_count2,sensor_model_array);
     loc=247; call cke(sub,ret,loc);

     nc_array_count2(1) = 16;
     !count2(2) = sensor_serno_count;
      ret=nf_put_vara_text(ncid, SENSOR_SERIAL_NO_id, nc_array_start2,nc_array_count2,sensor_ser_no_array);
     loc=248; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------
    ! This section is  to write all the parameter variables.
     nc_array_start2(1) = 1; nc_array_start2(2) = 1;
     nc_array_count2(1) = 64; nc_array_count2(2) = N_PARAM_len;
     
      ret=nf_put_vara_text(ncid, PARAMETERX_id, nc_array_start2,nc_array_count2,parameter_name_array);
     loc=249; call cke(sub,ret,loc);
     
     nc_array_count2(1) = 128;                   
      ret=nf_put_vara_text(ncid, PARAMETER_SENSOR_id, nc_array_start2,nc_array_count2,parameter_sensor_array);
     loc=250; call cke(sub,ret,loc);
     
     nc_array_count2(1) = 32;
      ret=nf_put_vara_text(ncid, PARAMETER_UNITS_id, nc_array_start2,nc_array_count2,parameter_unit_array);
     loc=251; call cke(sub,ret,loc);
      ret=nf_put_vara_text(ncid, PARAMETER_ACCURACY_id, nc_array_start2,nc_array_count2,parameter_accuracy_array);
     loc=252; call cke(sub,ret,loc);
      ret=nf_put_vara_text(ncid, PARAMETER_RESOLUTION_id, nc_array_start2,nc_array_count2,parameter_resolution_array);
     loc=253; call cke(sub,ret,loc);
     
     nc_array_count2(1) = 1024;
      ret=nf_put_vara_text(ncid, PREDEPLOYMENT_CALIB_EQUATION_id, nc_array_start2,nc_array_count2,predep_calib_equation);
     loc=254; call cke(sub,ret,loc);
      ret=nf_put_vara_text(ncid, PREDEPLOYMENT_CALIB_COEFFICIENT_id, nc_array_start2,nc_array_count2,predep_calib_coeff);
     loc=255; call cke(sub,ret,loc);
      ret=nf_put_vara_text(ncid, PREDEPLOYMENT_CALIB_COMMENT_id, nc_array_start2,nc_array_count2,predep_calib_comment);
     loc=256; call cke(sub,ret,loc);
    !--------------------------------------------------------------------------------------------------------------


    !================================================================================
    !if(DEBUG .AND. DBLEV==70)then; write(msg,'(" Leaving put_ncmeta_data ")'); call wrtdbg(sub,msg); endif

end subroutine put_ncmeta_data
!----------------------------------------------------------------------------------------------
!       Subroutine to prepare the netcdf variables for start time and start time QC
!----------------------------------------------------------------------------------------------
subroutine prepare_metanc_startdate(caller,nctype,ncid)

    !Note from Claudia on start time calculation
    !---------------------------------------------
    ! Netcdf definitions for the variables:
    !     START_DATE:long_name   = "Date (UTC) of the first descent of the float" ;
    !     STARTUP_DATE:long_name = "Date (UTC) of the activation of the float" ;
    !
    !  Calculation Logic:
    !  -------------------	    
    !  Is "start_time_pa" available?
    !  if yes,   start up time = launch time from meta file.
    !	         start time = launch time + delay of first down time.
    !	         Is (start time - startup time) large? 
    !                if yes, issue a warning.
    !  if no, is delay of first down time > 20 hours?
    !             if no, start up time = start time from meta file.
    !                    start time = start time + delay of first down time.
    !                    Is (start time - startup time) large? 
    !                    if yes, issue a warning.
    !             if yes, start up time = launch time
    !                    start time = start time + delay of first down time.
    !                    Is (start time - startup time) large? 
    !                    if yes, issue a warning.
    character(*), intent( in ) :: caller        ! who is calling me?
    character(*), intent( in ) :: nctype        ! prof/traj/meta
    integer,      intent( in ) :: ncid          ! file id

    character(len=48)  :: tmp_date_str1 = "", tmp_date_str2 = ""
    integer            :: delay_first_dtime_numeric = 0
    character(len=8)   :: delay_first_dtime_str
    integer*8          :: start_date_bigint , adjusted_start_date_bigint
    integer*8          :: launch_date_bigint , adjusted_launch_date_bigint
    integer            :: start_day,start_mon,start_year,start_hh,start_mm
    real               :: tmp_real
    logical            :: delayfirstdowntime_found,delayfirstdowntime_is_small,start_time_pa_found; 
    character(len=256) :: scan_meta_result
    character(len=15), parameter :: sub = 'prepare_metanc_startdate'
    double precision   :: juld_startdate, juld_startup_date;
    character(len=80)  :: time_diff;
    integer :: dyr,dmo,ddy,dhh,dmn,dsc 
    integer :: ddate(6)
    equivalence (ddate(1),dyr), (ddate(2),dmo),(ddate(3),ddy),(ddate(4),dhh),(ddate(5),dmn),(ddate(6),dsc)
    
    delayfirstdowntime_found =.false.; delayfirstdowntime_is_small = .false.; start_time_pa_found = .false.
    scan_meta_result = "";
    call scan_metline_for_literal("start_time_PA", scan_meta_result );
    if(len_trim(scan_meta_result) > 0)then ! Start time PA is available for the Float.
        tmp_date_str1 = trim(scan_meta_result(41:));
        start_time_pa_found = .true. ;
    endif
    
    scan_meta_result = ""
    call scan_metline_for_literal("start time [dd mm yyyy hh mm (Z)]", scan_meta_result );
    tmp_date_str1 = trim(scan_meta_result(41:));
    if(len_trim(tmp_date_str1) > 0)then
        read(tmp_date_str1,'(i2,x,i2,x,i4,x,i2,x,i2)')start_day,start_mon,start_year,start_hh,start_mm
        if(DEBUG) write(DBG,'("START_DATE @6Z tmp_date_str1: ",a)')trim(tmp_date_str2)
        if(DEBUG) write(DBG,'("START_DATE @7Z int day mon year mm: ",5i5)')&
        start_day,start_mon,start_year,start_hh,start_mm
        if(DEBUG) write(DBG,'("START_DATE @8Z tmp_date_str1: ",a)')trim(tmp_date_str2)
        write(tmp_date_str2, '(i4,i2.2,i2.2,i2.2,i2.2)')start_year,start_mon,start_day,start_hh,start_mm
        if(DEBUG) write(DBG,'("START_DATE @9Z tmp_date_str2: ",a)')trim(tmp_date_str2)
        tmp_date_str2 = trim(tmp_date_str2)//"00"
        if(DEBUG) write(DBG,'("START_DATE @10Z tmp_date_str2: ",a)')trim(tmp_date_str2)
        write(ascii_meta_start_date,'(a14)') tmp_date_str2;
        if(DEBUG) write(DBG,'("START_DATE @11Z ascii_meta_start_date: ",a)')trim(ascii_meta_start_date);      
    endif
    
    delay_first_dtime_numeric=0;;scan_meta_result = "";
    call scan_metline_for_literal("delay of first down time [hours]", scan_meta_result );
    if(DEBUG) write(DBG,'("START_DATE @1Z scan_meta_result: ",a)')trim(scan_meta_result);
    if(len_trim(scan_meta_result) > 0)then ! Delay of first down time is found
       delay_first_dtime_str = scan_meta_result(41:)
       if(INDEX(delay_first_dtime_str,'.')> 0)then
          read( delay_first_dtime_str, * )tmp_real
          delay_first_dtime_numeric = NINT(tmp_real)
       else
          read( delay_first_dtime_str, * )delay_first_dtime_numeric ! Converting the hours from string to integer
       endif
       if(delay_first_dtime_numeric < 20) delayfirstdowntime_is_small = .true.
       if(delay_first_dtime_numeric .ge. 0)then
       	   delayfirstdowntime_found=.true.;
       else
       	   delayfirstdowntime_found=.false.;
	   write(wrnmsg,'("WARNING 170 Invalid delay of first down time (=>20) found in ASCII META File")'); ! WRN
	   call warning(sub,170,wrnmsg); ! WARNING 170 in ncmeta_arrays subroutine put_ncmeta_data ! WRN
       endif
    endif 
      
    if(start_time_pa_found)then ! Start time pa is available in the ASCII Meta file.   
        !Start up time is the launch time.
        startup_date_metanc=ascii_meta_launch_date
         ret=nf_put_var_text(ncid, STARTUP_DATE_id, adjustl(startup_date_metanc));
        loc=232; call cke(sub,ret,loc); 
        
        !Now find out Start time which is launch time + delay of first down time.
        read(ascii_meta_launch_date,'(i14)')launch_date_bigint
        if(delayfirstdowntime_found)then
            call adjust_hours(launch_date_bigint,delay_first_dtime_numeric,adjusted_launch_date_bigint)
            write(start_date_metanc,'(i14)')adjusted_launch_date_bigint;
             ret=nf_put_var_text(ncid, START_DATE_id, adjustl(start_date_metanc));
            loc=230; call cke(sub,ret,loc);
        else
           start_date_metanc=ascii_meta_start_date;
            ret=nf_put_var_text(ncid, START_DATE_id, adjustl(start_date_metanc));
           loc=230; call cke(sub,ret,loc);
        endif
    else  ! No start_time_PA
        if(delayfirstdowntime_found)then
            if(delayfirstdowntime_is_small)then
            	startup_date_metanc=ascii_meta_start_date
                 ret=nf_put_var_text(ncid, STARTUP_DATE_id, adjustl(startup_date_metanc));
                loc=232; call cke(sub,ret,loc ) ; 
                !Now find out Start time which is start time + delay of first down time.
                read(ascii_meta_start_date,'(i14)')start_date_bigint
                call adjust_hours(start_date_bigint,delay_first_dtime_numeric,adjusted_start_date_bigint)
                write(start_date_metanc,'(i14)')adjusted_start_date_bigint;
                 ret=nf_put_var_text(ncid, START_DATE_id, adjustl(start_date_metanc));
                loc=230; call cke(sub,ret,loc);                                         
            else  !Delay of First downtime is a big number
                startup_date_metanc=ascii_meta_launch_date
                 ret=nf_put_var_text(ncid, STARTUP_DATE_id, adjustl(startup_date_metanc));
                loc=232; call cke(sub,ret,loc);
                read(ascii_meta_start_date,'(i14)')start_date_bigint
                call adjust_hours(start_date_bigint,delay_first_dtime_numeric,adjusted_start_date_bigint)
                write(start_date_metanc,'(i14)')adjusted_start_date_bigint;
                 ret=nf_put_var_text(ncid, START_DATE_id, adjustl(start_date_metanc));
                loc=230; call cke(sub,ret,loc);
            endif  !end of delay of down time  is small     
        else   
            start_date_metanc=ascii_meta_start_date;
             ret=nf_put_var_text(ncid, START_DATE_id, adjustl(start_date_metanc));
            loc=230; call cke(sub,ret,loc);
            startup_date_metanc=ascii_meta_start_date
             ret=nf_put_var_text(ncid, STARTUP_DATE_id, adjustl(startup_date_metanc));
            loc=232; call cke(sub,ret,loc ) ;
        endif  !end of delayfirstdowntime_found     
    endif !end of if start_time_PA found               
   
    START_DATE_QC = "0"
    if(check_cdate(start_date_metanc))then
	       START_DATE_QC = "1"
    else
       write(wrnmsg,'("WARNING 174 START_DATE_QC Fails in META NC File! ")'); ! WRN
       call warning(sub,174,wrnmsg); ! WARNING 174 in ncmeta_arrays subroutine put_ncmeta_data ! WRN
       if(DEBUG)then; write(msg,'(" START_DATE_QC Fails in META NC File!")'); call wrtdbg(sub,msg); endif
    endif
     ret=nf_put_var_text(ncid, START_DATE_QC_id, START_DATE_QC);
    loc=231; call cke(sub,ret,loc);

    STARTUP_DATE_QC = "0"
    if(check_cdate(startup_date_metanc))then             
       STARTUP_DATE_QC = "1"
    else
       write(wrnmsg,'("WARNING 175 START_UPDATE_QC Fails in META NC File! ")'); ! WRN
       call warning(sub,175,wrnmsg); ! WARNING 175 A in ncmeta_arrays subroutine put_ncmeta_data ! WRN
       if(DEBUG)then; write(msg,'(" START_UPDATE_QC Fails in META NC File!")'); call wrtdbg(sub,msg); endif
    endif
     ret=nf_put_var_text(ncid, STARTUP_DATE_QC_id, STARTUP_DATE_QC);
    loc=233; call cke(sub,ret,loc);
    
    read(start_date_metanc,'(i4,5i2)')dyr,dmo,ddy,dhh,dmn,dsc;
    juld_startdate=juldays1950(ddate);
    read(startup_date_metanc,'(i4,5i2)')dyr,dmo,ddy,dhh,dmn,dsc;
    juld_startup_date=juldays1950(ddate);
    if(juld_startup_date - juld_startdate > 30)then
        write(time_diff,*)juld_startup_date - juld_startdate;
	write(wrnmsg,'("WARNING 176  Difference between START TIME and STARTUP Time is more than usual! (>30)")'); ! WRN
	call warning(sub,176,wrnmsg); ! WARNING 176 in ncmeta_arrays subroutine prepare_metanc_startdate ! WRN
    endif	    
    
end subroutine prepare_metanc_startdate

  ! To get the sensor model from name, maker and type
  !---------------------------------------------------------------------------------------------------
subroutine get_sensor_model (ascii_name, sensor_manufactr, sensor_type, sensor_model,sensor_name)
    character(len=*), intent(in)      :: ascii_name, sensor_manufactr, sensor_type
    character(len=*), intent(out)     :: sensor_model,sensor_name
    character(len=16),parameter      :: sub = "get_sensor_model";
    integer :: i_sensor_model_tbl_row, i_sensor_model_tbl_column
    character (len=256) :: sensor_type_edited, sensor_manufactr_edited,table_str_uppercase

    sensor_model = ""
    sensor_type_edited = sensor_type
    sensor_manufactr_edited = sensor_manufactr
    call to_upper(sensor_type_edited)
    call to_upper(sensor_manufactr_edited)
    if(trim(sensor_type_edited) == "") sensor_type_edited = "N/A"
    if(trim(sensor_manufactr_edited) == "") sensor_manufactr_edited = "N/A"
    if(sensor_type_edited == 'SBE63 REV G' .OR. &
    sensor_type_edited == 'SBE63 REV J')then
       sensor_model = 'SBE63_OPTODE' 
       sensor_name = 'OPTODE_DOXY'
       return
    endif
    if(INDEX(sensor_type_edited, 'FLUORO') > 0)then
       sensor_model = 'ECO_FLBB'
       sensor_name = 'FLUOROMETER_CHLA'
       return
    endif

    call remove_charset_within_string (sensor_type_edited,' ')
    do i_sensor_model_tbl_row = 1, n_sensor_model_table
       table_str_uppercase=sensor_model_table(i_sensor_model_tbl_row,2)
       call to_upper(table_str_uppercase)
       if(trim(sensor_model_table(i_sensor_model_tbl_row,1)) == trim(ascii_name))then
               !write(msg,*) "DBG:>> sensor model " , sensor_type_edited; call wrtlog(sub,msg);
	       if(INDEX(trim(table_str_uppercase), trim(sensor_manufactr_edited))>0  &
               .AND. INDEX(trim(table_str_uppercase), trim(sensor_type_edited))>0 )then
		  sensor_model = sensor_model_table(i_sensor_model_tbl_row,2)
		  sensor_name = sensor_model_table(i_sensor_model_tbl_row,3)
		  exit; !Breaks the loop
	       endif
       endif
    enddo

    ! In case the model does not have both the maker and the type, try just with maker name.
    if(len_trim(sensor_model) == 0)then
       do i_sensor_model_tbl_row = 1, n_sensor_model_table
          table_str_uppercase=sensor_model_table(i_sensor_model_tbl_row,2)
          call to_upper(table_str_uppercase)
          if(trim(sensor_model_table(i_sensor_model_tbl_row,1)) == trim(ascii_name))then
	       if(INDEX(trim(table_str_uppercase),trim(sensor_type_edited))>0)then
	         !write(msg,*) "DBG:>>sensor model found only with model - " ,&
	         !sensor_model_table(i_sensor_model_tbl_row,2); call wrtlog(sub,msg);
		 sensor_model = sensor_model_table(i_sensor_model_tbl_row,2)
		 sensor_name = sensor_model_table(i_sensor_model_tbl_row,3)
		 exit; !Breaks the loop
	       endif
          endif
       enddo
    endif
    ! In case the model does not have both the maker and the type, try just with maker name.
    if(len_trim(sensor_model) == 0)then
       do i_sensor_model_tbl_row = 1, n_sensor_model_table
          table_str_uppercase=sensor_model_table(i_sensor_model_tbl_row,2)
          call to_upper(table_str_uppercase)
          if(trim(sensor_model_table(i_sensor_model_tbl_row,1)) == trim(ascii_name))then
	       if(INDEX(trim(table_str_uppercase),trim(sensor_manufactr_edited))>0)then
	         !write(msg,*) "DBG:>>sensor model found only with maker- " ,&
	         !sensor_model_table(i_sensor_model_tbl_row,2); call wrtlog(sub,msg);
		 sensor_model = sensor_model_table(i_sensor_model_tbl_row,2)
		 sensor_name = sensor_model_table(i_sensor_model_tbl_row,3)
		 exit; !Breaks the loop
	       endif
          endif
       enddo
    endif

    ! In case no match found from the sensor model table with maker or type
    if(len_trim(sensor_model) == 0)then
	sensor_model ="UNKNOWN"
	sensor_name = ascii_name;
	write(msg,'(a20,": sensor_model =UNKNOWN")')qcfile; write(MSS,'(a)')trim(msg);				! NCMETA MISS_CNFG
	if(DEBUG) write(DBG,'("DBG NCMETA MISS_SNSR: ",a)')trim(msg);							! DBG
    endif
    
     ! In case the model could not be found, atleast get the sensor type.
    if(trim(sensor_model) == "UNKNOWN")then
       do i_sensor_model_tbl_row = 1, n_sensor_model_table
          if(trim(sensor_model_table(i_sensor_model_tbl_row,1)) == trim(ascii_name))then
              sensor_name = sensor_model_table(i_sensor_model_tbl_row,3)
              exit; !Breaks the loop
          endif
       enddo
    endif
    
    !if(DEBUG)then;write(msg,*) "outgoing model name = ", trim(sensor_model);call wrtdbg(sub,msg);endif
end subroutine get_sensor_model

!--------------------------------------------------------------------------------------------------
! To get the parameter info from SENSOR_PARAMS_TABLE for every parameter from ASCII Meta file.
!--------------------------------------------------------------------------------------------------
subroutine get_meta_nc_params(param_str, param_name, param_unit)

character(len=*), intent(in)     :: param_str
character(len=*), intent(out)    :: param_name, param_unit
character(len=16),parameter      :: sub = "get_meta_nc_params";
character(len=100)               :: param_str_uppercase
integer :: i_sensor_params_tbl;
param_name="";param_unit="";

param_str_uppercase=param_str;
call to_upper(param_str_uppercase);
do i_sensor_params_tbl = 1, n_sensor_params_table
    if(trim(param_str_uppercase)== sensor_params_table(i_sensor_params_tbl)(1:60))then 
        param_name=trim(sensor_params_table(i_sensor_params_tbl)(61:120));
        param_unit=trim(sensor_params_table(i_sensor_params_tbl)(121:160));
        exit; !Got what I want, getting out of here!
    endif
enddo

if(i_sensor_params_tbl > n_sensor_params_table)then ! that means - no match was found in the SENSOR_PARAMS_TABLE
    if(trim(param_str)== 'PAL::ACOUSTIC')return; ! This is not recognized by the GDAC yet., so jsut discard for now.
    write(errmsg(1),'("NON-FATAL ERROR 218 in get_meta_nc_params in ncmeta_arrays")');
    write(errmsg(2),'("NON-FATAL ERROR 218 unrecognized parameter ",a," in META NC File!")')param_str;
    call error_report(sub,218,2,errmsg); ! 
    if(DEBUG)then;write(msg,*) "Unrecognized parameter - ",param_str," in META NC File, update SENSOR_PARAMS_TABLE!";
    call wrtdbg(sub,msg);endif  
endif

end subroutine get_meta_nc_params

	!--------------------------------------------------------------------------------------------------
	! To scan content of meta file and to return the number of lines which contains the incoming literal
	!---------------------------------------------------------------------------------------------------
subroutine count_linesof_occurence(in_literal, occurence_count_result )
	! If the flag is ok, run the test

		character(len=20),parameter       :: sub = "count_linesof_ocrnce";
		character(len=*), intent( in )    :: in_literal
		integer , intent(out)             :: occurence_count_result
		integer :: idx
                occurence_count_result = 0
		do idx = 1,nmet
			if(INDEX(metline(idx), in_literal)> 0)then
				occurence_count_result = occurence_count_result + 1
			endif
		enddo

end subroutine count_linesof_occurence

	!---------------------------------------------------------------------------------------------------
	! To replace all occurence of a char in a String
	!---------------------------------------------------------------------------------------------------
subroutine replace_char(str_under_action, char_to_be_replaced, replace_with)
             character(len=*) , intent(in out) :: str_under_action
	     character(len=*) , intent(in) :: char_to_be_replaced, replace_with
	     character(len=12),parameter    :: sub = "replace_char";
	     integer :: indx = 0, index_found
	     do indx = 1,len_trim(str_under_action)
	       index_found = index(str_under_action, char_to_be_replaced)
	       if(index_found >0 .AND. index_found <= len_trim(str_under_action) )then
		   str_under_action(index_found:index_found) = replace_with
	       endif
	     enddo
end subroutine replace_char
 ! ===============================================================================


!--------------------------------------------------------------------------------------
! To scan content of meta file and to return the line which contains the incoming literal
!--------------------------------------------------------------------------------------
subroutine scan_metline_for_literal(in_literal, result_literal) ! If the flag is ok, run the test

                character(len=20),parameter         :: sub = "scan_metline_4_litrl";
                character(len=*), intent( in )      :: in_literal
                character(*), intent( out )         :: result_literal
                integer                             :: idx

                do idx = 1,nmet
                        if(INDEX(metline(idx), in_literal)> 0)then
                                result_literal = metline(idx)
                                return
                        endif
                enddo
                result_literal = ""
                return

end subroutine scan_metline_for_literal


  !--------------------------------------------------------------------------------------------------
  ! To find the netcdf manual name for  sensor maker name from ASCII META file
  !---------------------------------------------------------------------------------------------------
subroutine get_nc_sensor_maker(ascii_name, nc_name)   ! If the flag is ok, run the test

           character(len=16),parameter       :: sub = "get_sensor_maker";
           character(len=*), intent( in )    :: ascii_name
           character(len=*), intent( out )   :: nc_name
           nc_name = trim(ascii_name)
           call to_upper(nc_name)
           if(len_trim(nc_name) == 0)nc_name = "N/A"
           if(INDEX(nc_name, 'FSI')> 0)then
              nc_name = 'FSI'
           elseif(INDEX(nc_name, 'WETLABS')> 0)then
              nc_name = 'WETLABS'
           elseif(INDEX(nc_name, 'APL_UW')> 0)then
              nc_name = 'APL_UW'
           elseif(INDEX(nc_name, 'WASHINGTON')> 0)then
              nc_name = 'APL_UW'
           elseif(INDEX(nc_name, 'UW')> 0)then
              nc_name = 'APL_UW'
           elseif(INDEX(nc_name, 'MBARI')> 0)then
              nc_name = 'MBARI'
           elseif(INDEX(nc_name, 'MONTEREY')> 0)then
              nc_name = 'MBARI'
           elseif(INDEX(nc_name, 'SATLANTIC')> 0)then
              nc_name = 'SATLANTIC'
           elseif(INDEX(nc_name, 'AANDERAA')> 0)then
              nc_name = 'AANDERAA'
           elseif(INDEX(nc_name, 'SBE')> 0)then
              nc_name = 'SBE'
           elseif(INDEX(nc_name, 'SEA-BIRD')> 0)then
              nc_name = 'SBE'
           elseif(INDEX(nc_name, 'SEA BIRD')> 0)then
              nc_name = 'SBE'
           elseif(INDEX(nc_name, 'SEABIRD')> 0)then
              nc_name = 'SBE'
           elseif(INDEX(nc_name, '2900PSIA')> 0)then ! Since some meta file has type as maker
              nc_name = 'DRUCK'
           elseif(INDEX(nc_name, 'AMETEK')> 0)then
              nc_name = 'AMETEK'
           elseif(INDEX(nc_name, 'AMETEC')> 0)then
              nc_name = 'AMETEK'
           elseif(INDEX(nc_name, 'DRUCK')> 0)then
              nc_name = 'DRUCK'
           elseif(INDEX(nc_name, 'KISTLER')> 0)then
              nc_name = 'KISTLER'
           elseif(INDEX(nc_name, 'PAINE')> 0)then
              nc_name = 'PAINE'
           elseif(INDEX(nc_name, 'SEASCAN')> 0)then
              nc_name = 'SEASCAN'
           elseif(INDEX(nc_name, 'MICRON')> 0)then
              nc_name = 'MICRON'
!          Adding new makers according to the updated ARGO Reference Table online. 
           elseif(INDEX(nc_name, 'JAC')> 0)then
               nc_name = 'JAC'
           elseif(INDEX(nc_name, 'TSK')> 0)then
              nc_name = 'TSK'
           elseif(INDEX(nc_name, 'RBR')> 0)then
             nc_name = 'RBR'
           elseif(INDEX(nc_name, 'KELLER')> 0)then
              nc_name = 'KELLER'
           elseif(INDEX(nc_name, 'SEAPOINT')> 0)then
              nc_name = 'SEAPOINT'
           elseif(INDEX(nc_name, 'TURNER_DESIGN')> 0)then
               nc_name = 'TURNER_DESIGN'
           else
               nc_name = "Unknown"
	       write(errmsg(1),'("NON-FATAL ERROR 259 in get_nc_sensor_maker in ncmeta_arrays")');
               write(errmsg(2),'("NON-FATAL ERROR 259 unrecognized Sensor Maker:",a," in META NC File!")')ascii_name;
               call error_report(sub,259,2,errmsg); ! get_nc_sensor_maker in ncmeta_arrays unrecognized Sensor Maker
               if(DEBUG)then;write(msg,*) "Unrecognized sensor maker - ",ascii_name," in META NC File!";
               call wrtdbg(sub,msg);endif
          endif
          !If(DEBUG)then; write(msg,*) "DBG:>>>>> outgoing maker name = ", nc_name; call wrtlog(sub,msg);endif
end subroutine get_nc_sensor_maker


        !--------------------------------------------------------------------------------------------------
        ! To count the occurence of a character in a string
        !---------------------------------------------------------------------------------------------------
subroutine count_given_char_in_string(in_str, in_char_to_count, out_count)

                character(len=20),parameter       :: sub = "cnt_given_chr_in_str";
                character(len=*), intent( in )    :: in_str
                character, intent( in )           :: in_char_to_count
                integer, intent(out)              :: out_count
                integer :: idx

                out_count = 0
                do idx = 1,len(trim(in_str))
                        if(in_str(idx:idx)==in_char_to_count)then
                           out_count = out_count + 1;
                        endif
                enddo
end subroutine count_given_char_in_string

        !--------------------------------------------------------------------------------------------------
        ! To add given number of hours to the given date-time variable.
        ! Basic idea has been adapted from the - sub adjust_minutes
        !---------------------------------------------------------------------------------------------------
subroutine adjust_hours(int_inp,hours,int_out)

        integer*8 , intent( in )  :: int_inp            ! input big integer - the date to modify
        integer*4 , intent( in )  :: hours              ! input small integer - themnutes to add/sub
        integer*8 , intent( out ) :: int_out            ! output big integer - the date after adjustment

        integer :: days(0:13) = (/ 31, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31, 31 /)
                                  ! d   j   f   m   a   m   j   j   a   s   o   n   d   j
        integer :: iyr, imo, idy, ihr, imn, isc         ! year/month/day/hour/min/sec
        character(len=14) :: c14;
        character(len=14), parameter ::  sub = 'adjust_hours'

	integer itst; ! TESTING

         write(c14,'(i14)')int_inp
         read(c14,'(i4,5i2)')iyr, imo, idy, ihr, imn, isc

          ihr = ihr + hours;
          if(ihr > 23)then
                idy = idy + (ihr/24);
                ihr = mod(ihr,24);
           endif
	   if(imo==2 .and. leap(iyr))then; days(2) = 29; endif
           do while (idy > days(imo))
                        idy = idy - days(imo)
                        imo = imo + 1;
                   if(imo > 12)then;
                      iyr = iyr+ (imo/12)
                      imo = mod(imo,12);
                   endif
            enddo
        write(c14,'(i4.4,5i2.2)')iyr, imo, idy, ihr, imn, isc
        ! write(msg, '(a14)') c14 ;  call wrtdbg(sub,msg);
	read(c14,'(i14)')int_out
end subroutine adjust_hours



        !---------------------------------------------------------------------------------------------------
        ! To get the netcdf name of the platform manufacturer for the name from ascii meta file.
        !---------------------------------------------------------------------------------------------------
subroutine get_platform_maker_reftable24(plat_str)

             character(len=*) , intent(in out) :: plat_str
             character(len=20),parameter    :: sub = "get_pltfrm_maker_t24";

             plat_str = trim(plat_str)
             if(index(plat_str,'MARTEC') > 0)then
                    plat_str = 'MARTEC' ; return
             elseif(index(plat_str,'METOCEAN') > 0)then
                    plat_str = "METOCEAN"
             elseif(index(plat_str,'MRV') > 0)then
                    plat_str = "MRV"
             elseif(index(plat_str,'NKE') > 0)then
                    plat_str = "NKE"
             elseif(index(plat_str,'OPTIMARE') > 0)then
                     plat_str = "OPTIMARE"
             elseif(index(plat_str,'SEA-BIRD') > 0 .OR. index(plat_str,'SEABIRD') > 0)then
                     plat_str = "SBE"
             elseif(index(plat_str,'SCRIPPS') > 0 .OR. index(plat_str,'SIO') > 0)then
                     plat_str = "SIO_IDG"
             elseif(index(plat_str,'TSURUMI') > 0)then
                     plat_str = "TSK"
             elseif(index(plat_str,'TELEDYNE') > 0)then
                    plat_str = "TWR"
             elseif(index(plat_str,'WOODS HOLE') > 0 .OR. index(plat_str,'WHOI') > 0)then
                    plat_str = "WHOI"
             elseif(index(plat_str,'WEBB') > 0)then
                    plat_str = "WRC"
             else
                   write(wrnmsg,'("WARNING 177  No corresponding Platform maker found for - ",a)')trim(plat_str); ! WRN
                   call wrtlog(sub,msg);
                   call warning(sub,177,wrnmsg); ! WARNING 177 in subroutine adjust_hours ! WRN
             endif
end subroutine get_platform_maker_reftable24

        !--------------------------------------------------------------------------------------------------
        !To minutes to hours conversion (For ex: while looking for the corresponding config params )
        !---------------------------------------------------------------------------------------------------
        !--------------------------------------------------------------------------------------------------
	!To minutes to hours conversion (For ex: while looking for the corresponding config params )
	!---------------------------------------------------------------------------------------------------
	subroutine convert_config_param(str_from_left, str_from_right, duration_to_convert)
             character(len=*) , intent(in)  :: str_from_left, str_from_right
	     character(len=256) :: tmp_l, tmp_r;
	     character(len=20),parameter    :: sub = "convert_config_param";
	     double precision , intent (in out) :: duration_to_convert
	     tmp_l = str_from_left 
	     tmp_r = str_from_right
	     call to_upper(tmp_l)
	     call to_upper(tmp_r)
	      
	     if((INDEX(tmp_l, 'MINUTE')> 0) .AND. &
	    (INDEX(tmp_r, 'HOUR')> 0 .OR. INDEX(tmp_r, 'HRS')> 0))then
	        if(duration_to_convert >= 0)then	          
		      duration_to_convert = duration_to_convert/dble(60.0);
		    endif
	     elseif((INDEX(tmp_l, 'HOUR')> 0 .OR. INDEX(tmp_l, 'HRS')> 0) .AND. &
	     (INDEX(tmp_r, 'MINUTE')> 0))then
	        if(duration_to_convert >= 0)then
              duration_to_convert = duration_to_convert*dble(60.0);
		    endif
        elseif((INDEX(tmp_l, 'DAY')> 0 ) .AND. &
	     (INDEX(tmp_r, 'HOUR')> 0 .OR. INDEX(tmp_r, 'HRS')> 0))then
		    if(duration_to_convert >= 0)then
              duration_to_convert = duration_to_convert*dble(24.0);
		    endif
	     elseif((INDEX(tmp_l, 'HOUR')> 0 .OR. INDEX(tmp_l, 'HRS')> 0) .AND. &
	     (INDEX(tmp_r, 'DAY')> 0))then
		    if(duration_to_convert >= 0)then
	          duration_to_convert = duration_to_convert/dble(24.0);
		    endif
	     elseif((INDEX(tmp_l, '(SEC)')> 0 .OR. INDEX(tmp_l, '[SEC]')> 0  &
	     .OR. INDEX(tmp_l, 'SECONDS')> 0)  .AND. (INDEX(tmp_r, 'MINUTE')> 0))then
		    if(duration_to_convert >= 0)then
	          duration_to_convert = duration_to_convert/dble(60.0);
		    endif
	     elseif(INDEX(tmp_l, 'MINUTE')> 0 .AND. INDEX(tmp_r, 'SECONDS')> 0)then
		    if(duration_to_convert >= 0)then
              duration_to_convert = duration_to_convert*dble(60.0);
		    endif
	     elseif((INDEX(tmp_l, 'HOUR')> 0 .OR. INDEX(tmp_l, 'HRS')> 0) .AND. INDEX(tmp_r, 'SECONDS')> 0)then
		    if(duration_to_convert >= 0)then
              duration_to_convert = duration_to_convert*dble(3600.0);
		    endif
	     elseif((INDEX(tmp_l, '(SEC)')> 0 .OR. INDEX(tmp_l, '[SEC]')> 0  &
	     .OR. INDEX(tmp_l, 'SECONDS')> 0) .AND. INDEX(tmp_r, 'HOURS')> 0)then
		     if(duration_to_convert >= 0)then
               duration_to_convert = duration_to_convert/dble(3600.0);
		     endif
	    endif
            duration_to_convert=(ANINT(duration_to_convert*1000.0))/1000.0  !Rounds to 3 decimal points,to avoid minor differences while comparing.
	end subroutine convert_config_param

        !---------------------------------------------------------------------------------------------------
        ! To remove a character/character set from a string
        !---------------------------------------------------------------------------------------------------
subroutine remove_charset_within_string (str_under_action, charset_to_be_removed)
           character(len=*), intent(in out)  :: str_under_action
           character(len=*), intent(in)      :: charset_to_be_removed
           integer                           :: old, new, stride
           character(len=20),parameter      :: sub = "remove_chr_in_string";

           old = 1; new = 1
           do
             stride = scan( str_under_action( old : ), charset_to_be_removed )
             if( stride > 0 )then
               str_under_action( new : new+stride-2 ) = str_under_action( old : old+stride-2 )
               old = old+stride
               new = new+stride-1
             else
               str_under_action( new : ) = str_under_action( old : )
               return
             endif
           enddo
end subroutine remove_charset_within_string
!---------------------------------------------------------------------------------------------------
        ! To set the config params from "CONFIG SAMPLING SCHEME (DBAR)" line for meta files.
!---------------------------------------------------------------------------------------------------
subroutine extract_conf_sampling_scheme (cyc_num, match_cnt, config_val_str)
    integer, intent(in)               :: match_cnt,cyc_num;
    character(len=*), intent(in out)  :: config_val_str
    character(len=20),parameter      :: sub = "extr_cnf_smpl_scheme";
    integer :: idx1,idx2,idx3,idx4;
    !OLD need to initialize hcharacter(len=128) :: s1,s2,s3,s4,s5;
    character(len=128) :: s1="",s2="",s3="",s4="",s5="";
    
    if(match_cnt == 1 .and. index(config_val_str,',')>0)then
        idx1=index(config_val_str,',');idx2=index(config_val_str(idx1+1:),',')+idx1;
        idx3=index(config_val_str(idx2+1:),';')+idx2;idx4=index(config_val_str(idx3+1:),',')+idx3;
    	if(idx1==0.or. idx2==0 .or. idx3==0 .or. idx4==0)then
            write(errmsg(1),'("NON-FATAL ERROR 260 in ",a," icyc: ",i4," Invalid config param values! @1")')sub,cyc_num;
	    write(errmsg(1),'("NON-FATAL ERROR 260 in extr_cnf_smpl_scheme in ncmeta_arrays")');
            write(errmsg(2),'("NON-FATAL ERROR 260 test line: ",a)')trim(config_val_str);
            write(errmsg(3),'("NON-FATAL ERROR 260  pqc line: CONFIG SAMPLING SCHEME (DBAR)")');
            call error_report(sub,260,3,errmsg); ! extr_cnf_smpl_scheme in ncmeta_arrays problem with CONFIG SAMPLING SCHEME
        else
            s1=config_val_str(1:idx1-1);s2=config_val_str(idx1+1:idx2-1);
            s3=config_val_str(idx2+1:idx3-1);s4=config_val_str(idx3+1:idx4-1)
            s5=config_val_str(idx4+1:len(config_val_str));
        endif
    endif
    if(match_cnt ==1)config_val_str=s1;
    if(match_cnt ==2)config_val_str=s2;
    if(match_cnt ==3)config_val_str=s3;
    if(match_cnt ==4)config_val_str=s4;
    if(match_cnt ==5)config_val_str=s5;
end subroutine extract_conf_sampling_scheme
! ===========================================================================================================
! ===========================================================================================================
subroutine extract_sensor_meta_data(caller)

     character(*), intent( in ) :: caller        ! who is calling me?
     character(len=256) :: tmp,sensor_tmp_str2 = "", smodel_returned= "" ,sensor_maker_returned= ""
     character(len=256), dimension(META_SENSOR_ARRAY_LEN) :: tmp_meta_sensor_array
     character(len=16), parameter :: sub = 'ext_snsr_meta_dt'
     integer :: idx; !sensor_type_count, sensor_manufa_count, sensor_serno_count
     integer :: i_tmp_meta_sensor = 0 , tmp_meta_sensor_len = 0
     character(len=256) :: tmp_sensor_name = ""
     character(len=40)  :: sensorline_first40char,sname_returned;
   
     ! Note: The following loop is to extract sensor related info from meta file
     ! and to store it in a temporary array before preparing
     ! the nectcfd variable
     !--------------------------------------------------------------------------
     count_multiple_param_sensor=0;
     calib_date_oxy="";calib_date_temp="";calib_date_press="";calib_date_conduct="";
     !all sensor info lines are saved into a temporary array - tmp_meta_sensor_array
     do idx = 1,nmet
        if((INDEX(metline(idx), 'sensor')> 0).AND.   &
	(INDEX(metline(idx), 'type')> 0 .OR.        &
	INDEX(metline(idx), 'model')> 0 .OR.        &
	INDEX(metline(idx), 'manufact')>  0 .OR. &
	INDEX(metline(idx), 'serial number')> 0 ))then
	   i_tmp_meta_sensor = i_tmp_meta_sensor + 1
           tmp_meta_sensor_array(i_tmp_meta_sensor) = metline(idx)
        elseif(index(trim(metline(idx)), 'parameter ') > 0 .and. index(trim(metline(idx)), ' list ') <= 0)then
	   i_tmp_meta_sensor = i_tmp_meta_sensor + 1
           tmp_meta_sensor_array(i_tmp_meta_sensor) = metline(idx)
        elseif(index(trim(metline(idx)), 'calibration date') > 0)then
           if(index(trim(metline(idx)), 'oxygen') > 0 .OR. index(trim(metline(idx)), 'O2') > 0)calib_date_oxy=trim(metline(idx));
           if(index(trim(metline(idx)), 'pressure') > 0)calib_date_press=trim(metline(idx));
           if(index(trim(metline(idx)), 'temperature') > 0)calib_date_temp=trim(metline(idx));
           if(index(trim(metline(idx)), 'conductivity') > 0)calib_date_conduct=trim(metline(idx));
	endif
     enddo	
     
     tmp_meta_sensor_len = i_tmp_meta_sensor
     sensor_type_count = 0 ; sensor_manufa_count = 0; sensor_serno_count = 0; !Ideally all these should be EQUAL at any time.
     
     if(tmp_meta_sensor_len > 0)then
	! loop to prepare sensor model
        do i_tmp_meta_sensor = 1,tmp_meta_sensor_len
           sensorline_first40char=tmp_meta_sensor_array(i_tmp_meta_sensor)(1:40);
	   if(INDEX(sensorline_first40char, 'sensor') > 0 .AND. &
	   (INDEX(sensorline_first40char, ' type')> 0 .OR. &
	   INDEX(sensorline_first40char, 'model')> 0))then
	       sensor_type_count = sensor_type_count + 1
	       param_info_metline(sensor_type_count) = trim(tmp_meta_sensor_array(i_tmp_meta_sensor));
	       ascii_sensor_type(sensor_type_count)=param_info_metline(sensor_type_count)&
	       (1:index(param_info_metline(sensor_type_count)(1:40),"sensor")-1);
	       if(index(ascii_sensor_type(sensor_type_count),"(2)")>0)ascii_sensor_type(sensor_type_count)=&
	       ascii_sensor_type(sensor_type_count)(1:index(ascii_sensor_type(sensor_type_count),"2")-1);
               if(index(ascii_sensor_type(sensor_type_count),"(3)")>0)ascii_sensor_type(sensor_type_count)=&
               ascii_sensor_type(sensor_type_count)(1:index(ascii_sensor_type(sensor_type_count),"3")-1);
               if(index(ascii_sensor_type(sensor_type_count),"(")>0)ascii_sensor_type(sensor_type_count)=&
               ascii_sensor_type(sensor_type_count)(1:index(ascii_sensor_type(sensor_type_count),"(")-1);
               call to_upper(ascii_sensor_type(sensor_type_count));
	       tmp = tmp_meta_sensor_array(i_tmp_meta_sensor); tmp = tmp(41:)
	       call to_upper(tmp) ;
	       !call remove_charset_within_string (tmp,' ')
	       sensor_model_array(sensor_type_count) = tmp
	   endif
	   
	   if(index(trim(tmp_meta_sensor_array(i_tmp_meta_sensor)), 'parameter ') > 0  .and. & ! since one sensor can have multiple parameters.
	   index(trim(tmp_meta_sensor_array(i_tmp_meta_sensor)), ' list ') <= 0)then
	      count_multiple_param_sensor=count_multiple_param_sensor+1;
	      write(info_multiple_param_sensor(count_multiple_param_sensor,1),*)sensor_type_count;
	      info_multiple_param_sensor(count_multiple_param_sensor,2)=tmp_meta_sensor_array(i_tmp_meta_sensor);
	   endif
	enddo   

	sensorline_first40char="";
	do i_tmp_meta_sensor = 1,tmp_meta_sensor_len
               sensorline_first40char=tmp_meta_sensor_array(i_tmp_meta_sensor)(1:40);
	       if(INDEX(sensorline_first40char, 'sensor') > 0 .AND. &
	       INDEX(sensorline_first40char, 'manufact')> 0 )then
	          sensor_manufa_count = sensor_manufa_count + 1
	          tmp = tmp_meta_sensor_array(i_tmp_meta_sensor); call to_upper(tmp)
	          if(tmp(41:) == 'N/A')then
	            sensor_maker_array(sensor_manufa_count) = "Unknown"
                    write(msg,'(a20,": sensor_maker = Unknown")')qcfile; write(MSS,'(a)')trim(msg);	! NCMETA MISS_CNFG
                    if(DEBUG) write(DBG,'("DBG NCMETA MISS_SNSR: ",a)')trim(msg);			! DBG
                    ! The below warning is not issued - it is taken care of the in sensor_modis line in the MISS_SNSR file
		  else
		     call get_nc_sensor_maker(tmp(41:), sensor_maker_returned)
		     sensor_maker_array(sensor_manufa_count) = sensor_maker_returned
		  endif
		  smodel_returned = "";sname_returned="";
		  !Using the type, model, manufacturer from ascii meta file, retrieving 
		  !the sensor model and sensor name for Meta NC file.
		  call get_sensor_model(ascii_sensor_type(sensor_manufa_count),sensor_maker_array(sensor_manufa_count)&
		  , sensor_model_array(sensor_manufa_count) ,smodel_returned,sname_returned);
		  sensor_name_array(sensor_manufa_count)=sname_returned;
		  sensor_model_array(sensor_manufa_count)=smodel_returned;
	       endif
	enddo
	
	N_SENSOR_len=sensor_type_count; ! Assign the number of sensors as the N_SENSORS dimesnion in Meta.NC file.
	
        ! loop to prepare sensor and sensor serial no:
        sensorline_first40char="";
	do i_tmp_meta_sensor = 1,tmp_meta_sensor_len
               sensorline_first40char=tmp_meta_sensor_array(i_tmp_meta_sensor)(1:40);
	       if(INDEX(sensorline_first40char, 'sensor') > 0 .AND. &
	       INDEX(sensorline_first40char, 'serial number')> 0 )then
	          sensor_serno_count = sensor_serno_count + 1
		  tmp = tmp_meta_sensor_array(i_tmp_meta_sensor);
		  call to_upper(tmp)
		  if(tmp(41:) == 'N/A')then
		      sensor_ser_no_array(sensor_serno_count) = "Unknown"
                      write(msg,'(a20,": sensor_seiial_no = Unknown")')qcfile; write(MSS,'(a)')trim(msg);	! NCMETA MISS_CNFG
                      if(DEBUG) write(DBG,'("DBG NCMETA MISS_SNSR: ",a)')trim(msg);			! DBG
                      ! The below warning is not issued - it is taken care of the in sensor_modis line in the MISS_SNSR file
		  else
		          sensor_ser_no_array(sensor_serno_count) = tmp(41:)
		  endif
	       endif
        enddo
        
     endif
     
     if(sensor_type_count /= sensor_manufa_count .or. sensor_manufa_count /= sensor_serno_count .or. &
         sensor_type_count /= sensor_serno_count)then
         if(sensor_type_count > sensor_manufa_count)then
             write(errmsg(1),'("NON-FATAL ERROR 217 Incomplete sensor manufacturer info in ASCII Meta file! ")');
         elseif(sensor_type_count > sensor_serno_count)then
     	     write(errmsg(1),'("NON-FATAL ERROR 217 Incomplete sensor serial number info in ASCII Meta file! ")'); 
         else
     	     write(errmsg(1),'("NON-FATAL ERROR 217 Incomplete sensor info in ASCII Meta file! ")');    
         endif
         write(errmsg(2),'("NON-FATAL ERROR 217 Meta.NC file may not be complete!")');
         call error_report(sub,217,2,errmsg); ! 
         if(DEBUG)then;write(msg,*) "Incomplete sensor info in ASCII Meta file!";
         call wrtdbg(sub,msg);endif
     endif
     
end subroutine extract_sensor_meta_data 
! ===========================================================================================================
subroutine extract_parameter_meta_data(caller)

     character(*), intent( in ) :: caller        ! who is calling me?
     character(len=1024)   :: tmp;
     character(len=18), parameter :: sub = 'extract_sensor_meta_data';
     integer :: idx, parameter_count
     character(len=1024):: c1024
     integer :: j,sensor_idx,ii;
     logical :: multi_params_added;
     integer :: pres_coeff_indx, temp_coeff_indx, sal_coeff_indx, conduct_coeff_indx, oxy_coeff_indx
     integer :: nitrate_coeff_indx, ph_coeff_indx, pal_coeff_indx, cdom_coeff_indx, fluoro_coeff_indx
     character(len=40):: metline_first40char,param_category;
     character(len=2),parameter :: CONNECTOR_STR="::";
     
     parameter_count = 0 ;
 
     do idx = 1,N_SENSOR_len 
          param_name="n/a";param_unit="n/a";! reset the variables.
          multi_params_added=.false.;
          param_category=param_info_metline(idx)(1:index(param_info_metline(idx)(1:40),"sensor")-1);
          if(index(param_category,"(2)")>0)param_category=param_category(1:index(param_category,"(2)")-1);
          if(index(param_category,"(3)")>0)param_category=param_category(1:index(param_category,"(3)")-1);
          if(index(param_category,"(")>0)param_category=param_category(1:index(param_category,"(")-1);
          call to_upper(param_category);
          if(trim(param_category) == 'CONDUCTIVITY' .and. hasCNDC_meta)then 
              ! Bcz some conductivity sensors measure both salinity and conductiviyt as parameters 
              ! (those ascii meta files have the line "calib eq 1 for conductivity" Ex: float 2155
              param_sensor=trim(param_category)//CONNECTOR_STR//trim(sensor_name_array(idx))&
              //CONNECTOR_STR//"CNDC";
              call get_meta_nc_params(param_sensor,param_name,param_unit);
              if(len_trim(param_name)==0)cycle;
              parameter_count = parameter_count + 1;
              parameter_sensor_array(parameter_count)=sensor_name_array(idx);
              if(index(param_info_metline(idx)(1:40),"(2)")>0)param_name=trim(param_name)//"2";
              if(index(param_info_metline(idx)(1:40),"(3)")>0)param_name=trim(param_name)//"3";
              parameter_name_array(parameter_count)=param_name;
              parameter_unit_array(parameter_count)=param_unit;
          elseif(trim(param_category) == 'OXYGEN' .and. trim(sensor_name_array(idx))=='OPTODE_DOXY')then
              ! Bcz OPTODE_DOXY oxygen sensors measure has 2 parameters DOXY and TEMP_DOXY
              param_sensor=trim(param_category)//CONNECTOR_STR//trim(sensor_name_array(idx))&
              //CONNECTOR_STR//"DOXY";	  
              call get_meta_nc_params(param_sensor,param_name,param_unit);
              if(len_trim(param_name)==0)cycle;
              parameter_count = parameter_count + 1;
              parameter_sensor_array(parameter_count)=sensor_name_array(idx);
              if(index(param_info_metline(idx)(1:40),"(2)")>0)param_name=trim(param_name)//"2";
              if(index(param_info_metline(idx)(1:40),"(3)")>0)param_name=trim(param_name)//"3";
              parameter_name_array(parameter_count)=param_name;
              parameter_unit_array(parameter_count)=param_unit;
              param_sensor=trim(param_category)//CONNECTOR_STR//trim(sensor_name_array(idx))&
              //CONNECTOR_STR//"TEMP_DOXY";	  
              call get_meta_nc_params(param_sensor,param_name,param_unit);
              if(len_trim(param_name)==0)cycle;
              parameter_count = parameter_count + 1;
              parameter_sensor_array(parameter_count)=sensor_name_array(idx);
              if(index(param_info_metline(idx)(1:40),"(2)")>0)param_name=trim(param_name)//"2";
              if(index(param_info_metline(idx)(1:40),"(3)")>0)param_name=trim(param_name)//"3";
              parameter_name_array(parameter_count)=param_name;
              parameter_unit_array(parameter_count)=param_unit;
              multi_params_added=.true.; 
          endif
          
          do j=1,count_multiple_param_sensor
              param_sensor=sensor_name_array(idx);
              read(info_multiple_param_sensor(j,1),*)sensor_idx;
              if(sensor_idx == idx)then! it has multiple parameters.
                  if(index(trim(info_multiple_param_sensor(j,2)),"CDOM")>0)then  
                      !Because the ascii meta file says FLUOROMETER_CHLA for all fluoro sensors.
                      param_sensor=trim(param_category)//CONNECTOR_STR//"FLUOROMETER_CDOM"//CONNECTOR_STR//&
                      trim(adjustl(info_multiple_param_sensor(j,2)(40:)))
                  else
                      param_sensor=trim(param_category)//CONNECTOR_STR//trim((sensor_name_array(idx)))//CONNECTOR_STR//&
                      trim(adjustl(info_multiple_param_sensor(j,2)(40:)))	  
                  endif
                  
                  call get_meta_nc_params(param_sensor,param_name,param_unit);
                  if(len_trim(param_name)==0)cycle;
                  parameter_count = parameter_count + 1;
                  if(index(trim(param_sensor),"FLUOROMETER_CDOM")>0 )then
                      parameter_sensor_array(parameter_count)="FLUOROMETER_CDOM";	  
                  else
                      parameter_sensor_array(parameter_count)=sensor_name_array(idx);
                  endif
                  if(index(param_info_metline(idx)(1:40),"(2)")>0)param_name=trim(param_name)//"2";
                  if(index(param_info_metline(idx)(1:40),"(3)")>0)param_name=trim(param_name)//"3";
                  parameter_name_array(parameter_count)=param_name;
                  parameter_unit_array(parameter_count)=param_unit;
                  multi_params_added=.true.;                  
              endif
          enddo
          if(.not. multi_params_added)then
              param_sensor=trim(param_category)//CONNECTOR_STR//trim(adjustl(sensor_name_array(idx)));
              call get_meta_nc_params(param_sensor,param_name,param_unit);
              if(len_trim(param_name)==0)cycle;
              parameter_count = parameter_count + 1;
              parameter_sensor_array(parameter_count)=sensor_name_array(idx);
              if(index(param_info_metline(idx)(1:40),"(2)")>0)param_name=trim(param_name)//"2";
              if(index(param_info_metline(idx)(1:40),"(3)")>0)param_name=trim(param_name)//"3";
              parameter_name_array(parameter_count)=param_name;
              parameter_unit_array(parameter_count)=param_unit;
          endif
     enddo
     nparam_meta=parameter_count; ! assign the final count of parameters as the n_param for Meta NC file.
    
     if (nparam_meta > MAXPAR_META) then
	      write(errmsg(1),'("NON-FATAL ERROR 234 ",a," prof: ",a20," nparam_meta exceeds MAXPAR_META")')sub,profile;
	      write(errmsg(2),'("NON-FATAL ERROR 234 nparam_meta (",i2," exceeds MAXPAR_META (",i2,")")') &
	      nparam_meta,MAXPAR_META;
	      call error_report(sub,234,2,errmsg); ! read_meta nparam_meta exceeds mas
	      make_ncmeta = .false.;
	      write(HEDF,'("FILMSG ",a," wrt l: ",i1," var/filmsg: ")')'NCMETA',3
	      filmsg(LMETA:LMETA) = 'N'; write(msg,1000)HEDF,'N',filmsg,'CAnnot make requested ncmet file'; call wrterr(sub,msg);
	      1000 format(a,a1,a9,x,a);
	      write(msg,'("NON-FATAL ERROR 276 occurred, nparam_meta exceeds mas")'); call wrterr(sub,msg);
	      write(msg,'("NON-FATAL ERROR 276 occurred, set make_ncmeta to False")'); call wrterr(sub,msg);
     elseif(nparam_meta == 0)then
          write(errmsg(1),'("NON-FATAL ERROR 217 No Parameters were found for Meta.NC File! ")');
          write(errmsg(2),'("NON-FATAL ERROR 217 ASCII Meta File needs review!")');
          call error_report(sub,217,2,errmsg); ! 
          if(DEBUG)then;write(msg,*) "No Parameters were found for Meta.NC File! ";
          call wrtdbg(sub,msg);endif 
     endif
     !filling calib comment field.
     do idx = 1,nparam_meta
         if(trim(parameter_name_array(idx)) == 'DOXY')then
             if(len_trim(calib_date_oxy)>0 .and. index(calib_date_oxy(1:40), '2')<=0)predep_calib_comment(idx)=calib_date_oxy;
         elseif(trim(parameter_name_array(idx)) == 'PRES')then
             if(len_trim(calib_date_press)>0 .and. index(calib_date_press(1:40), '2')<=0)predep_calib_comment(idx)=calib_date_press;
         elseif(trim(parameter_name_array(idx)) == 'TEMP')then
             if(len_trim(calib_date_temp)>0 .and. index(calib_date_temp(1:40), '2')<=0)predep_calib_comment(idx)=calib_date_temp;  
         elseif(trim(parameter_sensor_array(idx)) == 'CTD_CNDC')then
             if(len_trim(calib_date_conduct)>0 .and. index(calib_date_conduct(1:40), '2')<=0)&
             predep_calib_comment(idx)=calib_date_conduct;
         endif    
     enddo
end subroutine extract_parameter_meta_data     
! ===========================================================================================================
end module ncmeta_arrays
! ===========================================================================================================
