! ========================================================================================================== ! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20200325 VH      Extensive changes to START OF TRANS APEX/ARGOS tests and the error messages
! 20191127 VH     Fix loop that calculates JULD to also use when JULD_QC = 8.
! 20191126 VH     Change in determination of JULD. Three categories: no MCLines (ASC only) MCLines/ASC
!                 and MCLines/DSC. For no-MCLines/ASC select from (in order) TIME_ASCENT_END, TIME_TRANS_START,
!                 first good posiiton, emergency PROFILE_UPLOAD_TIME.  For MCLines/ASC select fromm MC100,
!                 first non-IR position (if good), MC700, second non-IR position (if good), MC599, MC590.
!                 For MCLines/DSC select from MC100, MC099, MC090.
! 20190521 VH     If JULD is from the position block, add the position number to the "JULD FROM" line
!                 Also omit the "JULD FROM part of the variable juld_from, (in qc02) and add writing 
!                 it in write_qcprof.
! 20180619 VH     Streamline chpoces (not complete yet)
!                 START OF TRANSMISSION (YYY ... is now being recognized
!                 The prof_JULD is calculated at the end (some rare instances did not calculate it)
!                 If no other time is available, use PROFILE_UPLOAD_TIME for porf_JULD
! 20180515 VH     Reorganization in progress, not complete yet.
! 20180417 VH     General WARNING revisions.
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0

! ===========================================================================================================
! Included
!
! logical function check_cdate(cdate)
! logical function check_date(iyr,imo,idy,ihh,imn,isc)
! subroutine cdate_check(cdate,io)
! subroutine date_check(iyr,imo,idy,ihh,imn,io)
! ===========================================================================================================
! Notes
!
! This will check dates in the heaer and position blocks.
! If an impossible date is found, it will be reported as a return status of -1
!
! HOWEVER, if the format of a date is bad (garbled), this will be reported AND
! QC WILL EXIT IMMEDIATELY.
!
! Function return values:
!  0 = PASS - Success, all positions passed
!  1 = FAIL - Failure, some or all positions failed
! -1 = NOTD - cannot be determined
!

integer function qc02_impossible_date(caller)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller

character(len=14), parameter :: sub = 'QC02'
character(len=21) ,parameter :: QCTEST='Impossible Date'
integer :: result;

character(len=40) :: from; ! The 'value' part of the JULD FROM line
character(len=14) :: testdate
character(len=19) :: c19, t19
character(len=14) :: c14
logical :: gooddate;
double precision  :: JULD_TRANS_START, JULD_TRANS_EPOCH
double precision  :: JULD_FIRST_999L, JULD_FIRST_POSN, JULD_LAST_POSN
character(len=14) :: dat1 = '99999999999999', dat2 = '99999999999999'; 
character(len=8)  :: dat1l, dat2l
character(len=6)  :: dat1r, dat2r
character(len=15) :: tvar1 = 'n/a', tvar2 = 'n/a';
double precision  :: juld1 = 0.0,  juld2 = 0.0,  juldd = 0.0; 	! juld values to compare
double precision  :: diff1 = 0.0, diff2 = 0.0
double precision  :: unadj_juld = 0.0; ! unadjusted juld - before UP_TIME_DAYS used
double precision  :: adjust = 0.0 ! the UP_TIME_DAYS adjustment
character(len=70) :: cjuld1 = 'default1', cjuld2 = 'default2'	! character representations of compared values
character(len=5)  :: cdiff1 = 'NOTD' , cdiff2 = 'NOTD';		! What is the result?
character(len=22) :: early					! Heaer of dbg msgs about early START OF TRANS
integer :: iyr,imo,idy,ihh,imn,isc, idate(6)
integer :: min_adj;				! Minute adjustment for yyyymmddhhmmss dates
integer :: n703;				! If the first mc703 line is bad, use the second
character(len=SZC) :: tst_date_time
integer*8         :: int_adj_time,  int_max_test, int_test, int_diff_mc589, int_diff_mc590;
integer*8         :: int_date_time, int_unadj_time
integer*8         :: int_703_1, int_700, int_703_2, int_last_589, int_last_590;
logical           :: got_703_1, got_700, got_703_2, got_last_589, got_last_590, need_703_1, need_703_2;
character(len=14) :: chr_703_1, chr_700, chr_703_2, chr_last_589, chr_last_590;
character(len=14) :: chr_date_time, chr_test
character(len=24) :: ch24
integer :: iyea
integer :: iyear
integer :: rio		! return status from date_check
character(len=2) :: TF; character(len=16) :: profout;
character(len=SZC) :: msq1, msq2
equivalence (idate(1),iyr), (idate(2),imo),(idate(3),idy),(idate(4),ihh),(idate(5),imn),(idate(6),isc)
integer :: num_pos_lines;
integer :: bpos; ! Beginning of positions in the position block (1 if no 999 line, 2 if a i999 line is present)
integer :: mxtflag; ! Maximum local test flag
character(len=8) :: YYYYMMDD; character(len=6) :: HHMMSS;

logical :: DATE_TIME_PROFILE_found

character(len=19) :: TIME_FIRST_999L = FILL19CH,      TIME_FIRST_POSN = FILL19CH,      TIME_LAST_POSN        = FILL19CH;
character(len=1)  :: TIME_FIRST_999L_QC = '9',        TIME_FIRST_POSN_QC = '9',        TIME_LAST_POSN_QC     = '9';
integer           :: TIME_FIRST_999L_ipos = 0,        TIME_FIRST_POSN_ipos = 0,        TIME_LAST_POSN_ipos   = 0;
logical           :: TIME_FIRST_999L_found = .false., TIME_FIRST_POSN_found = .false., TIME_LAST_POSN_found = .false.;
logical           :: SOT_Early_needed = .false., SOT_Early_performed = .false.;
integer           :: SOT_wrn_num
character(len=40) :: SOT_endmsg;
character(len=4)  :: SOT_Early_result = 'NOTD';   ! NOTD - not determined; PASS SOT is NOT Early; FAIL = Sot IS Early
character(len=80) :: tmsg;
character(len=60) :: SOT_Early_body  = 'Currently undefined';
character(len=60) :: SOT_Early_cmnt  = 'The SOT Early test is only used for ARGOS/APEX floats.'
integer :: si1 = 0, si2 = 0;      ! Which variable is used for the sot test
character(len=SZC) :: sot;
character(len=14) :: chr14;
character(len=16) :: lefty; ! left-justify profile name
character(len=12) :: jftmp  ! temporary message for juld_from
! ===========================================================================================================
! xecutable statements

1000 format(i4,5i2.2); ! read/write format for 14ch dates

result = NOTD ! cannot be determined

! Before we don anything, check what the currently values are for the time inputs

! MC codes to look for: MC100 MC600 MC700 MC090 MC099 MC589 MC590 - Note that MC703 is only available in the pos block

!===============================================================================================================
! OLD Not a smart decicion to do this, so eliminate it and define the JUld, even if unused in processing
! OLD ! Note that all of the times have been qc'd in read_phy, as the qc value was needed for the date search
! OLD ! If there is no profile, the JULD is not used for anything else. Therefore, if there is not profile,
! OLD ! The JULD will NOT be populated
! OLD if(nlev == 0)then;
! OLD     juld_from = 'UNDEFINED NO PROFILE'; prof_DATE_TIME_PROFILE = FILL14CH; prof_JULD_QC = '9'; prof_JULD = prof_JULD_LOCATION;
! OLD     qc02_impossible_date = NOTD;
! OLD     return;
! OLD endif;
if(DEBUG) write(DBG,*)' -------------------------------------------------------------------------------------'
if(hasMC)then
    if(phy_descending)then
	if(DEBUG) write(DBG,7000)  TIME_MC090_found,         TIME_MC090,TIME_MC090_QC,'TIME_MC090';
	if(DEBUG) write(DBG,7000)  TIME_MC099_found,         TIME_MC099,TIME_MC099_QC,'TIME_MC099';
	if(DEBUG) write(DBG,7000)  TIME_MC100_found,         TIME_MC100,TIME_MC100_QC,'TIME_MC100';
	go to 35; ! GO TO 35 - Jump down to DATE_TIME_PROFILE selection for hasMC and DSC
    else
	if(DEBUG) write(DBG,7000)  TIME_MC600_found,         TIME_MC600,TIME_MC600_QC,'TIME_MC600';
	if(DEBUG) write(DBG,7000)  TIME_MC700_found,         TIME_MC700,TIME_MC700_QC,'TIME_MC700';
	if(DEBUG) write(DBG,7000)  TIME_MC589_found,         TIME_MC589,TIME_MC589_QC,'TIME_MC589';
	if(DEBUG) write(DBG,7000)  TIME_MC590_found,         TIME_MC590,TIME_MC590_QC,'TIME_MC590';
	go to 25; ! GO TO 25 - Jump down to DATE_TIME_PROFILE selection for hasMC and ASC
    endif
else
    if(phy_descending)then
	! this is a problem - we do not have a priority list of choices for JULD
	if(DEBUG) write(DBG,'("DBG QC02 Entry the profile is DSC but does NOT have MCLines")')
	write(errmsg(1),'("NON-FATAL ERROR 242 The profile is DESCENDING, but there are NO MCLines")');
	write(errmsg(2),'("NON-FATAL ERROR 242 We do not have a set of priorities for choice of JULD")');
	write(errmsg(3),'("NON-FATAL ERROR 242 We wll just return, without choosing a JULD")');
	call error_report(sub,242,3,errmsg); ! Profile is DSC, but MCLines are no present
	return
    endif
    if(DEBUG) write(DBG,7001)  TIME_ASCENT_END_found,    TIME_ASCENT_END_QC,TIME_ASCENT_END,'TIME_ASCENT_END';
    if(DEBUG) write(DBG,7001) TIME_TRANS_START_found,    TIME_TRANS_START_QC,TIME_TRANS_START,'TIME_TRANS_START';
    if(DEBUG) write(DBG,7001) TIME_TRANS_EPOCH_found,    TIME_TRANS_EPOCH_QC,TIME_TRANS_EPOCH,'TIME_TRANS_EPOCH';
    if(DEBUG) write(DBG,7001) PROFILE_UPLOAD_TIME_found, PROFILE_UPLOAD_TIME_QC, PROFILE_UPLOAD_TIME,'PROFILE_UPLOAD_TIME';
endif

7000 format("DBG BADGPSFIX QC02 bef foud: ",l2," TIME: ",a14," QC: ",a1," type: ",a);
7001 format("DBG BADGPSFIX QC02 bef foud: ",l2," TIME QC: ",a1," TIME: ",a," TYPE: ",a)

!===============================================================================================================
! This section deals with non-mclines variables

write(msg,'("QC02 Find JULD for non-MCLines profile.")')
write(msg,1001)  TIME_ASCENT_END_found,    TIME_ASCENT_END_QC,TIME_ASCENT_END,'TIME_ASCENT_END'; call wrtdbg(sub,msg);
write(msg,1001) TIME_TRANS_START_found,    TIME_TRANS_START_QC,TIME_TRANS_START,'TIME_TRANS_START'; call wrtdbg(sub,msg);
write(msg,1001) TIME_TRANS_EPOCH_found,    TIME_TRANS_EPOCH_QC,TIME_TRANS_EPOCH,'TIME_TRANS_EPOCH'; call wrtdbg(sub,msg);
write(msg,1001) PROFILE_UPLOAD_TIME_found, PROFILE_UPLOAD_TIME_QC, PROFILE_UPLOAD_TIME,'PROFILE_UPLOAD_TIME'; call wrtdbg(sub,msg);
write(msg,'("QC02 The positions will be analyzed if needed.")'); call wrtdbg(sub,msg);
1001 format("QC02 foud?: ",l2," TIME QC: ",a1," TIME: ",a," TYPE: ",a)
!===============================================================================================================
! Revused sectuin to identify bad START OF TRANSMISISON, which also affects IJMTE ASCENT END for ARGOS/APEX floats


if((trans_type == "ARGOS") .and. (inst_type(1:4) == "APEX") .and. (TIME_TRANS_START_QC == '1'))then
    ! oct2014 - add a test for BAD TIME_TRANS_START - is it too early?

    read(TIME_TRANS_START,1000)idate;
    JULD_TRANS_START = juldays1950(idate);
    if(DEBUG) write(DBG,*)'SOT TST @1 TRANS_START: ',TIME_TRANS_START,' TRANS_START: ',JULD_TRANS_START

    SOT_Early_needed = .true.;

    if(npos == 0)then;
	SOT_Early_cmnt = 'There are no lines in the osition block.';
	go to 15; ! GO TO 15 - There are no lines in the position block.
    endif

    1201 format("SOT too Early? ",a);		! One character string 
    1202 format("SOT too Early? ",a60,a60)	! Two character strings
    1101 format("TIME FIRST 999L found:",l1," qc:",a1," ipos: ",i3," time: ",a14);
    1102 format("TIME FIRST POSN found:",l1," qc:",a1," ipos: ",i3," time: ",a14);
    1103 format("TIME LAST  POSN found:",l1," qc:",a1," ipos: ",i3," time: ",a14);
    1104 format(a," cmnt: ",a)
    ! Find what you need from the position bock
    ! Is the first line a position line or an 999 line?
    bpos = 1; ! Beginning of position lines in the position block
    if(this_lat(1) > 91.0)then
	TIME_FIRST_999L_found = .true.; TIME_FIRST_999L_ipos = 1;
	TIME_FIRST_999L = this_date_time(1); TIME_FIRST_999L_QC = this_date_time_qc(1);
	write(SOT_Early_body,1101)TIME_FIRST_999L_found,TIME_FIRST_999L_QC,TIME_FIRST_999L_ipos,TIME_FIRST_999L
	write(SOT_Early_cmnt,'("Thre first position IS a 999 line")');
	bpos = 2; ! The posiitons start at line 2
    endif
    if((npos == 1) .and. (bpos == 2))then
	SOT_Early_cmnt = 'There is only one line in the osition block. a 999 line';
	go to 11; ! GO TO 11 Jump analysis of positions - olnly 1 line, a 999 line
    endif
    do ipos = bpos,npos
 	if(this_lat(ipos) > 91.0)then;
 	    write(msg,'("SOT too Early? The second 999 line is present")'); call wrtdbg(sub,msg);
 	    go to 11; ! GO TO 11 - if the positions have been analyzed for the SOT_Early tests
 	endif
	if(this_date_time_qc(ipos) == '1')then
	    if(.not. TIME_FIRST_POSN_found)then
		TIME_FIRST_POSN_ipos = ipos;		TIME_FIRST_POSN_found = .true.;		! Found a good first position
		TIME_FIRST_POSN = this_date_time(ipos);;	TIME_FIRST_POSN_QC = this_date_time_qc(ipos);
		write(SOT_Early_body,1102)TIME_FIRST_POSN_found,TIME_FIRST_POSN_QC,TIME_FIRST_POSN_ipos,TIME_FIRST_POSN
		write(SOT_Early_cmnt,'("The first good position is set to ",i3)')ipos;
		write(msg,1202)SOT_Early_body,SOT_Early_cmnt; call wrtdbg(sub,msg);
	   else
		TIME_LAST_POSN_ipos = ipos;		TIME_LAST_POSN_found = .true.;		! Found a good last
		TIME_LAST_POSN = this_date_time(ipos);	TIME_LAST_POSN_QC = this_date_time_qc(ipos);
		if(DEBUG)then
		    write(SOT_Early_body,1103)TIME_LAST_POSN_found,TIME_LAST_POSN_QC,TIME_LAST_POSN_ipos,TIME_LAST_POSN
		    write(SOT_Early_cmnt,'("The last good position is reset to ",i3)')ipos;
		    write(msg,1202)SOT_Early_body,SOT_Early_cmnt; call wrtdbg(sub,msg);
		endif
	    endif
	endif
    enddo
    11 continue; ! GO TO 11 - the positions have been analyzed for the SOT_Early tests
    ! Which SOT parameters are available?

    ! -----------------------------------------------------------------------------------------------------------
    ! Now, do the actual test for SOT too Early?

    if((.not. UP_TIME_found) .or. (UP_TIME_DAYS < 0))then;
	juld1 = 0.0; cjuld1 = 'juld1 cannot be determined, UP_TIME_DAYS is not available';
	tmsg='UP_TIME_DAYS is not available, so the juld1 test cannot be performed';
	write(msg,1201)trim(tmsg); call wrtlog(sub,msg);

        write(errmsg(1),'("NON-FATAL ERROR 250 in qc02, The float is APEX/ARGOS, but the up time is not available.")');
        write(errmsg(2),'("NON-FATAL ERROR 250 in qc02, It is either fill value or missing from the meta file")');
        write(errmsg(3),'("NON-FATAL ERROR 250 in qc02, The first Early (DIFF1) START OF TRANSMISION test will not be done.")');
	call error_report(sub,250,3,errmsg);

	go to 12; ! GO TO 12 - juld1 cannot be used, UP_TIME_DYAS not available
    endif
    tvar1 = 'No suitable time available for DIFF1.'
    ! NOTEST TIME_LAST_POSN_QC = '4'; ! DBG TEST SOTE
    ! NOTEST TIME_FIRST_POSN_QC = '4'; ! DBG TEST SOTE
    ! NOTEST TIME_FIRST_999L_QC = '4'; ! DBG TEST SOTE
    if(TIME_LAST_POSN_QC == '1')then
	write(msg,*)'DBG TEST SOTE Use date #1 (TIME_LAST_POSN) for DIFF1'; call wrtlog(sub,msg);
	read(TIME_LAST_POSN,1000)idate;
	tvar1 = 'TIME_LAST_POSN';
	JULD_LAST_POSN = juldays1950(idate);
	write(dat1,'(i4,5i2.2)')idate; dat1l = dat1(1:8); dat1r = dat1(9:14);
	unadj_juld = JULD_LAST_POSN;
	! 26Mar2015 - UP_TIME_DAYS determined to be too strict - myltiply by 1.5, and see how it goes
	adjust = DBLE(UP_TIME_DAYS) * 1.5;
	juld1 = unadj_juld - adjust;
	! OLD juld1 = JULD_LAST_POSN - (UP_TIME_DAYS*1.5);
	cjuld1 = 'JULD_LAST_POSN - (UP_TIME_DAYS*1.5)'; si1 = 1;
    elseif(TIME_FIRST_POSN_QC == '1')then
	write(msg,*)'DBG TEST SOTE Use date #2 (TIME_FIRST_POSN) for DIFF1'; call wrtlog(sub,msg);
	! OLD read(TIME_FIRST_POSN,'(i4,x,i2,x,i2,x,i2,x,i2,x,i2)')iyr,imo,idy,ihh,imn,isc;ST_POSN_QC
	read(TIME_FIRST_POSN,1000)idate;
	tvar1 = 'TIME_FIRST_POSN';
	write(dat1,'(i4,5i2.2)')idate;  dat1l = dat1(1:8); dat1r = dat1(9:14);
	JULD_FIRST_POSN = juldays1950(idate);
	unadj_juld = JULD_FIRST_POSN;
	adjust = DBLE(UP_TIME_DAYS);
	juld1 = unadj_juld - adjust;
	! OLD juld1 = JULD_FIRST_POSN - UP_TIME_DAYS;
	cjuld1 = 'JULD_FIRST_POSN - UP_TIME_DAYS;'; si1 = 2;
    elseif(TIME_FIRST_999L_QC == '1')then
	write(msg,*)'DBG TEST SOTE Use date #3 (TIME_FIRST_999L) for DIFF1'; call wrtlog(sub,msg);
	read(TIME_FIRST_999L,1000)idate;
	tvar1 = 'TIME_FIRST_999L';
	write(dat1,'(i4,5i2.2)')idate;  dat1l = dat1(1:8); dat1r = dat1(9:14);
	JULD_FIRST_999L = juldays1950(idate);
	juld1 = unadj_juld - adjust;
	! OLD juld1 = JULD_FIRST_999L - UP_TIME_DAYS;
	unadj_juld = JULD_FIRST_999L;
	adjust = DBLE(UP_TIME_DAYS);
	cjuld1 = 'JULD_FIRST_999L - UP_TIME_DAYS;'; si1 = 3;
    else
	write(msg,*)'DBG TEST SOTE Use date #0 for DIFF1, cannot be determined'; call wrtlog(sub,msg);
	si1 = 0;
    endif
    12 continue; ! GO TO 12 - juld1 cannot be used, UP_TIME_DYAS not available
    tvar2 = 'No suitable time available for DIFF2.'
    ! NOTEST TIME_FIRST_POSN_QC = '4'; ! DBG TEST SOTE
    ! NOTEST TIME_FIRST_999L_QC= '4'; ! DBG TEST SOTE
    if(TIME_FIRST_POSN_QC == '1')then
	write(msg,*)'DBG TEST SOTE Use date #1 for DIFF2'; call wrtlog(sub,msg);
	read(TIME_FIRST_POSN,1000)idate;
	tvar2 = 'TIME_FIRST_POSN';
	JULD_FIRST_POSN = juldays1950(idate);
	write(dat2,'(i4,5i2.2)')idate; dat2l = dat2(1:8); dat2r = dat2(9:14);
	juld2 = JULD_FIRST_POSN;
	cjuld2 = 'JULD_FIRST_POSN'; si2 = 1;
    elseif(TIME_FIRST_999L_QC == '1')then
	write(msg,*)'DBG TEST SOTE Use date #2 for DIFF2'; call wrtlog(sub,msg);
	read(TIME_FIRST_999L,1000)idate
	tvar2 = 'TIME_FIRST_999L';
	JULD_FIRST_999L = juldays1950(idate);
	write(dat2,'(i4,5i2.2)')idate; dat2l = dat2(1:8); dat2r = dat2(9:14);
	juld2 = JULD_FIRST_999L;
	cjuld2 = 'JULD_FIRST_999L'; si2 = 2;
   else
	write(msg,*)'DBG TEST SOTE Use date #0 for DIFF2, cannot be determined'; call wrtlog(sub,msg);
	si2 = 0;
    endif
    write(tmsg,'("to test: ",f16.9,"      JULD_TRANS_START")')JULD_TRANS_START;
    write(msg,1201)trim(tmsg); call wrtdbg(sub,msg);
    if(juld1 > 0.0)then
	diff1 = JULD_TRANS_START - juld1;
	if(JULD_TRANS_START >= juld1)then
	    cdiff1 = "PASS";	! The SOT is NOT too Early
	    if(DBLEV==2) write(DBG,*)'DBG TEST DIFF1 PASS diff1: ',diff1
	else
	    cdiff1 = "FAIL";	! The SOT IS too Early
	    if(DBLEV==2) write(DBG,*)'DBG TEST DIFF1 FAIL diff1: ',diff1
	endif
	SOT_Early_performed = .true.;
    endif
    if(juld2 > 0.0)then
	diff2 = JULD_TRANS_START - juld2;
	if(JULD_TRANS_START <= juld2) then
	    cdiff2 = "PASS";
	    if(DBLEV==2) write(DBG,*)'DBG TEST DIFF2 PASS diff2: ',diff2
	else
	    cdiff2 = "FAIL";
	    if(DBLEV==2) write(DBG,*)'DBG TEST DIFF2 FAIL diff2: ',diff2
	endif
	SOT_Early_performed = .true.;
    endif
    lefty = adjustl(profile);
    write(sot,3121)lefty,JULD_TRANS_START,si1,cdiff1,juld1,diff1,si2,cdiff2,juld2,diff2
    3121 format("SOT ",a16," JULD",f16.9," 1.",i1,x,a4,f16.9,' D1 ',f16.9," 2.",i1,x,a4,f16.9," D2 ",f16.9)
    write(tmsg,'("  juld2: ",f16.9,x,a4,x,a43)')juld2,cdiff2,cjuld2; write(msg,1201)trim(tmsg); call wrtdbg(sub,msg);
    
    if(cdiff1 == "PASS" .and. cdiff2 == "PASS")then
        write(msg,'("SOT too Early diff test(s) PASSED, the START OF TRANSMISSION is NOT too early")');
        call wrttrc(sub,msg);
    elseif(cdiff1 == "FAIL" .or. cdiff2 == "FAIL")then
	! Mark the START OF TRANSMISSION as BAD
	TIME_TRANS_START_QC = '4';
	if(cdiff1 == "FAIL")then
	    write(errmsg(1),'("NON-FATAL ERROR 251 START OF TRANSMISSION fails, set JULD_QC=4 due to DIFF1 test")')
	    write(errmsg(2),'("NON-FATAL ERROR 251 For DIFF1, START OF TRANSMISSION is compared to ",a)')cjuld1
	    lefty = adjustl(tvar1);
	    write(errmsg(3),'("NON-FATAL ERROR 251 ",a16,a8,x,a6," JULD: ",f20.14" UP_TIME_DAYS: ",f7.2)')  &
		lefty,dat1l,dat1r,unadj_juld,UP_TIME_DAYS
	    ! BAD? write(errmsg(4),'("NON-FATAL ERROR 251 UP_TIME_DAYS: ",f6.3
	    write(errmsg(4),'("NON-FATAL ERROR 251                      Adjustment JULD: ",f20.14)')adjust
	    write(errmsg(5),'("NON-FATAL ERROR 251                      Comparison JULD: ",f20.14)')juld1
	    write(errmsg(6),'("NON-FATAL ERROR 251 START OF TRANS: ",a8,x,a6," JULD: ",f20.14)') &
		TIME_TRANS_START(1:8),TIME_TRANS_START(9:14),JULD_TRANS_START 
	    write(errmsg(7),'("NON-FATAL ERROR 251                            diff JULD: ",f20.14)')diff1
	    write(errmsg(8),'("NON-FATAL ERROR 251 START OF TRANSMISSION is earlier than the comparison JULD")')
	    call error_report(sub,251,8,errmsg); ! ERR
	endif
	if(cdiff2 == "FAIL")then
	    write(errmsg(1),'("NON-FATAL ERROR 252 START OF TRANSMISSION fails, set JULD_QC=4 due to DIFF2 test")')
	    write(errmsg(2),'("NON-FATAL ERROR 252 For DIFF2, START OF TRANSMISSION is compared to ",a)')cjuld2
	    lefty = adjustl(tvar2);
	    write(errmsg(3),'("NON-FATAL ERROR 252 ",a16,a8,x,a6," JULD: ",f20.14)')lefty,dat2l,dat2r,juld2
	    write(errmsg(4),'("NON-FATAL ERROR 251 START OF TRANS: ",a8,x,a6," JULD: ",f20.14)') &
		TIME_TRANS_START(1:8),TIME_TRANS_START(9:14),JULD_TRANS_START
	    write(errmsg(5),'("NON-FATAL ERROR 252                            diff JULD: ",f20.14)')diff2
	    write(errmsg(6),'("NON-FATAL ERROR 252  START OF TRANSMISSION is later than the omparison JULD")')
	    call error_report(sub,252,6,errmsg); ! ERR
	endif
    else
	if(cdiff1 == "NNOTD" .and. cdiff2 == "NNOTD")then
	write(wrnmsg,'("The DIFF1 and DIFF2 tests could not be performed: ",a)')trim(SOT_Early_cmnt)
	call warning(sub,150,wrnmsg); ! WRN DIFF1 & DIFF2 test NOTD
    elseif(cdiff1 == "NNOTD")then
	write(wrnmsg,'("The DIFF1 test could not be performed: ",a)')trim(SOT_Early_cmnt) 
	call warning(sub,151,wrnmsg); ! WRN DIFF1 NOTD
    elseif(cdiff2 == "NNOTD")then
	write(wrnmsg,'("The DIFF2 test could not be performed: ",a)')trim(SOT_Early_cmnt)
	call warning(sub,152,wrnmsg); ! WRN DIFF2 NOTD
    endif
    endif
     ! -----------------------------------------------------------------------------------------------------------

    15 continue; !  ! GO TO 15 - There are no lines in the position block.

    if(SOT_Early_performed)then
	write(msg,1105)trim(SOT_Early_cmnt); call wrtlog(sub,msg)
	1105 format("SOT too Early The ARGOS/APEX test was performed - ",a);
	if(cdiff1 == 'NOTD')then
	    write(msg,'("The DIFF2 test was performed, buit not the DIFF1.")');
	elseif(cdiff2 == 'NOTD')then
	    write(msg,'("The DIFF1 test was performed, buit not the DIFF2.")');
	else
	    write(msg,'("Both the DIFF1 and DIFF2 tests were performed.")');
	endif
    else
	write(msg,1106)trim(SOT_Early_cmnt); call wrtlog(sub,msg)
	1106 format("SOT too Early The ARGOS/APEX test cannot be done: ",a);
    endif
    call wrtlog(sub,msg);
endif; ! End of SOT_Early search
if(DBLEV== 2 .and. DBLEVEXIT)then;
    write(msg,'("DBG QC02 DBLEV=2 diff1: ",a4," diff2: :"a4)')cdiff1,cdiff2;
    lefty = adjustl(profile); write(msg,'(a,3x,a20)')trim(msg), lefty
    lefty = adjustl(inst_type); write(msg,'(a,3x,a20)')trim(msg), lefty
    write(DBG,'(a)')trim(msg);								! DBG LEV2
    if(index(msg,"FAIL") > 0)then
	write(ERR,'(a)')trim(msg);                                                          ! DBG LEV2
    endif
    write(LOG,'(a)')trim(msg);                                                          ! DBG LEV2
    if(DBLEV== 2 .and. DBLEVEXIT)then; write(DBG,*)' DBG QC02  BADSOTE TEST EXIT'; call exit(0); endif
endif


if(TIME_ASCENT_END_found .and. TIME_ASCENT_END_QC=='1')then
    juld_from = 'TIME ASCENT END'; prof_DATE_TIME_PROFILE = TIME_ASCENT_END; prof_JULD_QC = TIME_ASCENT_END_QC;
    write(msg,1002)prof_JULD_QC,prof_DATE_TIME_PROFILE,juld_from; call wrtdbg(sub,msg);
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE in TIME_ASCENT_END
endif
if(TIME_TRANS_START_found .and. TIME_TRANS_START_QC =='1')then
    juld_from = 'START OF TRANSMISSION'; prof_DATE_TIME_PROFILE = TIME_TRANS_START; ; prof_JULD_QC = TIME_TRANS_START_QC;
    write(msg,1002)prof_JULD_QC,prof_DATE_TIME_PROFILE,juld_from; call wrtdbg(sub,msg);
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE in TIME_TRANS_START
endif

! You have NOT found a good JULD for non-hasMC floats, take the first good position
3001 format("POSITION ",i1)
3002 format("POSITION ",i2)
do ipos=1,npos
    if(this_date_time_qc(ipos) == '1')then
	if(ipos < 10)then; write(jftmp,3001)ipos; else; write(jftmp,3002)ipos; endif
	write(juld_from,'(a)')trim(jftmp); prof_DATE_TIME_PROFILE = this_date_time(ipos); prof_JULD_QC = '1';
	go to 16; ! GO TO 16 - we have found a good JULD in the position block
    endif
enddo
! It looks like we did not get a good JULD from the position block
if(prof_JULD_QC/='1' .and. PROFILE_UPLOAD_TIME_QC == '1')then
    prof_DATE_TIME_PROFILE = PROFILE_UPLOAD_TIME; prof_JULD_QC = '1';
    juld_from = 'PROFILE_UPLOAD_TIME'
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE in PROFILE_UPLOAD_TIME
endif
16 continue; ! GO TO 16 - loop out of the do if a good position/date was found
! OLD if(prof_JULD_QC == '1')then
if((prof_JULD_QC == '1') .or. (prof_JULD_QC == '8'))then
    write(msg,1002)prof_JULD_QC,prof_DATE_TIME_PROFILE,juld_from;
else
    write(msg,*)prof_JULD_QC,prof_DATE_TIME_PROFILE,juld_from;
endif
call wrtlog(sub,msg); if(DEBUG) write(DBG,'(a)')trim(msg);

go to 99; ! GO TO 99 - Jump down to 'found a JULD maybe in the non-hasMC floats


! ========================================================================================================
! MC codes to look for: MC100 MC600 MC700 MC090 MC099 MC589 MC590
! Do we have MC lines"

! ========================================================================================================
if(badgpsfix_pos_found)then
    write(msg,'("QC02 BADGPS There are badgps dates in the position block")'); call wrtlog(sub,msg);
    call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
    do ipos=1,npos
        write(msg,'("QC02 BADGPS ipos: ",i4," fix? ",l2," date/time: ",a)')ipos,badgpsfix_pos(ipos),this_date_time(ipos)
	call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
    enddo
endif
write(msg,'("QC02 BADGPS ascnt qc: ",a1," fix? ",l2," date/time: ",a)')TIME_ASCENT_END_QC, badgpsfix_ascnt,TIME_ASCENT_END
call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
write(msg,'("QC02 BADGPS strns qc: ",a1," fix? ",l2," date/time: ",a)')TIME_TRANS_START_QC,badgpsfix_strns,TIME_TRANS_START
call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
write(msg,'("QC02 BADGPS MC100 qc: ",a1," fix? ",l2," date/time: ",a)')TIME_MC100_QC,badgpsfix_MC100,TIME_MC100
call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
write(msg,'("QC02 BADGPS MC600 qc: ",a1," fix? ",l2," date/time: ",a)')TIME_MC600_QC,badgpsfix_MC600,TIME_MC600
call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
write(msg,'("QC02 BADGPS MC700 qc: ",a1," fix? ",l2," date/time: ",a)')TIME_MC700_QC,badgpsfix_MC700,TIME_MC700
call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
write(msg,'("QC02 BADGPS MC090 qc: ",a1," fix? ",l2," date/time: ",a)')TIME_MC090_QC,badgpsfix_MC090,TIME_MC090
call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
write(msg,'("QC02 BADGPS MC099 qc: ",a1," fix? ",l2," date/time: ",a)')TIME_MC099_QC,badgpsfix_MC099,TIME_MC099
call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
write(msg,'("QC02 BADGPS MC589 qc: ",a1," fix? ",l2," date/time: ",a)')TIME_MC589_QC,badgpsfix_MC589,TIME_MC589
call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)
write(msg,'("QC02 BADGPS MC590 qc: ",a1," fix? ",l2," date/time: ",a)')TIME_MC590_QC,badgpsfix_MC590,TIME_MC590
call wrtlog(sub,msg); if(DEBUG) write(DBG,'("DBG ",a)')trim(msg)

do i=1,num_pos_lines
    if(badgpsfix_pos(i))then;
	! write(DBG,'("DBG BADGPS posn ",i3," ia badgps date, not pother checking")')i;
	go to 22; ! GO TO 22 - ??? wrong logic for badgps position
     endif
    if(this_year(i) == 9999)then
	pos_dateqc(i) = '9';	! year is fill value
	if(DEBUG)then; write(msg,'("QC02 ipos: ",i2," set pos_dateqc to 9, this_year=9999")')i; call wrtdbg(sub,msg); endif
    else
	if(DEBUG)then
	    write(msg,'("GMT QC02 calls check_date @3 for posn ",i2," vals: ",i4,5i2.2)')i, &
		this_year(i),this_mon(i),this_day(i),this_hh(i),this_mm(i),this_sc(i);
	    call wrttrc(sub,msg);
	endif
	gooddate = check_date(this_year(i),this_mon(i),this_day(i),this_hh(i),this_mm(i),this_sc(i));
	if(gooddate)then
	    pos_dateqc(i) = '1';
	    if(DEBUG)then; write(msg,'("QC02 ipos: ",i2," set pos_dateqc to 1, check_date T")')i; call wrtdbg(sub,msg); endif
	else
	    pos_dateqc(i) = '4';
	    write(msg,'("GMT QC02 found a BAD DATE @3 for posn: ",i2," time: ",i4,5i2.2)')i, &
		this_year(i),this_mon(i),this_day(i),this_hh(i),this_mm(i),this_sc(i); call wrttrc(sub,msg);
	    if(DEBUG)then; write(msg,'("QC02 ipos: ",i2," set pos_dateqc to 4, check_date F")')i; call wrtdbg(sub,msg); endif
	endif
    endif
    if(DBLEV== 2)then;
	write(testdate,'(i4.4,5i2.2)')this_year(i),this_mon(i),this_day(i),this_hh(i),this_mm(i),this_sc(i)
	write(msg,'("LEV02 TIM set for posn ",i2,"   DATEQC = ",a1," for ",a)')i,pos_dateqc(i) ,testdate;
	call wrtdbg(sub,msg);
    endif
    22 continue; ! GO TO 22 - skip to here if it is a fix_badgps date
enddo
if(DBLEV== 2)then;
    write(DBG,'("==============================================================================================")'); ! DBG LEV02
    do i=1,num_pos_lines
	write(DBG,3003)i,this_lat(i),this_lon(i),pos_dateqc(i),adjustl(this_date_time(i)); ! DBG LEV02;              ! DBG LEV02
	3003 format("DBG QC02 JULD POS #",I2," lat/lon: ",2f8.1," QC: ",a1," date/time: ",a," POSITION BLOCK")
    enddo
    write(DBG,'("==============================================================================================")'); ! DBG LEV02
endif



! 24Oct2014 add test for bad/early START OF TRANSMISSION - compare to TIME_TRANS_EPOCH & TIME_FIRST_POSN
! using the nominal up time from the meta fle
3004 format("DBG JULD COMPARE: ",3x,a1,2x,a14,2x,f20.12,"JULD_TRANS_START");
3005 format("DBG JULD COMPARE: ",3x,a1,2x,a14,2x,f20.12,"JULD_TRANS_EPOCH");
3006 format("DBG JULD COMPARE: ",3x,a1,2x,a14,2x,f20.12,"JULD_FIRST_POSN");

35 continue; ! GO TO 35 - Jump here to DATE_TIME_PROFILE selection for hasMC and DSC

prof_DATE_TIME_PROFILE = FILL14CH; prof_JULD = FILLDBLE; prof_JULD_QC = '9'; juld_from = 'N/A';;
if((TIME_MC100_found) .and. (TIME_MC100_QC == '1'))then;
    juld_from = 'MC100'; prof_DATE_TIME_PROFILE = TIME_MC100; prof_JULD_QC = TIME_MC100_QC; 
    write(msg,'("QC02 JULD is MC100 for hasMC/dsc")'); call wrttrc(sub,msg); if(DEBUG) write(DBG,'(a)')trim(msg);
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/dsc in MC100
elseif((TIME_MC099_found ) .and. (TIME_MC099_QC == '1'))then;
    juld_from = 'MC099'; prof_DATE_TIME_PROFILE = TIME_MC099; prof_JULD_QC = TIME_MC099_QC;
    write(msg,'("QC02 JULD is MC099 for hasMC/dsc")'); call wrttrc(sub,msg); if(DEBUG) write(DBG,'(a)')trim(msg);
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/dsc in MC099
elseif(TIME_MC090_found .and. (TIME_MC090_QC == '1'))then;
    juld_from = 'MC090'; prof_DATE_TIME_PROFILE = TIME_MC090; prof_JULD_QC = TIME_MC090_QC;
    write(msg,'("QC02 JULD is MC090 for hasMC/dsc")'); call wrttrc(sub,msg); if(DEBUG) write(DBG,'(a)')trim(msg);
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/dsc in MC090
endif

go to 99; ! GO TO 99 - you have NOT found a good DATE_TIME_PROFILE for hasMC/dsc



25 continue; ! GO TO 25 - Jump here to DATE_TIME_PROFILE selection for hasMC and ASC

prof_DATE_TIME_PROFILE = FILL14CH; prof_JULD = FILLDBLE; prof_JULD_QC = '9'; juld_from = 'N/A';;

! JG: 1) Use MC600 if not fillvalue, JULD_QC='1' (above)
! JG, 2) Use the first GPS MC703 if present, minus 10 minutes, JULD_QC='8'
! JG: 3) Use the MC700 if not fillvalue, minus 15 minutes, JULD_QC='8'
! JG: 4) Use the second GPS MC703 if present, minus 20 minutes, JULD_QC='8'
! JG: 5) Use the latest MC589/90 measurement code timestamp, JULD_QC='8'

if((TIME_MC600_found) .and. (TIME_MC600_QC == '1'))then;
    juld_from = 'MC600'; prof_DATE_TIME_PROFILE = TIME_MC600; prof_JULD_QC = TIME_MC600_QC;
    write(msg,'("QC02 JULD is MC600 for hasMC/dsc")'); call wrttrc(sub,msg); if(DEBUG) write(DBG,'(a)')trim(msg);
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/asc in MC600
endif
if(DEBUG) write(DBG,*)'Before test, vars are TIME_MC703_1_found: ',TIME_MC703_1_found,' and TIME_MC703_1_QC: ',TIME_MC703_1_QC
if(TIME_MC703_1_found .and. (TIME_MC703_1_QC == '1'))then;
    juld_from = 'MC703_1'; prof_DATE_TIME_PROFILE = TIME_MC703_1; prof_JULD_QC = '8';
    read(TIME_MC703_1,'(i4,5i2)')iyr,imo,idy,ihh,imn,isc; prof_JULD = juldays1950(idate);
    write(msg,'("QC02 JULD is MC703_1 for hasMC/dsc")'); call wrttrc(sub,msg); if(DEBUG) write(DBG,'(a)')trim(msg);
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/asc in MC703.1
endif
if((TIME_MC700_found) .and. (TIME_MC700_QC == '1'))then;
    juld_from = 'MC700'; prof_DATE_TIME_PROFILE = TIME_MC700; prof_JULD_QC = '8';
    read(TIME_MC700,'(i4,5i2)')iyr,imo,idy,ihh,imn,isc; prof_JULD = juldays1950(idate);
    write(msg,'("QC02 JULD is MC700 for hasMC/dsc")'); call wrttrc(sub,msg); if(DEBUG) write(DBG,'(a)')trim(msg);
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/asc in MC700
endif
if(TIME_MC703_2_found .and. (TIME_MC703_2_QC == '1'))then;
    juld_from = 'MC703_2'; prof_DATE_TIME_PROFILE = TIME_MC703_2; prof_JULD_QC = '8';
    read(TIME_MC703_2,'(i4,5i2)')iyr,imo,idy,ihh,imn,isc; prof_JULD = juldays1950(idate);
    write(msg,'("QC02 JULD is MC703_2 for hasMC/dsc")'); call wrttrc(sub,msg); if(DEBUG) write(DBG,'(a)')trim(msg);
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/asc in MC703.2
endif
! If we reached this section, then MC600,  MC700 and both MC703 values were missing or not good.
! Note that in both MC589 and MC590, there may be multiple entries of either.
! Note also that (in my six test cases, the lines can be interpsersed - that is, there may be MC589
! appear AFTER the first MV590. In all six test cases, the last MC590 was later than any MC589,
! but I will test this nonetheless.
! Note that only good values are retained, if the value is found, the QC is '1'

if(TIME_MC589_found .and. TIME_MC590_found)then
    if(DEBUG) write(DBG,*)'TIME_MC589: ',TIME_MC589
    if(DEBUG) write(DBG,*)'TIME_MC590: ',TIME_MC590
    if(TIME_MC589 > TIME_MC590)then
	juld_from = 'MC589'; prof_DATE_TIME_PROFILE = TIME_MC589; prof_JULD_QC = TIME_MC589_QC;
    else
	juld_from = 'MC590'; prof_DATE_TIME_PROFILE = TIME_MC590; prof_JULD_QC = TIME_MC590_QC;
    endif
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/asc in MC589 or MC590
elseif(TIME_MC589_found)then
    juld_from = 'MC589'; prof_DATE_TIME_PROFILE = TIME_MC589; prof_JULD_QC = TIME_MC589_QC;
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/asc in MC589
elseif(TIME_MC590_found)then
    juld_from = 'MC590'; prof_DATE_TIME_PROFILE = TIME_MC590; prof_JULD_QC = TIME_MC590_QC;
    go to 99; ! GO TO 99 - you have found a good DATE_TIME_PROFILE for hasMC/asc in MC590
endif

99 continue; ! GO TO 99 if you have found a good DATE_TIME_PROFILE, or have been unable to find one

if(DEBUG) write(DBG,'("DBG QC02 Did we find a good date? : ",a14," QC: ",a)')prof_DATE_TIME_PROFILE, prof_JULD_QC
if(DEBUG) write(DBG,'("DBG QC02 What is juld_from: ",a)')juld_from
! i At this point, if we did not get a good date yet, we ain't gonna get onee

read(prof_DATE_TIME_PROFILE,'(i4)')iyr; 
if(DEBUG) write(DBG,*)' iyr: ',iyr
if(iyr < 9000)then;
    read(prof_DATE_TIME_PROFILE,'(i4,5i2)')idate; prof_JULD = juldays1950(idate); 
    if(DEBUG) write(DBG,*)'JULD: ',prof_JULD
endif

write(msg,1002)prof_JULD_QC,prof_DATE_TIME_PROFILE,juld_from; call wrtdbg(sub,msg); if(DEBUG) write(DBG,'(a)')trim(msg);
 1002 format("QC02 JULD choice: qc: ",a1," I/D: ",a14," from: ",a); ! format for selection of JULD



! OLD if(prof_JULD_QC == '1')then
if((prof_JULD_QC == '1') .or. (prof_JULD_QC == '8'))then
    read(prof_DATE_TIME_PROFILE,'(i4,5i2)')idate; JULD_year=idate(1); JULD_mon=idate(2); JULD_day=idate(3);
    prof_JULD = juldays1950(idate);
else
    read(prof_DATE_TIME_PROFILE,'(i4,5i2)')idate; JULD_year=idate(1); JULD_mon=idate(2); JULD_day=idate(3);
    prof_JULD = FILLDBLE;
endif
! 1003 format("DBG QC02 END T/D_qc: ",a1," T/D: ",a14," from: ",a)


if(DBLEV== 2)then; write(msg,'("--------------------------------------------------")'); call wrttrc(sub,msg); endif
JULD_year = idate(1);	! GTS date & compare in grey list check
JULD_mon = idate(2);	! GTS date & compare in grey list check
JULD_day = idate(3);	! GTS date & compare in grey list check
JULD_hr  = idate(4);	! GTS date
JULD_mn  = idate(5);	! GTS date
! NOTE on the FIRST_POSITION - this is actually only a TIME - it is used as JULD regardless
! of the goodness/badness of the lat/lon - If the fitst pos line is a "999" line, it is used
! unless the time itself is not valid.
	
! result is already NOTD
! OLD if(prof_JULD_QC == '1')then		! Performed, good
if((prof_JULD_QC == '1') .or. (prof_JULD_QC == '8'))then	! Performed, good
    result = PASS
elseif(prof_JULD_QC == '4')then					! Performed, but bad
    result = FAIL
endif

if(DBLEV== 2)then
    write(msg,*)'LEV02 QC02      JULD: ',prof_JULD; call wrtdbg(sub,msg);		! LEV02
    write(msg,*)'LEV02 QC02   JULD_QC: ',prof_JULD_QC; call wrtdbg(sub,msg);		! LEV02
    write(msg,*)'LEV02 QC02 JULD_FROM: ',juld_from;   call wrtdbg(sub,msg); 		! LEV02
endif

! 11Sep2014 - do one final check of idate from the selected JULD

! One last check
if(prof_JULD_QC  == '1')then
    if(DBLEV== 2)then;
	write(msg,*)'LEV02 prof info: ',prof_JULD,' qc: ',prof_JULD_QC,' date/time: ',prof_DATE_TIME_PROFILE; call wrtdbg(sub,msg);
	write(msg,*)'LEV02 prof info: ',juld_from
    endif
    read(prof_DATE_TIME_PROFILE(1:4),*)iyr;
    read(prof_DATE_TIME_PROFILE(5:6),*)imo;
    read(prof_DATE_TIME_PROFILE(7:8),*)idy;
    read(prof_DATE_TIME_PROFILE(9:10),*)ihh;
    read(prof_DATE_TIME_PROFILE(11:12),*)imn;
    read(prof_DATE_TIME_PROFILE(13:14),*)isc;
    if(DEBUG) write(msg,*)'QC02 calls DATE_CHECK @1 for prof_DATE_TIME_PROFILE: ', &
	iyr,imo,idy,ihh,imn,rio; call wrtdbg(sub,msg);
    call date_check(iyr,imo,idy,ihh,imn,rio)
     prof_JULD = juldays1950(idate);

endif

if(badgpsfix_found .and.(prof_JULD_QC  == '1'))then

    read(prof_DATE_TIME_PROFILE,'(a8)')YYYYMMDD;
    if(badgpsfix_ascnt)then; HHMMSS=TIME_ASCENT_END(9:14);  TIME_ASCENT_END_QC = '5';  TIME_ASCENT_END = YYYYMMDD // HHMMSS; endif
    if(badgpsfix_strns)then; HHMMSS=TIME_TRANS_START(9:14); TIME_TRANS_START_QC = '5'; TIME_TRANS_START = YYYYMMDD // HHMMSS; endif
    if(badgpsfix_MC100)then; HHMMSS=TIME_MC100(9:14);       TIME_MC100_QC = '5'; TIME_MC100 = YYYYMMDD // HHMMSS; endif
    if(badgpsfix_MC600)then; HHMMSS=TIME_MC600(9:14);       TIME_MC600_QC = '5'; TIME_MC600 = YYYYMMDD // HHMMSS; endif
    if(badgpsfix_MC700)then; HHMMSS=TIME_MC700(9:14);       TIME_MC700_QC = '5'; TIME_MC700 = YYYYMMDD // HHMMSS; endif
    if(badgpsfix_MC090)then; HHMMSS=TIME_MC090(9:14);       TIME_MC090_QC = '5'; TIME_MC090 = YYYYMMDD // HHMMSS; endif
    if(badgpsfix_MC099)then; HHMMSS=TIME_MC099(9:14);       TIME_MC099_QC = '5'; TIME_MC099 = YYYYMMDD // HHMMSS; endif
    if(badgpsfix_MC589)then; HHMMSS=TIME_MC589(9:14);       TIME_MC589_QC = '5'; TIME_MC589 = YYYYMMDD // HHMMSS; endif
    if(badgpsfix_MC590)then; HHMMSS=TIME_MC590(9:14);       TIME_MC590_QC = '5'; TIME_MC590 = YYYYMMDD // HHMMSS; endif

endif
mxtflag = 0; if(result == PASS)then; mxtflag = 1; elseif(result == FAIL)then; mxtflag = 4; endif
if(DEBUG)then;
    write(msg,'("QCTEST_FLAG QC02 result: ",i2," mxtflag: ",i2," nprf: ",i2)')result,mxtflag,nprf;	! DBG
    call wrtlog(sub,msg);
endif
do iprf=1,nprf; QCTEST_FLAG(iprf,2) = mxtflag; enddo

if(DBGIO) call dbg_leave(sub,result);

! For the individual/non-full-profile tests, report the result directly to all output logs/files 
write(msg,'("Impossible Date Test        ",a)')report(result); call wrttrc(sub,msg);

! The result to return to the main is based on the JULD only - only fails if NO GOOD DATES exist

qc02_impossible_date = result;

if(DEBUG) write(DBG,*)'DATE_QC qc08 JULD         qc: ',prof_JULD_QC,' ',prof_DATE_TIME_PROFILE,' from: ',trim(juld_from)

2021 format("LEV02 TEST DATE qc02 ",a3," DATE_TIME: ",a," QC: ",a1," JULD: ",f19.12,x,a);

if(DBLEV== 2)then;
    write(DBG,2021)'end',prof_DATE_TIME_PROFILE,prof_JULD_QC,prof_JULD, 'Final value';                                   ! DBG LEV02
    if(prof_JULD==FILLDBLE .and. prof_DATE_TIME_PROFILE/=FILL14CH)then;
	write(errmsg(1),'("NON-FATAL ERROR 999 in qc02, JULD is FILL, DATE is NOT,  QC: ",a)')prof_JULD_QC;
	write(errmsg(2),'("NON-FATAL ERROR 999 in qc02, JULD: ",f19.12)')prof_JULD;;
	write(errmsg(3),'("NON-FATAL ERROR 999 in qc02, DATE: ",a)')PROF_DATE_TIME_PROFILE;
	call error_report(sub,999,3,errmsg);
    elseif(prof_JULD/=FILLDBLE .and. prof_DATE_TIME_PROFILE==FILL14CH)then;
	write(errmsg(1),'("NON-FATAL ERROR 999 in qc02, date  is FILL, JULD is NOT,  QC: ",a)')prof_JULD_QC;
	write(errmsg(2),'("NON-FATAL ERROR 999 in qc02, JULD: ",f19.12)')prof_JULD;;
	write(errmsg(3),'("NON-FATAL ERROR 999 in qc02, DATE: ",a)')PROF_DATE_TIME_PROFILE;
	call error_report(sub,999,3,errmsg);
    endif
endif
return
end function qc02_impossible_date ! primary
! ===========================================================================================================
