! ===========================================================================================================
! character function write_ncbiop.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180307 VH     Make functional a non-functional f90. Activate all calls to create/def/put subroutines to module
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes:
!
! This function creates the nc biop file.
! ===========================================================================================================
! Function returns 'y' for successful file creation, "F" for failure
! ===========================================================================================================

character function write_ncbiop(ncbiop)

use communication                       ! module
use data_arrays                         ! module
use netcdf_arrays			! module
use ncprof_arrays                       ! module
use ncbiop_arrays			! module  

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=128) , intent( in ) :: ncbiop

integer :: ncid;

character(len=12),parameter :: sub = 'write_ncbiop'
character(len=4),parameter :: nctype = 'biop'
character(len=5),parameter :: ncjob = 'WRITE'; character(len=4) :: nctup;  ! What job are we doing, nctupe_uppercase
integer :: ret = 0;
! ===========================================================================================================
! executable statements

! Which file is being written, and wat is the filmsg index?
filmsg(l:l) = 'R'; call wrtflm(sub,'Initialize to Running')

call set_uppercase(nctype,nctup)
gloc = 5000; ! gloc - global location of the nc call
gnctype = nctype; ! For calls to cke

if(DEBUG)then; call dbg_ncbeg(sub,nctup,ncjob); endif


! 'N_PROF',                  N_PROF_len
! 'N_PARAM',                 N_PARAM_len
! 'N_LEVELS',                N_LEVELS_len
! 'N_CALIB',                 N_CALIB_len
! 'N_HISTORY',               N_HISTORY_len_prof            ! UNLIMITED for ncbiop


ret=create_ncfile(sub,nctype,ncbiop,ncid); ! This also defines the dimensins & global vars
write(msg,'("write NCBIOP: ",a)')trim(ncbiop); call wrtstd(sub,msg); ! STD

write(msg,'(a,"output file opened: ",a)')WRTFILE,trim(ncbiop); call wrtlog(sub,msg);

if(DBLEV==66)then; write(DBG,3001)WRTFILE; call close_ncfile(sub,ncid); call direct_exit(sub,0,'LEV666'); endif
3001 format("LEV=66 Exit after nc file creation for ",a);

call def_common_vars(sub,nctype,ncid);

call def_ncbiop_vars(sub,ncid);

call def_ncbiop_params(sub,ncid);

call def_ncbiop_calhis(sub,ncid);

call end_define_mode(sub,ncid);

call put_common_data(sub,nctype,ncid);
 
call put_ncbiop_vars(sub,ncid);

call put_ncbiop_params(sub,ncid);

call put_ncbiop_calhis(sub,ncid);

10 continue; ! Skip to the close

call close_ncfile(sub,ncid);

! See if an error occurred in an array module
if(filmsg(l:l) == 'R')then;
    filmsg(l:l) = 'y'; call wrtflm(sub,'Normal exit, return y for SUCCESS');
elseif((filmsg(l:l) == 'f') .or. (filmsg(l:l) == 'F'))then
    if(DEBUG) write(msg,'(a,a1,x,a8,x,a)')HEDF,'R',filmsg,'ERROR exit, return F or f'; call wrtlog(sub,msg);
else
    write(msg,'(a,a1,x,a8,x,a)')HEDF,filmsg(l:l),'W Non-standard file result'; call wrtlog(sub,msg);
endif
write(msg,'(a," output file closed: ",a)')WRTFILE,trim(ncbiop); call wrtlog(sub,msg);

write_ncbiop = filmsg(l:l)
 
if(DEBUG)then; call dbg_ncret(sub,nctup,ncjob,write_ncbiop); endif
if(DEBUG)then; call dbg_ncend(sub,nctup,ncjob); endif

return          ! Normal return
! ===========================================================================================================
end function write_ncbiop
! ===========================================================================================================
